@@ -36,7 +36,7 @@
 
 #ifdef G_OS_WIN32
 /*
- * Instead of polluting the Dia namespace with indoze headers, declare the
+ * Instead of polluting the Dia namespace with windoze headers, declare the
  * required prototype here. This is bad style, but not as bad as namespace
  * clashes to be resolved without C++   --hb
  */
@@ -56,6 +56,9 @@ ShellExecuteA (long        hwnd,
 #include "paginate_gnomeprint.h"
 #else
 #include "paginate_psprint.h"
+#  ifdef G_OS_WIN32
+#  include "paginate_gdiprint.h"
+#  endif
 #endif
 #include "intl.h"
 #include "commands.h"
@@ -69,6 +72,7 @@ ShellExecuteA (long        hwnd,
 #include "message.h"
 #include "grid.h"
 #include "properties.h"
+#include "propinternals.h"
 #include "preferences.h"
 #include "layer_dialog.h"
 #include "connectionpoint_ops.h"
@@ -81,6 +85,7 @@ ShellExecuteA (long        hwnd,
 #include "gdk/gdkkeysyms.h"
 #include "lib/properties.h"
 #include "dia-props.h"
+#include "diagram_tree_window.h"
 
 GdkPixbuf *logo;
 
@@ -89,59 +94,6 @@ void file_quit_callback(gpointer data, guint action, GtkWidget *widget)
   app_exit();
 }
 
-static void
-file_import_from_xfig_dialog_ok_callback (GtkWidget        *w,
-				     GtkFileSelection *fs)
-{
-  char *filename;
-  Diagram *dia;
-  struct stat stat_struct;
-
-  dia = gtk_object_get_user_data(GTK_OBJECT(fs));
-  
-  filename = gtk_file_selection_get_filename (GTK_FILE_SELECTION (fs));
-
-  if (stat(filename, &stat_struct)!=0) {
-    message_error(_("No such file found\n%s\n"), filename);
-    gtk_grab_remove(GTK_WIDGET(fs));
-    gtk_widget_destroy(GTK_WIDGET(fs));
-    return;
-  }
-  
-  diagram_import_from_xfig(dia, filename);
-  
-  gtk_grab_remove(GTK_WIDGET(fs));
-  gtk_widget_destroy(GTK_WIDGET(fs));
-}
-
-void
-file_import_from_xfig_callback (gpointer data, guint action, GtkWidget *widget)
-{
-  DDisplay *ddisp;
-  GtkWidget *window = NULL;
-
-  ddisp = ddisplay_active();
-
-  window = gtk_file_selection_new (_("Import from XFig"));
-  gtk_window_set_position (GTK_WINDOW (window), GTK_WIN_POS_MOUSE);
-  
-  gtk_object_set_user_data(GTK_OBJECT(window), ddisp->diagram);
-  
-  gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (window)->ok_button),
-		      "clicked", GTK_SIGNAL_FUNC(file_import_from_xfig_dialog_ok_callback),
-		      window);
-  
-  gtk_signal_connect_object (GTK_OBJECT (GTK_FILE_SELECTION (window)->cancel_button),
-			     "clicked", GTK_SIGNAL_FUNC(gtk_widget_destroy),
-			     GTK_OBJECT (window));
-  
-  gtk_widget_show (window);
-
-  /* Make dialog modal: */
-  gtk_widget_grab_focus(window);
-  gtk_grab_add(window);
-}
-
 void
 file_pagesetup_callback(gpointer data, guint action, GtkWidget *widget)
 {
@@ -160,7 +112,13 @@ file_print_callback(gpointer data, guint action, GtkWidget *widget)
 #ifdef GNOME_PRINT
   diagram_print_gnome(dia);
 #else
-  diagram_print_ps(dia);
+#  ifdef G_OS_WIN32
+  /* This option could be used with Gnome too. Does it make sense there ? */
+  if (!prefs.prefer_psprint)
+    diagram_print_gdi(dia);
+  else
+#  endif
+    diagram_print_ps(dia);
 #endif
 }
 
@@ -182,6 +140,7 @@ file_new_callback(gpointer data, guint action, GtkWidget *widget)
   
   dia = new_diagram(buffer);
   ddisp = new_display(dia);
+  diagram_tree_add(diagram_tree(), dia);
 }
 
 void
@@ -201,6 +160,8 @@ edit_copy_callback(gpointer data, guint action, GtkWidget *widget)
   copy_list = diagram_get_sorted_selected(ddisp->diagram);
 
   cnp_store_objects(object_copy_list(copy_list));
+
+  ddisplay_do_update_menu_sensitivity(ddisp);
 }
 
 void
@@ -221,9 +182,10 @@ edit_cut_callback(gpointer data, guint action, GtkWidget *widget)
   change = undo_delete_objects(ddisp->diagram, cut_list);
   (change->apply)(change, ddisp->diagram);
   
-  diagram_update_menu_sensitivity(ddisp->diagram);
+  ddisplay_do_update_menu_sensitivity(ddisp);
   diagram_flush(ddisp->diagram);
 
+  diagram_modified(ddisp->diagram);
   undo_set_transactionpoint(ddisp->diagram->undo);
 }
 
@@ -259,6 +221,7 @@ edit_paste_callback(gpointer data, guint action, GtkWidget *widget)
   change = undo_insert_objects(ddisp->diagram, paste_list, 0);
   (change->apply)(change, ddisp->diagram);
 
+  diagram_modified(ddisp->diagram);
   undo_set_transactionpoint(ddisp->diagram->undo);
   
   diagram_remove_all_selected(ddisp->diagram, TRUE);
@@ -334,13 +297,33 @@ get_selection_handler(GtkWidget *widget, GtkSelectionData *selection,
   }
 }
 
+
+static PropDescription text_prop_singleton_desc[] = {
+    { "text", PROP_TYPE_STRING },
+    PROP_DESC_END};
+
+
+static void 
+make_text_prop_singleton(GPtrArray **props, StringProperty **prop) 
+{
+  *props = prop_list_from_descs(text_prop_singleton_desc,pdtpp_true);
+  g_assert((*props)->len == 1);
+
+  *prop = g_ptr_array_index((*props),0);
+  g_free((*prop)->string_data);
+  (*prop)->string_data = NULL;
+  (*prop)->num_lines = 0;
+}
+
+
 void
 edit_copy_text_callback(gpointer data, guint action, GtkWidget *widget)
 {
   Focus *focus = active_focus();
   DDisplay *ddisp;
   Object *obj;
-  Property textprop;
+  GPtrArray *textprops;
+  StringProperty *prop;
 
   if ((focus == NULL) || (!focus->has_focus)) return;
 
@@ -351,21 +334,15 @@ edit_copy_text_callback(gpointer data, guint action, GtkWidget *widget)
   if (obj->ops->get_props == NULL) 
     return;
 
-  textprop.name = "text";
-  textprop.type = PROP_TYPE_INVALID;
-  PROP_VALUE_STRING(textprop) = NULL;
-
+  make_text_prop_singleton(&textprops,&prop);
   /* Get the first text property */
-  obj->ops->get_props(obj, &textprop, 1);
+  obj->ops->get_props(obj, textprops);
   
   if (current_clipboard) g_free(current_clipboard);
 
-  if (textprop.type != PROP_TYPE_STRING)
-    return;
+  current_clipboard = g_strdup(prop->string_data);
 
-  current_clipboard = g_strdup(PROP_VALUE_STRING(textprop));
-
-  prop_free(&textprop);
+  prop_list_free(textprops);
 
   gtk_selection_owner_set(GTK_WIDGET(ddisp->shell),
 			  GDK_SELECTION_PRIMARY,
@@ -382,7 +359,8 @@ edit_cut_text_callback(gpointer data, guint action, GtkWidget *widget)
   DDisplay *ddisp;
   Object *obj;
   Text *text;
-  Property textprop;
+  GPtrArray *textprops;
+  StringProperty *prop;
   Change *change;
 
   if ((focus == NULL) || (!focus->has_focus)) return;
@@ -395,21 +373,15 @@ edit_cut_text_callback(gpointer data, guint action, GtkWidget *widget)
   if (obj->ops->get_props == NULL) 
     return;
 
-  textprop.name = "text";
-  textprop.type = PROP_TYPE_INVALID;
-  PROP_VALUE_STRING(textprop) = NULL;
-
+  make_text_prop_singleton(&textprops,&prop);
   /* Get the first text property */
-  obj->ops->get_props(obj, &textprop, 1);
+  obj->ops->get_props(obj, textprops);
   
   if (current_clipboard) g_free(current_clipboard);
 
-  if (textprop.type != PROP_TYPE_STRING)
-    return;
+  current_clipboard = g_strdup(prop->string_data);
 
-  current_clipboard = g_strdup(PROP_VALUE_STRING(textprop));
-
-  prop_free(&textprop);
+  prop_list_free(textprops);
 
   gtk_selection_owner_set(GTK_WIDGET(ddisp->shell),
 			  GDK_SELECTION_PRIMARY,
@@ -421,7 +393,7 @@ edit_cut_text_callback(gpointer data, guint action, GtkWidget *widget)
   if (text_delete_all(text, (ObjectChange **)&change)) { 
     (change->apply)(change, ddisp->diagram);
     
-    diagram_update_menu_sensitivity(ddisp->diagram);
+    diagram_update_menu_sensitivity(ddisp->diagram, &ddisp->updatable_menu_items);
     diagram_flush(ddisp->diagram);
   }
 }
@@ -454,7 +426,9 @@ edit_delete_callback(gpointer data, guint action, GtkWidget *widget)
   change = undo_delete_objects(ddisp->diagram, delete_list);
   (change->apply)(change, ddisp->diagram);
   
-  diagram_update_menu_sensitivity(ddisp->diagram);
+  diagram_modified(ddisp->diagram);
+
+  ddisplay_do_update_menu_sensitivity(ddisp);
   diagram_flush(ddisp->diagram);
 
   undo_set_transactionpoint(ddisp->diagram->undo);
@@ -504,6 +478,12 @@ help_manual_callback(gpointer data, guint action, GtkWidget *widget)
 
   /* search through helpdir for the helpfile that matches the user's locale */
   dp = opendir(helpdir);
+  if (dp == NULL) {
+    g_free(helpdir);
+    message_warning(_("Could not find help directory"));
+    return;
+  }
+
   while ((dirp = readdir(dp)) != NULL) {
     guint score;
 
@@ -519,6 +499,7 @@ help_manual_callback(gpointer data, guint action, GtkWidget *widget)
       bestscore = score;
     }
   }
+  closedir(dp);
   g_free(helpdir);
   if (!helpindex) {
     message_warning(_("Could not find help directory"));
@@ -560,6 +541,7 @@ help_about_callback(gpointer data, guint action, GtkWidget *widget)
     "Fredrik Hallenberg",
     "Francis J. Lacoste",
     "Steffen Macke",
+    "M. C. Nelson",
     "Jacek Pliszka",
     "Henk Jan Priester",
     "Alejandro Aguilar Sierra",
