@@ -21,6 +21,7 @@
 #endif
 
 #include <stdio.h>
+#include <string.h>
 #include <math.h>
 #include <gtk/gtk.h>
 #include <gdk/gdkkeysyms.h>
@@ -39,6 +40,7 @@
 #include "message.h"
 #include "intl.h"
 #include "magnify.h"
+#include "charconv.h"
 
 /* This contains the point that was clicked to get this menu */
 static Point object_menu_clicked_point;
@@ -80,6 +82,14 @@ object_menu_proxy(GtkWidget *widget, gpointer data)
   diagram_flush(ddisp->diagram);
 }
 
+static void
+dia_menu_free(DiaMenu *dia_menu) {
+  if (dia_menu->app_data)
+    gtk_object_destroy((GtkObject *)dia_menu->app_data);
+  dia_menu->app_data = NULL;
+  dia_menu->app_data_free = NULL;
+}
+
 static void
 create_object_menu(DiaMenu *dia_menu)
 {
@@ -91,10 +101,10 @@ create_object_menu(DiaMenu *dia_menu)
   gtk_menu_ensure_uline_accel_group (GTK_MENU (menu)) ;
 
   if ( dia_menu->title ) {
-  menu_item = gtk_menu_item_new_with_label(gettext(dia_menu->title));
-  gtk_widget_set_sensitive(menu_item, FALSE);
-  gtk_menu_append(GTK_MENU(menu), menu_item);
-  gtk_widget_show(menu_item);
+    menu_item = gtk_menu_item_new_with_label(gettext(dia_menu->title));
+    gtk_widget_set_sensitive(menu_item, FALSE);
+    gtk_menu_append(GTK_MENU(menu), menu_item);
+    gtk_widget_show(menu_item);
   }
 
   menu_item = gtk_menu_item_new();
@@ -124,12 +134,13 @@ create_object_menu(DiaMenu *dia_menu)
 	  /* Create the popup menu items for the submenu. */
           create_object_menu( (DiaMenu*)(dia_menu->items[i].callback_data) ) ;
           gtk_menu_item_set_submenu( GTK_MENU_ITEM (menu_item), 
-	    GTK_MENU(((DiaMenu*)(dia_menu->items[i].callback_data))->app_data));
+      GTK_WIDGET(((DiaMenu*)(dia_menu->items[i].callback_data))->app_data));
 	}
       }
     }
   }
   dia_menu->app_data = menu;
+  dia_menu->app_data_free = dia_menu_free;
 }
 
 static void
@@ -210,7 +221,7 @@ ddisplay_focus_in_event(GtkWidget *widget, GdkEventFocus *event, gpointer data)
   gtk_widget_draw_focus(widget);
 
 #ifdef USE_XIM
-  if (ddisp->ic)
+  if (gdk_im_ready () && ddisp->ic)
     gdk_im_begin(ddisp->ic, widget->window);
 #endif
 
@@ -242,74 +253,79 @@ ddisplay_focus_out_event(GtkWidget *widget, GdkEventFocus *event,gpointer data)
 }
 
 void
-ddisplay_realize(GtkWidget *widget, gpointer data)
+ddisplay_realize (GtkWidget *widget, gpointer data)
 {
-  DDisplay *ddisp;
+	DDisplay *ddisp;
 
-  g_return_if_fail(widget != NULL);
-  g_return_if_fail(data != NULL);
+	g_return_if_fail(widget != NULL);
+	g_return_if_fail(data != NULL);
 
-  ddisp = (DDisplay *)data;
+	ddisp = (DDisplay *)data;
 
 #ifdef USE_XIM
-  if (gdk_im_ready() && (ddisp->ic_attr = gdk_ic_attr_new()) != NULL) {
-    gint width, height;
-    GdkColormap *colormap;
-    GdkICAttr *attr = ddisp->ic_attr;
-    GdkICAttributesType attrmask = GDK_IC_ALL_REQ;
-    GdkIMStyle style;
-    GdkIMStyle supported_style =
-      GDK_IM_PREEDIT_NONE |
-      GDK_IM_PREEDIT_NOTHING |
-      GDK_IM_PREEDIT_POSITION |
-      GDK_IM_STATUS_NONE |
-      GDK_IM_STATUS_NOTHING;
+	if (gdk_im_ready () && (ddisp->ic_attr = gdk_ic_attr_new ()) != NULL) {
+		gint width, height;
+		GdkColormap *colormap;
+		GdkEventMask mask;
+		GdkICAttr *attr = ddisp->ic_attr;
+		GdkICAttributesType attrmask = GDK_IC_ALL_REQ;
+		GdkIMStyle style;
+		GdkIMStyle supported_style =
+			GDK_IM_PREEDIT_NONE |
+			GDK_IM_PREEDIT_NOTHING |
+			GDK_IM_PREEDIT_POSITION |
+			GDK_IM_STATUS_NONE |
+			GDK_IM_STATUS_NOTHING;
 
-    if (widget->style && widget->style->font->type != GDK_FONT_FONTSET)
-      supported_style &= ~GDK_IM_PREEDIT_POSITION;
+		if (widget->style && widget->style->font->type != GDK_FONT_FONTSET)
+			supported_style &= ~GDK_IM_PREEDIT_POSITION;
 
-    attr->style = style = gdk_im_decide_style(supported_style);
-    attr->client_window = widget->window;
+		attr->style = style = gdk_im_decide_style (supported_style);
+		attr->client_window = widget->window;
 
-    if ((colormap = gtk_widget_get_colormap(widget)) !=
-	gtk_widget_get_default_colormap()) {
-      attrmask |= GDK_IC_PREEDIT_COLORMAP;
-      attr->preedit_colormap = colormap;
-    }
+		if ((colormap = gtk_widget_get_colormap (widget)) !=
+		    gtk_widget_get_default_colormap ()) {
+			attrmask |= GDK_IC_PREEDIT_COLORMAP;
+			attr->preedit_colormap = colormap;
+		}
 
-    attrmask |= GDK_IC_PREEDIT_FOREGROUND;
-    attrmask |= GDK_IC_PREEDIT_BACKGROUND;
-    attr->preedit_foreground = widget->style->fg[GTK_STATE_NORMAL];
-    attr->preedit_background = widget->style->base[GTK_STATE_NORMAL];
+		attrmask |= GDK_IC_PREEDIT_FOREGROUND;
+		attrmask |= GDK_IC_PREEDIT_BACKGROUND;
+		attr->preedit_foreground = widget->style->fg[GTK_STATE_NORMAL];
+		attr->preedit_background = widget->style->base[GTK_STATE_NORMAL];
 
 
-    switch (style & GDK_IM_PREEDIT_MASK) {
-    case GDK_IM_PREEDIT_POSITION:
-      if (ddisp->canvas->style &&
-	  ddisp->canvas->style->font->type != GDK_FONT_FONTSET) {
-	g_warning("over-the-spot style requires fontset");
-	break;
-      }
-      attrmask |= GDK_IC_PREEDIT_POSITION_REQ;
-      gdk_window_get_size(widget->window, &width, &height);
-      attr->spot_location.x = 0;
-      attr->spot_location.y = 14;
-      attr->preedit_area.x = 0;
-      attr->preedit_area.y = 0;
-      attr->preedit_area.width = width;
-      attr->preedit_area.height = height;
-      attr->preedit_fontset = widget->style->font;
-    }
-    ddisp->ic = gdk_ic_new(attr, attrmask);
-    if (ddisp->ic == NULL)
-      g_warning("could not create input context");
-    else {
-      gtk_widget_add_events(ddisp->canvas, gdk_ic_get_events(ddisp->ic));
+		switch (style & GDK_IM_PREEDIT_MASK) {
+		    case GDK_IM_PREEDIT_POSITION:
+			    if (widget->style &&
+				widget->style->font->type != GDK_FONT_FONTSET) {
+				    g_warning("over-the-spot style requires fontset");
+				    break;
+			    }
+			    gdk_window_get_size (widget->window, &width, &height);
 
-      if (GTK_WIDGET_HAS_FOCUS(widget))
-	gdk_im_begin(ddisp->ic, widget->window);
-    }
-  }
+			    attrmask |= GDK_IC_PREEDIT_POSITION_REQ;
+			    attr->spot_location.x = 0;
+			    attr->spot_location.y = 14;
+			    attr->preedit_area.x = 0;
+			    attr->preedit_area.y = 0;
+			    attr->preedit_area.width = width;
+			    attr->preedit_area.height = height;
+			    attr->preedit_fontset = widget->style->font;
+			    break;
+		}
+		ddisp->ic = gdk_ic_new (attr, attrmask);
+		if (ddisp->ic == NULL)
+			g_warning("could not create input context");
+		else {
+			mask = gdk_window_get_events (widget->window);
+			mask |= gdk_ic_get_events (ddisp->ic);
+			gdk_window_set_events (widget->window, mask);
+
+			if (GTK_WIDGET_HAS_FOCUS (widget))
+				gdk_im_begin (ddisp->ic, widget->window);
+		}
+	}
 #endif
 }
 
@@ -324,27 +340,46 @@ ddisplay_unrealize (GtkWidget *widget, gpointer data)
   ddisp = (DDisplay *) data;
 
 #ifdef USE_XIM
-  if (ddisp->ic)
-    gdk_ic_destroy (ddisp->ic);
-  ddisp->ic = NULL;
-  if (ddisp->ic_attr)
-    gdk_ic_attr_destroy (ddisp->ic_attr);
-  ddisp->ic_attr = NULL;
+  if (gdk_im_ready ()) {
+    if (ddisp->ic)
+      gdk_ic_destroy (ddisp->ic);
+    ddisp->ic = NULL;
+    if (ddisp->ic_attr)
+      gdk_ic_attr_destroy (ddisp->ic_attr);
+    ddisp->ic_attr = NULL;
+  }
 #endif
  
 }
 
-#ifdef USE_XIM
-static void
-set_input_dialog(DDisplay *ddisp, int x, int y)
+void
+ddisplay_size_allocate (GtkWidget *widget,
+			GtkAllocation *allocation,
+			gpointer data)
 {
-  if (ddisp->ic && (gdk_ic_get_style(ddisp->ic) & GDK_IM_PREEDIT_POSITION)) {
-    ddisp->ic_attr->spot_location.x = x;
-    ddisp->ic_attr->spot_location.y = y; 
-    gdk_ic_set_attr(ddisp->ic, ddisp->ic_attr, GDK_IC_SPOT_LOCATION);
-  }
-}
+	DDisplay *ddisp;
+
+	g_return_if_fail (widget != NULL);
+	g_return_if_fail (allocation != NULL);
+	g_return_if_fail (data != NULL);
+
+	widget->allocation = *allocation;
+	ddisp = (DDisplay *)data;
+#ifdef USE_XIM
+	if (GTK_WIDGET_REALIZED (widget)) {
+		if (gdk_im_ready () && ddisp->ic &&
+		    (gdk_ic_get_style (ddisp->ic) & GDK_IM_PREEDIT_POSITION)) {
+			gint width, height;
+
+			gdk_window_get_size (widget, &width, &height);
+			ddisp->ic_attr->preedit_area.width = width;
+			ddisp->ic_attr->preedit_area.height = height;
+			gdk_ic_set_attr (ddisp->ic, ddisp->ic_attr,
+					 GDK_IC_PREEDIT_AREA);
+		}
+	}
 #endif
+}
 
 void
 ddisplay_popup_menu(DDisplay *ddisp, GdkEventButton *event)
@@ -377,7 +412,7 @@ ddisplay_canvas_events (GtkWidget *canvas,
   int key_handled;
   int width, height;
   int new_size;
-  int modified;
+  int modified = FALSE;
   int x, y;
 
   return_val = FALSE;
@@ -418,7 +453,8 @@ ddisplay_canvas_events (GtkWidget *canvas,
       break;
 
     case GDK_FOCUS_CHANGE:
-      diagram_update_menu_sensitivity(ddisp->diagram);
+      display_set_active (ddisp);
+      ddisplay_do_update_menu_sensitivity(ddisp);
       break;
       
     case GDK_2BUTTON_PRESS:
@@ -464,10 +500,13 @@ ddisplay_canvas_events (GtkWidget *canvas,
 	  break;
 
 	case 2:
+	  if (ddisp->menu_bar == NULL) {
 	  popup_object_menu(ddisp, bevent);
+	  }
 	  break;
 
 	case 3:
+	  if (ddisp->menu_bar == NULL) {
 	  if (bevent->state & GDK_CONTROL_MASK) {
 	    /* for two button mouse users ... */
 	    popup_object_menu(ddisp, bevent);
@@ -475,8 +514,28 @@ ddisplay_canvas_events (GtkWidget *canvas,
 	  }
 	  ddisplay_popup_menu(ddisp, bevent);
  	  break;
-
-	default:
+	  }
+	  else {
+	     popup_object_menu(ddisp, bevent);
+	     break;
+	  }
+        case 4: /* for wheel mouse; button 4 and 5 */
+          ddisplay_scroll_up(ddisp);
+          ddisplay_flush(ddisp);
+          break;
+        case 5:
+          ddisplay_scroll_down(ddisp);
+          ddisplay_flush(ddisp);
+          break;
+        case 6: /* for two-wheel mouse; button 6 and 7 */
+          ddisplay_scroll_left(ddisp);
+          ddisplay_flush(ddisp);
+          break;
+        case 7:
+          ddisplay_scroll_right(ddisp);
+          ddisplay_flush(ddisp);
+          break;
+        default:
 	  break;
 	}
       break;
@@ -531,19 +590,24 @@ ddisplay_canvas_events (GtkWidget *canvas,
 	obj = focus->obj;
 	if (diagram_is_selected(ddisp->diagram, obj)) {
 	  ObjectChange *obj_change = NULL;
+	  utfchar *utf;
+	  int len = 0;
 	
 	  object_add_updates(obj, ddisp->diagram);
-#ifdef USE_XIM
-	  ddisplay_transform_coords(ddisp, obj->position.x, obj->position.y,
-				    &x, &y);
-	  set_input_dialog(ddisp, x, y);
+#if defined (UNICODE_WORK_IN_PROGRESS) && !defined(GTK_TALKS_UTF8)
+	  utf = charconv_utf8_from_gtk_event_key (kevent->keyval, kevent->string);
+#else
+	  utf = g_strndup (kevent->string, kevent->length);
 #endif
+	  if (utf != NULL) len = uni_strlen (utf, strlen (utf));
 	  modified = (focus->key_event)(focus, kevent->keyval,
-					kevent->string, kevent->length,
-					&obj_change);
-	  { /* Make sure object updates its data: */
+					utf, len, &obj_change);
+	  if (utf != NULL) g_free (utf);
+	  { /* Make sure object updates its data and its connected: */
 	    Point p = obj->position;
-	    (obj->ops->move)(obj,&p);  }
+	    (obj->ops->move)(obj,&p);  
+            diagram_update_connections_object(ddisp->diagram,obj,TRUE);
+          }
 	  
 	  object_add_updates(obj, ddisp->diagram);
 
@@ -558,7 +622,7 @@ ddisplay_canvas_events (GtkWidget *canvas,
 	}
       }
       if (!key_handled) {
-	/* No focus to recive keys, take care of it ourselves. */
+	/* No focus to receive keys, take care of it ourselves. */
 	return_val = TRUE;
 	switch(kevent->keyval) {
 	case GDK_Up:
@@ -599,7 +663,11 @@ ddisplay_canvas_events (GtkWidget *canvas,
 	    set_zoom_out(active_tool);
 	  break;
 	default:
-	  return_val = FALSE;
+          if (kevent->string && 0 == strcmp(" ",kevent->string)) {
+            tool_select_former();
+          } else { 
+            return_val = FALSE;
+          }
 	}
       }
       break;
