@@ -43,6 +43,7 @@
 #include "app_procs.h"
 #include "layer_dialog.h"
 #include "load_save.h"
+#include "dia-props.h"
 
 #ifdef THIS_IS_PROBABLY_DEAD_CODE
 #include "pixmaps/snap-to-grid.xpm"
@@ -175,9 +176,23 @@ new_display(Diagram *dia)
   DDisplay *ddisp;
   char *filename;
   int embedded = app_is_embedded();
-
+  Rectangle visible;
+  
   ddisp = g_new0(DDisplay,1);
 
+  ddisp->menu_bar = NULL;
+  ddisp->mbar_item_factory = NULL;
+
+  ddisp->rulers = NULL;
+  ddisp->visible_grid = NULL;
+  ddisp->snap_to_grid = NULL;
+  ddisp->show_cx_pts_mitem = NULL;
+#ifdef HAVE_LIBART
+  ddisp->antialiased = NULL;
+#endif
+  /* initialize the whole struct to 0 so that we are sure to catch errors.*/
+  memset (&ddisp->updatable_menu_items, 0, sizeof (UpdatableMenuItems));
+  
   ddisp->diagram = dia;
 
   ddisp->grid.visible = prefs.grid.visible;
@@ -206,19 +221,30 @@ new_display(Diagram *dia)
   ddisp->origo.x = 0.0;
   ddisp->origo.y = 0.0;
   ddisp->zoom_factor = prefs.new_view.zoom/100.0*DDISPLAY_NORMAL_ZOOM;
-  ddisp->visible.left = 0.0;
-  ddisp->visible.top = 0.0;
-  ddisp->visible.right = prefs.new_view.width/ddisp->zoom_factor;
-  ddisp->visible.bottom = prefs.new_view.height/ddisp->zoom_factor;
+  if ((ddisp->diagram) && (ddisp->diagram->data)) {
+    Rectangle *extents = &ddisp->diagram->data->extents;
+
+    visible.left = extents->left;
+    visible.top = extents->top;
+  } else {
+    visible.left = 0.0;
+    visible.top = 0.0;
+  }
+  visible.right = visible.left + prefs.new_view.width/ddisp->zoom_factor;
+  visible.bottom = visible.top + prefs.new_view.height/ddisp->zoom_factor;
+
+  ddisp->visible = visible;
 
   create_display_shell(ddisp, prefs.new_view.width, prefs.new_view.height,
-		       filename, !embedded);
-
+		       filename, prefs.new_view.use_menu_bar, !embedded);
   
-  /*  ddisplay_set_origo(ddisp, 0.0, 0.0); */
 
   ddisplay_update_statusbar (ddisp);
+
+  ddisplay_set_origo(ddisp, visible.left, visible.top);
+  ddisp->visible = visible; /* force the visible area extents */
   ddisplay_update_scrollbars(ddisp);
+  ddisplay_add_update_all(ddisp);
 
   if (!display_ht)
     display_ht = g_hash_table_new ((GHashFunc) display_hash, NULL);
@@ -229,6 +255,7 @@ new_display(Diagram *dia)
   g_hash_table_insert (display_ht, ddisp->shell, ddisp);
   g_hash_table_insert (display_ht, ddisp->canvas, ddisp);
 
+
   return ddisp;  /*  set the user data  */
 }
 
@@ -539,15 +566,11 @@ ddisplay_update_scrollbars(DDisplay *ddisp)
   Rectangle *extents = &ddisp->diagram->data->extents;
   Rectangle *visible = &ddisp->visible;
   GtkAdjustment *hsbdata, *vsbdata;
-  real extra_border_x, extra_border_y;
 
-  extra_border_x = (visible->right - visible->left);
-  extra_border_y = (visible->bottom - visible->top);
-  
   hsbdata = ddisp->hsbdata;
   /* Horizontal: */
-  hsbdata->lower = MIN(extents->left, visible->left)  - extra_border_x;
-  hsbdata->upper = MAX(extents->right, visible->right)  + extra_border_x;
+  hsbdata->lower = MIN(extents->left, visible->left);
+  hsbdata->upper = MAX(extents->right, visible->right);
   hsbdata->page_size = visible->right - visible->left - 0.0001;
   /* remove some to fix strange behaviour in gtk_range_adjustment_changed */
   hsbdata->page_increment = (visible->right - visible->left) / 2.0;
@@ -558,8 +581,8 @@ ddisplay_update_scrollbars(DDisplay *ddisp)
 
   /* Vertical: */
   vsbdata = ddisp->vsbdata;
-  vsbdata->lower = MIN(extents->top, visible->top)  - extra_border_y;
-  vsbdata->upper = MAX(extents->bottom, visible->bottom) + extra_border_y;
+  vsbdata->lower = MIN(extents->top, visible->top);
+  vsbdata->upper = MAX(extents->bottom, visible->bottom);
   vsbdata->page_size = visible->bottom - visible->top - 0.00001;
   /* remove some to fix strange behaviour in gtk_range_adjustment_changed */
   vsbdata->page_increment = (visible->bottom - visible->top) / 2.0;
@@ -667,54 +690,57 @@ ddisplay_autoscroll(DDisplay *ddisp, int x, int y)
 
   if ((scroll.x != 0) || (scroll.y != 0))
   {
+    gboolean scrolled;
+
     scroll.x = ddisplay_untransform_length(ddisp, scroll.x);
     scroll.y = ddisplay_untransform_length(ddisp, scroll.y);
 
-    ddisplay_scroll(ddisp, &scroll);
-    ddisplay_flush(ddisp);
+    scrolled = ddisplay_scroll(ddisp, &scroll);
 
-    return TRUE;
-  }
-  else
-  {
-    return FALSE;
+    if (scrolled) {
+      ddisplay_flush(ddisp);        
+      return TRUE;
+    }
   }
+  return FALSE;
 }
 
-void ddisplay_scroll(DDisplay *ddisp, Point *delta)
+gboolean 
+ddisplay_scroll(DDisplay *ddisp, Point *delta)
 {
-  Point new_origo;
-  Rectangle extents;
   Rectangle *visible = &ddisp->visible;
-  real width, height;
+  real width = visible->right - visible->left;
+  real height = visible->bottom - visible->top;
 
-  new_origo = ddisp->origo;
+  Rectangle extents = ddisp->diagram->data->extents;
+  real ex_width = extents.right - extents.left;
+  real ex_height = extents.bottom - extents.top;
+
+  Point new_origo = ddisp->origo;
   point_add(&new_origo, delta);
 
-  width = visible->right - visible->left;
-  height = visible->bottom - visible->top;
-  
-  extents = ddisp->diagram->data->extents;
   rectangle_union(&extents, visible);
-  
-  if (new_origo.x < extents.left)
-    new_origo.x = extents.left;
 
-  if (new_origo.x+width > extents.right)
-    new_origo.x = extents.right - width;
+  if (new_origo.x < extents.left - ex_width)
+    new_origo.x = extents.left - ex_width;
 
-  if (new_origo.y < extents.top)
-    new_origo.y = extents.top;
+  if (new_origo.x+width > extents.right + ex_width)
+    new_origo.x = extents.right - width + ex_width;
+
+  if (new_origo.y < extents.top - ex_height)
+    new_origo.y = extents.top - ex_height;
   
-  if (new_origo.y+height > extents.bottom)
-    new_origo.y = extents.bottom - height;
+  if (new_origo.y+height > extents.bottom + ex_height)
+    new_origo.y = extents.bottom - height + ex_height;
 
   if ( (new_origo.x != ddisp->origo.x) ||
        (new_origo.y != ddisp->origo.y) ) {
     ddisplay_set_origo(ddisp, new_origo.x, new_origo.y);
     ddisplay_update_scrollbars(ddisp);
     ddisplay_add_update_all(ddisp);
+    return TRUE;
   }
+  return FALSE;
 }
 
 void ddisplay_scroll_up(DDisplay *ddisp)
@@ -958,7 +984,6 @@ ddisplay_close(DDisplay *ddisp)
 void
 display_update_menu_state(DDisplay *ddisp)
 {
-  Diagram *dia;
   static gboolean initialized = 0;
 
   static GtkWidget *rulers;
@@ -969,37 +994,66 @@ display_update_menu_state(DDisplay *ddisp)
   static GtkWidget *antialiased;
 #endif
 
-  if (!initialized) {
-    rulers       = menus_get_item_from_path("<Display>/View/Show Rulers");
-    visible_grid = menus_get_item_from_path("<Display>/View/Visible Grid");
-    snap_to_grid = menus_get_item_from_path("<Display>/View/Snap To Grid");
+  if ((!initialized) && (ddisp->menu_bar == NULL)) {
+    rulers       = menus_get_item_from_path("<Display>/View/Show Rulers", NULL);
+    visible_grid = menus_get_item_from_path("<Display>/View/Visible Grid", NULL);
+    snap_to_grid = menus_get_item_from_path("<Display>/View/Snap To Grid", NULL);
     show_cx_pts  = 
-      menus_get_item_from_path("<Display>/View/Show Connection Points");
+      menus_get_item_from_path("<Display>/View/Show Connection Points", NULL);
 #ifdef HAVE_LIBART
-    antialiased = menus_get_item_from_path("<Display>/View/AntiAliased");
+    antialiased = menus_get_item_from_path("<Display>/View/AntiAliased", NULL);
 #endif
 
     initialized = TRUE;
   }
+  ddisplay_do_update_menu_sensitivity (ddisp);
   
-  dia = ddisp->diagram;
-
-  diagram_update_menu_sensitivity(dia);
-
-  gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(rulers),
-				 GTK_WIDGET_VISIBLE (ddisp->hrule) ? 1 : 0); 
-  gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(visible_grid),
-				 ddisp->grid.visible);
-  gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(snap_to_grid),
-				 ddisp->grid.snap);
-  gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(show_cx_pts),
-				 ddisp->show_cx_pts); 
+  if (ddisp->menu_bar) {
+      gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(ddisp->rulers),
+				     GTK_WIDGET_VISIBLE (ddisp->hrule) ? 1 : 0); 
+      gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(ddisp->visible_grid),
+				     ddisp->grid.visible);
+      gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(ddisp->snap_to_grid),
+				     ddisp->grid.snap);
+      gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(ddisp->show_cx_pts_mitem),
+				     ddisp->show_cx_pts); 
 #ifdef HAVE_LIBART
-  gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(antialiased),
-				 ddisp->aa_renderer);
+      gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(ddisp->antialiased),
+				     ddisp->aa_renderer);
+#endif
+  }
+  else {
+      gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(rulers),
+				     GTK_WIDGET_VISIBLE (ddisp->hrule) ? 1 : 0); 
+      gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(visible_grid),
+				     ddisp->grid.visible);
+      gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(snap_to_grid),
+				     ddisp->grid.snap);
+      gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(show_cx_pts),
+				     ddisp->show_cx_pts); 
+#ifdef HAVE_LIBART
+      gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(antialiased),
+				     ddisp->aa_renderer);
 #endif 
+  }  
 }
 
+void 
+ddisplay_do_update_menu_sensitivity (DDisplay *ddisp)
+{
+    Diagram *dia;
+    
+    dia = ddisp->diagram; 
+    if (ddisp->menu_bar) {
+	diagram_update_menubar_sensitivity(dia, &ddisp->updatable_menu_items);
+    }
+    else {
+	diagram_update_popupmenu_sensitivity(dia);
+    }
+}
+
+
+
 /* This is called when ddisp->shell is destroyed... */
 void
 ddisplay_really_destroy(DDisplay *ddisp)
@@ -1099,6 +1153,8 @@ display_set_active(DDisplay *ddisp)
 
     /* perform notification here (such as switch layers dialog) */
     layer_dialog_set_diagram(ddisp ? ddisp->diagram : NULL);
+    diagram_properties_set_diagram(ddisp ? ddisp->diagram : NULL);
+
     if (ddisp)
       display_update_menu_state(ddisp);
   }
