@@ -21,6 +21,7 @@
 #endif
 
 #include <gtk/gtk.h>
+#include <gdk/gdkkeysyms.h>
 #include <string.h>
 
 #ifdef GNOME
@@ -37,16 +38,29 @@
 #include "plugin-manager.h"
 #include "select.h"
 #include "dia_dirs.h"
+#include "diagram_tree_window.h"
+#include "object_ops.h"
+#include "sheets.h"
 
 static void plugin_callback (GtkWidget *widget, gpointer data);
 
 #ifdef GNOME
+
 static GnomeUIInfo toolbox_filemenu[] = {
   GNOMEUIINFO_MENU_NEW_ITEM(N_("_New diagram"), N_("Create new diagram"),
 			    file_new_callback, NULL),
   GNOMEUIINFO_MENU_OPEN_ITEM(file_open_callback, NULL),
+  GNOMEUIINFO_SEPARATOR,
+  GNOMEUIINFO_TOGGLEITEM(N_("_Diagram tree"), N_("Show diagram tree"),
+			 diagtree_show_callback, NULL),
+  GNOMEUIINFO_ITEM_NONE(N_("_Sheets and Objects..."),
+                        N_("Modify sheets and their objects"),
+                        sheets_dialog_show_callback),
+  GNOMEUIINFO_SEPARATOR,
   GNOMEUIINFO_MENU_PREFERENCES_ITEM(file_preferences_callback, NULL),
-  GNOMEUIINFO_ITEM_NONE(N_("Plug-ins"), NULL, file_plugins_callback),
+  GNOMEUIINFO_ITEM_NONE(N_("P_lugins"), NULL, file_plugins_callback),
+  GNOMEUIINFO_SEPARATOR,
+    /* recent file list is dynamically inserted here */
   GNOMEUIINFO_SEPARATOR,
   GNOMEUIINFO_MENU_EXIT_ITEM(file_quit_callback, NULL),
   GNOMEUIINFO_END  
@@ -161,20 +175,20 @@ static GnomeUIInfo selectmenu[] = {
 };
 
 static GnomeUIInfo objects_align_h[] = {
-  GNOMEUIINFO_ITEM_NONE_DATA(N_("Left"), NULL, objects_align_h_callback, 0),
-  GNOMEUIINFO_ITEM_NONE_DATA(N_("Center"), NULL, objects_align_h_callback, 1),
-  GNOMEUIINFO_ITEM_NONE_DATA(N_("Right"), NULL, objects_align_h_callback, 2),
-  GNOMEUIINFO_ITEM_NONE_DATA(N_("Equal Distance"), NULL, objects_align_h_callback, 4),
-  GNOMEUIINFO_ITEM_NONE_DATA(N_("Adjacent"), NULL, objects_align_h_callback,5),
+  GNOMEUIINFO_ITEM_NONE_DATA(N_("Left"), NULL, objects_align_h_callback, DIA_ALIGN_LEFT),
+  GNOMEUIINFO_ITEM_NONE_DATA(N_("Center"), NULL, objects_align_h_callback, DIA_ALIGN_CENTER),
+  GNOMEUIINFO_ITEM_NONE_DATA(N_("Right"), NULL, objects_align_h_callback, DIA_ALIGN_RIGHT),
+  GNOMEUIINFO_ITEM_NONE_DATA(N_("Equal Distance"), NULL, objects_align_h_callback, DIA_ALIGN_EQUAL),
+  GNOMEUIINFO_ITEM_NONE_DATA(N_("Adjacent"), NULL, objects_align_h_callback,DIA_ALIGN_ADJACENT),
   GNOMEUIINFO_END
 };
 
 static GnomeUIInfo objects_align_v[] = {
-  GNOMEUIINFO_ITEM_NONE_DATA(N_("Top"), NULL, objects_align_v_callback, 0),
-  GNOMEUIINFO_ITEM_NONE_DATA(N_("Center"), NULL, objects_align_v_callback, 1),
-  GNOMEUIINFO_ITEM_NONE_DATA(N_("Bottom"), NULL, objects_align_v_callback, 2),
-  GNOMEUIINFO_ITEM_NONE_DATA(N_("Equal Distance"), NULL, objects_align_v_callback, 4),
-  GNOMEUIINFO_ITEM_NONE_DATA(N_("Adjacent"), NULL, objects_align_h_callback,5),
+  GNOMEUIINFO_ITEM_NONE_DATA(N_("Top"), NULL, objects_align_v_callback, DIA_ALIGN_TOP),
+  GNOMEUIINFO_ITEM_NONE_DATA(N_("Center"), NULL, objects_align_v_callback, DIA_ALIGN_CENTER),
+  GNOMEUIINFO_ITEM_NONE_DATA(N_("Bottom"), NULL, objects_align_v_callback, DIA_ALIGN_BOTTOM),
+  GNOMEUIINFO_ITEM_NONE_DATA(N_("Equal Distance"), NULL, objects_align_v_callback, DIA_ALIGN_EQUAL),
+  GNOMEUIINFO_ITEM_NONE_DATA(N_("Adjacent"), NULL, objects_align_h_callback, DIA_ALIGN_ADJACENT),
   GNOMEUIINFO_END
 };
 
@@ -249,20 +263,29 @@ static GtkItemFactoryEntry toolbox_menu_items[] =
   {   "/File/tearoff",         NULL,         NULL,       0,   "<Tearoff>" },
   {N_("/File/_New"),           "<control>N", file_new_callback,         0 },
   {N_("/File/_Open"),          "<control>O", file_open_callback,        0 },
-  {N_("/File/Open _Recent"),   NULL,         NULL,       0,    "<Branch>" },
-  {N_("/File/Open Recent/---"),
-                               NULL,         NULL,       0, "<Separator>" },
+  {N_("/File/---"),            NULL,         NULL,       0, "<Separator>" },
+  {N_("/File/_Diagram tree"),  NULL,         diagtree_show_callback,    0,
+   "<ToggleItem>" },
+  {N_("/File/Sheets and Objects..."),
+                               NULL,         sheets_dialog_show_callback, 0 },
+  {N_("/File/---"),            NULL,         NULL,       0, "<Separator>" },
   {N_("/File/_Preferences..."),NULL,         file_preferences_callback, 0 },
   {N_("/File/P_lugins"),       NULL,         file_plugins_callback,     0 },
+  {N_("/File/---"),            NULL,         NULL,       0, "<Separator>" },
+    /* recent file list is dynamically inserted here */
   {N_("/File/---"),            NULL,         NULL,       0, "<Separator>" },
   {N_("/File/_Quit"),          "<control>Q", file_quit_callback,        0 },
   {N_("/_Help"),               NULL,         NULL,       0,    "<Branch>" },
   {   "/Help/tearoff",         NULL,         NULL,       0,   "<Tearoff>" },
-  {N_("/Help/_Manual"),        NULL,         help_manual_callback,      0 },
+  {N_("/Help/_Manual"),        "F1",         help_manual_callback,      0 },
   {N_("/Help/---"),            NULL,         NULL,       0, "<Separator>" },
   {N_("/Help/_About"),         NULL,         help_about_callback,       0 },
 };
 
+/* calculate the number of menu_item's */
+static int toolbox_nmenu_items = sizeof(toolbox_menu_items) / sizeof(toolbox_menu_items[0]);
+
+/* this one is needed while we create the menubar in GTK instead of Gnome */
 static GtkItemFactoryEntry display_menu_items[] =
 {
   {   "/tearoff",                 NULL,         NULL,         0, "<Tearoff>" },
@@ -307,7 +330,7 @@ static GtkItemFactoryEntry display_menu_items[] =
   {N_("/View/Zoom/50%"),          NULL,         view_zoom_set_callback,   500},
   {N_("/View/Zoom/35.4%"),        NULL,         view_zoom_set_callback,   354},
   {N_("/View/Zoom/25%"),          NULL,         view_zoom_set_callback,   250},
-  {N_("/View/Diagram Properties"),NULL,         view_diagram_properties_callback, 0},
+  {N_("/View/Diagram Properties..."),NULL,         view_diagram_properties_callback, 0},
 #ifdef HAVE_LIBART  
   {N_("/View/_AntiAliased"),      NULL,         view_aa_callback,           0, "<ToggleItem>"},
 #endif
@@ -347,18 +370,18 @@ static GtkItemFactoryEntry display_menu_items[] =
   {N_("/Objects/---"),            NULL,         NULL,        0, "<Separator>"},
   {N_("/Objects/Align _Horizontal"),       NULL, NULL,          0, "<Branch>"},
   {   "/Objects/Align Horizontal/tearoff", NULL, NULL,        0, "<Tearoff>" },
-  {N_("/Objects/Align Horizontal/Left"),   NULL, objects_align_h_callback,  0},
-  {N_("/Objects/Align Horizontal/Center"), NULL, objects_align_h_callback,  1},
-  {N_("/Objects/Align Horizontal/Right"),  NULL, objects_align_h_callback,  2},
-  {N_("/Objects/Align Horizontal/Equal Distance"), NULL, objects_align_h_callback,    4},
-  {N_("/Objects/Align Horizontal/Adjacent"), NULL, objects_align_h_callback,    5},
+  {N_("/Objects/Align Horizontal/Left"),   NULL, objects_align_h_callback,  DIA_ALIGN_LEFT},
+  {N_("/Objects/Align Horizontal/Center"), NULL, objects_align_h_callback,  DIA_ALIGN_CENTER},
+  {N_("/Objects/Align Horizontal/Right"),  NULL, objects_align_h_callback,  DIA_ALIGN_RIGHT},
+  {N_("/Objects/Align Horizontal/Equal Distance"), NULL, objects_align_h_callback,    DIA_ALIGN_EQUAL},
+  {N_("/Objects/Align Horizontal/Adjacent"), NULL, objects_align_h_callback,    DIA_ALIGN_ADJACENT},
   {N_("/Objects/Align _Vertical"),         NULL, NULL,          0, "<Branch>"},
   {   "/Objects/Align Vertical/tearoff",   NULL, NULL,        0, "<Tearoff>" },
-  {N_("/Objects/Align Vertical/Top"),      NULL, objects_align_v_callback,  0},
-  {N_("/Objects/Align Vertical/Center"),   NULL, objects_align_v_callback,  1},
-  {N_("/Objects/Align Vertical/Bottom"),   NULL, objects_align_v_callback,  2},
-  {N_("/Objects/Align Vertical/Equal Distance"),   NULL, objects_align_v_callback,    4},
-  {N_("/Objects/Align Vertical/Adjacent"), NULL, objects_align_v_callback,  5},
+  {N_("/Objects/Align Vertical/Top"),      NULL, objects_align_v_callback,  DIA_ALIGN_TOP},
+  {N_("/Objects/Align Vertical/Center"),   NULL, objects_align_v_callback,  DIA_ALIGN_CENTER},
+  {N_("/Objects/Align Vertical/Bottom"),   NULL, objects_align_v_callback,  DIA_ALIGN_BOTTOM},
+  {N_("/Objects/Align Vertical/Equal Distance"),   NULL, objects_align_v_callback,    DIA_ALIGN_EQUAL},
+  {N_("/Objects/Align Vertical/Adjacent"), NULL, objects_align_v_callback,  DIA_ALIGN_ADJACENT},
 
   {N_("/_Tools"),                 NULL,     NULL,               0, "<Branch>"},
   {   "/Tools/tearoff",           NULL,         NULL,         0, "<Tearoff>" },
@@ -383,9 +406,6 @@ static GtkItemFactoryEntry display_menu_items[] =
   {N_("/Dialogs/_Layers"),        NULL,     dialogs_layers_callback,        0},
 };
 
-/* calculate the number of menu_item's */
-static int toolbox_nmenu_items = (sizeof(toolbox_menu_items) / 
-				  sizeof(toolbox_menu_items[0]));
 static int display_nmenu_items = (sizeof(display_menu_items) /
 				  sizeof(display_menu_items[0]));
 
@@ -587,15 +607,24 @@ dia_menu_signal_proxy (GtkWidget *widget, GnomeUIInfo *uiinfo)
     (GtkItemFactoryCallback1)uiinfo->moreinfo;
   guint action = GPOINTER_TO_UINT (uiinfo->user_data);
 
-  (* signal_handler) (NULL, action, widget);
+  if (signal_handler) {
+    (* signal_handler) (NULL, action, widget);
+  } else {
+    g_warning("called dia_menu_signal_proxy with NULL signal_handler !");
+  }
 }
 
 static void
 dia_menu_signal_connect_func (GnomeUIInfo *uiinfo, gchar *signal_name,
 			     GnomeUIBuilderData *uibdata)
 {
-  gtk_signal_connect (GTK_OBJECT(uiinfo->widget), signal_name,
-		      GTK_SIGNAL_FUNC(dia_menu_signal_proxy), uiinfo);
+    /* only connect the signal if uiinfo->moreinfo not NULL */
+    /* otherwise it is non sens and generate extraneous warnings */
+    /* see bug 55047 <http:/*bugzilla.gnome.org/show_bug.cgi?id=55047> */
+    if (uiinfo->moreinfo != NULL) {
+        gtk_signal_connect (GTK_OBJECT(uiinfo->widget), signal_name,
+                            GTK_SIGNAL_FUNC(dia_menu_signal_proxy), uiinfo);
+    }
 }
 
 static GnomeUIBuilderData dia_menu_uibdata = {
@@ -607,13 +636,43 @@ static GnomeUIBuilderData dia_menu_uibdata = {
 };
 #endif
 
+/*
+  Purpose: set the generic callback for all the items in the menu "Tools"
+ */
+static void 
+menus_set_tools_callback (const char * menu_name, GtkItemFactory *item_factory)
+{
+    gint i, len;
+    GString *path;
+    GtkWidget *menuitem;
+    
+    path = g_string_new(menu_name);
+    g_string_append(path, "/Tools/");
+    len = path->len;
+    for (i = 0; i < num_tools; i++) {
+	g_string_append(path, tool_data[i].menuitem_name);
+	menuitem = menus_get_item_from_path(path->str, item_factory);
+	if (menuitem != NULL)
+	    gtk_signal_connect(GTK_OBJECT(menuitem), "activate",
+			       GTK_SIGNAL_FUNC(tool_menu_select),
+			       &tool_data[i].callback_data);
+        else
+            g_warning ("couldn't find tool menu item %s", path->str);
+	g_string_truncate(path, len);
+    }
+    g_string_free(path, TRUE);
+}
+
+
 static void
 menus_init(void)
 {
+#ifndef GNOME
+  GtkItemFactoryEntry *translated_entries;
+#else
   GtkWidget *menuitem;
-  GString *path;
+#endif
   gchar *accelfilename;
-  gint i, len;
   GList *cblist;
 
   if (!initialise)
@@ -680,18 +739,7 @@ menus_init(void)
 					      "<Display>");
 #endif
 
-  path = g_string_new("<Display>/Tools/");
-  len = path->len;
-  for (i = 0; i < num_tools; i++) {
-    g_string_append(path, tool_data[i].menuitem_name);
-    menuitem = menus_get_item_from_path(path->str);
-    if (menuitem != NULL)
-      gtk_signal_connect(GTK_OBJECT(menuitem), "activate",
-			 GTK_SIGNAL_FUNC(tool_menu_select),
-			 &tool_data[i].callback_data);
-    g_string_truncate(path, len);
-  }
-  g_string_free(path, TRUE);
+  menus_set_tools_callback ("<Display>", NULL);
 
   gtk_menu_set_title(GTK_MENU(display_menus), _("Diagram Menu"));
   
@@ -744,6 +792,45 @@ menus_get_image_menu (GtkWidget **menu,
     *accel_group = display_accels;
 }
 
+void
+menus_get_image_menubar (GtkWidget **menu, GtkItemFactory **display_mbar_item_factory)
+{
+    if (menu) 
+    {
+	GtkAccelGroup *display_mbar_accel_group;
+#ifndef GNOME
+	/* the display menubar. Pure gtk. TODO: make it GNOME */
+	display_mbar_accel_group = gtk_accel_group_new ();
+	*display_mbar_item_factory =  gtk_item_factory_new (GTK_TYPE_MENU_BAR,
+							    "<DisplayMBar>",
+							    display_mbar_accel_group);
+#ifdef ENABLE_NLS
+	gtk_item_factory_set_translate_func (*display_mbar_item_factory, 
+					     menu_translate,
+					     "<DisplayMBar>", NULL);
+#endif
+	gtk_item_factory_create_items (*display_mbar_item_factory,
+				       display_nmenu_items - 1,
+				       &(display_menu_items[1]), NULL);
+
+	*menu = gtk_item_factory_get_widget (*display_mbar_item_factory, "<DisplayMBar>");
+#else
+	GtkWidget *menuitem;
+	
+	/* the display menu */
+	*menu = gtk_menu_bar_new();
+	display_mbar_accel_group = gtk_accel_group_new();
+	menuitem = gtk_tearoff_menu_item_new();
+	gnome_app_fill_menu_custom(GTK_MENU_SHELL(*menu), display_menu,
+				   &dia_menu_uibdata, display_mbar_accel_group,
+				   TRUE, 0);
+	/*	gtk_menu_bar_prepend(GTK_MENU_BAR(*menu), menuitem);
+		gtk_widget_show (menuitem);*/
+#endif
+	menus_set_tools_callback ("<DisplayMBar>", *display_mbar_item_factory);
+    }
+}
+
 #ifdef GNOME
 /* a function that performs a similar task to gnome_app_find_menu_pos,
  * but works from the GnomeUIInfo structures, and takes an untranslated path.  It ignores 
@@ -846,6 +933,7 @@ dia_gnome_menu_get_widget (GnomeUIInfo *uiinfo, const gchar *path)
   return NULL;
 }
 
+
 /* returns parent GtkMenu */
 static GtkMenuShell *
 find_parent(GtkMenuShell *top, GnomeUIInfo *uiinfo, GHashTable *extras,
@@ -975,8 +1063,14 @@ menus_add_path (const gchar *path)
 }
 #endif
 
+
+/*
+  Get a menu item widget from a path. 
+  In case of an item in the <DisplayMBar> menu bar, provide the corresponding 
+  item factory: ddisp->mbar_item_factory.
+ */
 GtkWidget *
-menus_get_item_from_path (const gchar *path)
+menus_get_item_from_path (char *path, GtkItemFactory *item_factory)
 {
   GtkWidget *widget = NULL;
 
@@ -984,16 +1078,19 @@ menus_get_item_from_path (const gchar *path)
   const gchar *menu_name;
   DDisplay *ddisp;
 
-  /* drop the "<Display>/" or "<Toolbox>/" at the start */
+  /* drop the "<Display>/", "<DisplayMBar>/" or "<Toolbox>/" at the start */
   menu_name = path;
   if (! (path = strchr (path, '/'))) return NULL;
   path ++; /* move past the / */
 
   if (strncmp(menu_name, "<Display>", strlen("<Display>")) == 0) {
-    ddisp = ddisplay_active ();
-    if (! ddisp)
-      return NULL;
+/*    ddisp = ddisplay_active ();
+      if (! ddisp)
+      return NULL; */
     widget = dia_gnome_menu_get_widget(display_menu, path);
+  } else if  (strncmp(menu_name, "<DisplayMBar>", strlen("<DisplayMBar>"))  == 0) {
+      /* finding this requires an item factory*/
+      widget = dia_gnome_menu_get_widget(display_menu, path);
   } else if (strncmp(menu_name, "<Toolbox>", strlen("<Toolbox>")) == 0) {
     widget = dia_gnome_menu_get_widget(toolbox_menu, path);
   } 
@@ -1003,6 +1100,10 @@ menus_get_item_from_path (const gchar *path)
     widget = gtk_item_factory_get_widget(display_item_factory, path);
   }
   
+  if ((widget == NULL) && (item_factory)) {
+      widget = gtk_item_factory_get_widget(item_factory, path);
+  }
+  
   if (widget == NULL) {
     widget = gtk_item_factory_get_widget(toolbox_item_factory, path);
   }
@@ -1016,6 +1117,74 @@ menus_get_item_from_path (const gchar *path)
   return widget;
 }
 
+
+void
+menus_initialize_updatable_items (UpdatableMenuItems *items, 
+				  GtkItemFactory *factory, const char *display)
+{
+    static GString *path;
+    
+#   ifdef GNOME
+    /* This should only happen with popmenu. I don't remember why... */
+    if ((strcmp(display, "<Display>") == 0) && (ddisplay_active () == NULL))
+    {
+	return;
+    }
+#   endif
+    path = g_string_new (display);
+    g_string_append (path,"/Edit/Copy");
+    items->copy = menus_get_item_from_path(path->str, factory);
+    g_string_append (g_string_assign(path, display),"/Edit/Cut");
+    items->cut = menus_get_item_from_path(path->str, factory);
+    g_string_append (g_string_assign(path, display),"/Edit/Paste");
+    items->paste = menus_get_item_from_path(path->str, factory);
+#   ifndef GNOME
+    g_string_append (g_string_assign(path, display),"/Edit/Delete");
+    items->edit_delete = menus_get_item_from_path(path->str, factory);
+#   endif
+
+    g_string_append (g_string_assign(path, display),"/Edit/Copy Text");
+    items->copy_text = menus_get_item_from_path(path->str, factory);
+    g_string_append (g_string_assign(path, display),"/Edit/Cut Text");
+    items->cut_text = menus_get_item_from_path(path->str, factory);
+    g_string_append (g_string_assign(path, display),"/Edit/Paste Text");
+    items->paste_text = menus_get_item_from_path(path->str, factory);
+
+    g_string_append (g_string_assign(path, display),"/Objects/Send to Back");
+    items->send_to_back = menus_get_item_from_path(path->str, factory);
+    g_string_append (g_string_assign(path, display),"/Objects/Bring to Front");
+    items->bring_to_front = menus_get_item_from_path(path->str, factory);
+  
+    g_string_append (g_string_assign(path, display),"/Objects/Group");
+    items->group = menus_get_item_from_path(path->str, factory);
+    g_string_append (g_string_assign(path, display),"/Objects/Ungroup");
+    items->ungroup = menus_get_item_from_path(path->str, factory);
+
+    g_string_append (g_string_assign(path, display),"/Objects/Align Horizontal/Left");
+    items->align_h_l = menus_get_item_from_path(path->str, factory);
+    g_string_append (g_string_assign(path, display),"/Objects/Align Horizontal/Center");
+    items->align_h_c = menus_get_item_from_path(path->str, factory);
+    g_string_append (g_string_assign(path, display),"/Objects/Align Horizontal/Right");
+    items->align_h_r = menus_get_item_from_path(path->str, factory);
+    g_string_append (g_string_assign(path, display),"/Objects/Align Horizontal/Equal Distance");
+    items->align_h_e = menus_get_item_from_path(path->str, factory);
+    g_string_append (g_string_assign(path, display),"/Objects/Align Horizontal/Adjacent");
+    items->align_h_a = menus_get_item_from_path(path->str, factory);
+    g_string_append (g_string_assign(path, display),"/Objects/Align Vertical/Top");
+    items->align_v_t = menus_get_item_from_path(path->str, factory);
+    g_string_append (g_string_assign(path, display),"/Objects/Align Vertical/Center");
+    items->align_v_c = menus_get_item_from_path(path->str, factory);
+    g_string_append (g_string_assign(path, display),"/Objects/Align Vertical/Bottom");
+    items->align_v_b = menus_get_item_from_path(path->str, factory);
+    g_string_append (g_string_assign(path, display),"/Objects/Align Vertical/Equal Distance");
+    items->align_v_e = menus_get_item_from_path(path->str, factory);
+    g_string_append (g_string_assign(path, display),"/Objects/Align Vertical/Adjacent");
+    items->align_v_a = menus_get_item_from_path(path->str, factory);
+    
+    g_string_free (path,FALSE);
+}
+
+
 static void
 plugin_callback (GtkWidget *widget, gpointer data)
 {
