@@ -43,6 +43,10 @@
 #include "message.h"
 #include "preferences.h"
 #include "dia_dirs.h"
+#include "diagramdata.h"
+#ifdef PREF_CHOICE
+#include "paper.h"
+#endif
 
 struct DiaPreferences prefs;
 
@@ -53,7 +57,11 @@ enum DiaPrefType {
   PREF_UINT,
   PREF_REAL,
   PREF_UREAL,
-  PREF_COLOUR
+  PREF_COLOUR,
+#ifdef PREF_CHOICE
+  PREF_CHOICE,
+#endif
+  PREF_STRING
 };
 
 struct DiaPrefsData {
@@ -64,6 +72,10 @@ struct DiaPrefsData {
   int tab;
   char *label_text;
   GtkWidget *widget;
+  gboolean hidden;
+#ifdef PREF_CHOICE
+  char *(*choice_list_function)();
+#endif
 };
 
 static int default_true = 1;
@@ -73,8 +85,30 @@ static real default_real_zoom = 100.0;
 static int default_int_w = 500;
 static int default_int_h = 400;
 static int default_undo_depth = 15;
+static guint default_recent_documents = 5;
 static Color default_colour = { 0.85, .90, .90 }; /* Grid colour */
 static Color pbreak_colour = { 0.0, 0.0, 0.6 }; 
+static guint default_dtree_width = 220;
+static guint default_dtree_height = 100;
+static guint default_dtree_dia_sort = DIA_TREE_SORT_INSERT;
+static guint default_dtree_obj_sort = DIA_TREE_SORT_INSERT;
+
+#ifdef PREF_CHOICE
+static PaperInfo default_paper =
+{ "A4", 2.82, 2.82, 2.82, 2.82, TRUE, 100.0, FALSE, 1, 1 };
+/*
+  static gboolean default_is_portrait = FALSE;
+  static gboolean default_fitto = FALSE;
+  static gfloat default_scaling = 100.0;
+  static gint default_fitwidth = 1;
+  static gint default_fitheight = 1;
+  static gfloat default_tmargin = 2.82;
+  static gfloat default_bmargin = 2.82;
+  static gfloat default_lmargin = 2.82;
+  static gfloat default_rmargin = 2.82;
+  static gchar *default_papertype = "A4";
+*/
+#endif
 
 struct DiaPrefsTab {
   char *title;
@@ -85,8 +119,10 @@ struct DiaPrefsTab {
 struct DiaPrefsTab prefs_tabs[] =
 {
   {N_("User Interface"), NULL, 0},
+  {N_("Diagram Defaults"), NULL, 0},
   {N_("View Defaults"), NULL, 0},
   {N_("Grid Lines"), NULL, 0},
+  {N_("Diagram Tree"), NULL, 0},
 };
 
 #define NUM_PREFS_TABS (sizeof(prefs_tabs)/sizeof(struct DiaPrefsTab))
@@ -104,26 +140,61 @@ struct DiaPrefsData prefs_data[] =
   { "compress_save", PREF_BOOLEAN, PREF_OFFSET(compress_save), &default_true, 0, N_("Compress saved files:") },
   { "undo_depth", PREF_UINT, PREF_OFFSET(undo_depth), &default_undo_depth, 0, N_("Number of undo levels:") },
   { "reverse_rubberbanding_intersects", PREF_BOOLEAN, PREF_OFFSET(reverse_rubberbanding_intersects), &default_true, 0, N_("Reverse dragging selects\nintersecting objects:") },
+  { "recent_documents_list_size", PREF_UINT, PREF_OFFSET(recent_documents_list_size), &default_recent_documents, 0, N_("Recent documents list size:") },
+
+#ifdef PREF_CHOICE
+  { NULL, PREF_NONE, 0, NULL, 1, N_("New diagram:") },
+  { "new_diagram_papertype", PREF_CHOICE, PREF_OFFSET(new_paper.name), &default_paper.name, 1, N_("Paper type:"), NULL, FALSE, get_paper_name_list },
+#endif
+
   { NULL, PREF_NONE, 0, NULL, 1, N_("New window:") },
   { "new_view_width", PREF_UINT, PREF_OFFSET(new_view.width), &default_int_w, 1, N_("Width:") },
   { "new_view_height", PREF_UINT, PREF_OFFSET(new_view.height), &default_int_h, 1, N_("Height:") },
   { "new_view_zoom", PREF_UREAL, PREF_OFFSET(new_view.zoom), &default_real_zoom, 1, N_("Magnify:") },
+  { "use_menu_bar", PREF_BOOLEAN, PREF_OFFSET(new_view.use_menu_bar), &default_false, 0, N_("Use menu bar:") },
 
   { NULL, PREF_NONE, 0, NULL, 1, N_("Connection Points:") },
   { "show_cx_pts", PREF_BOOLEAN, PREF_OFFSET(show_cx_pts), &default_true, 1, N_("Visible:") },
 
-  { NULL, PREF_NONE, 0, NULL, 2, N_("Grid:") },
-  { "grid_visible", PREF_BOOLEAN, PREF_OFFSET(grid.visible), &default_true, 2, N_("Visible:") },
-  { "grid_snap", PREF_BOOLEAN, PREF_OFFSET(grid.snap), &default_false, 2, N_("Snap to:") },
-  { "grid_x", PREF_UREAL, PREF_OFFSET(grid.x), &default_real_one, 2, N_("X Size:") },
-  { "grid_y", PREF_UREAL, PREF_OFFSET(grid.y), &default_real_one, 2, N_("Y Size:") },
-  { "grid_colour", PREF_COLOUR, PREF_OFFSET(grid.colour), &default_colour, 2, N_("Colour:") },
-  { "grid_solid", PREF_BOOLEAN, PREF_OFFSET(grid.solid), &default_true, 2, N_("Solid lines:") },
-
   { NULL, PREF_NONE, 0, NULL, 2, N_("Page breaks:") },
   { "pagebreak_visible", PREF_BOOLEAN, PREF_OFFSET(pagebreak.visible), &default_true, 2, N_("Visible:") },
   { "pagebreak_colour", PREF_COLOUR, PREF_OFFSET(pagebreak.colour), &pbreak_colour, 2, N_("Colour:") },
   { "pagebreak_solid", PREF_BOOLEAN, PREF_OFFSET(pagebreak.solid), &default_true, 2, N_("Solid lines:") },
+  
+  /*{ NULL, PREF_NONE, 0, NULL, 3, N_("Grid:") }, */
+  { "grid_visible", PREF_BOOLEAN, PREF_OFFSET(grid.visible), &default_true, 3, N_("Visible:") },
+  { "grid_snap", PREF_BOOLEAN, PREF_OFFSET(grid.snap), &default_false, 3, N_("Snap to:") },
+  { "grid_x", PREF_UREAL, PREF_OFFSET(grid.x), &default_real_one, 3, N_("X Size:") },
+  { "grid_y", PREF_UREAL, PREF_OFFSET(grid.y), &default_real_one, 3, N_("Y Size:") },
+  { "grid_colour", PREF_COLOUR, PREF_OFFSET(grid.colour), &default_colour, 3, N_("Colour:") },
+  { "grid_solid", PREF_BOOLEAN, PREF_OFFSET(grid.solid), &default_true, 3, N_("Solid lines:") },  
+
+  { "render_bounding_boxes", PREF_BOOLEAN,PREF_OFFSET(render_bounding_boxes),
+    &default_false,0,"render bounding boxes:",NULL, TRUE},
+
+  { "pretty_formated_xml", PREF_BOOLEAN,PREF_OFFSET(pretty_formated_xml),
+    &default_false,0,"pretty formated xml:",NULL, TRUE},
+
+  { "prefer_psprint", PREF_BOOLEAN,PREF_OFFSET(prefer_psprint),
+    &default_false,0,"prefer psprint:", NULL, TRUE},
+
+  { NULL, PREF_NONE, 0, NULL, 4, N_("Diagram tree window:") },
+  { "show_diagram_tree", PREF_BOOLEAN, PREF_OFFSET(dia_tree.show_tree),
+    &default_false, 4, N_("Show at startup:")},
+  { "diagram_tree_width", PREF_UINT, PREF_OFFSET(dia_tree.width),
+    &default_dtree_width, 4, N_("Default width:")},
+  { "diagram_tree_height", PREF_UINT, PREF_OFFSET(dia_tree.height),
+    &default_dtree_height, 4, N_("Default height:")},
+  { "diagram_tree_save_size", PREF_BOOLEAN, PREF_OFFSET(dia_tree.save_size),
+    &default_false, 4, N_("Remember last size:")},
+  { "diagram_tree_save_hidden", PREF_BOOLEAN, PREF_OFFSET(dia_tree.save_hidden),
+    &default_false, 4, N_("Save hidden object types:")},
+  { "diagram_tree_dia_sort", PREF_UINT, PREF_OFFSET(dia_tree.dia_sort),
+    &default_dtree_dia_sort, 4, "default diagram sort order", NULL, TRUE},
+  { "diagram_tree_obj_sort", PREF_UINT, PREF_OFFSET(dia_tree.obj_sort),
+    &default_dtree_obj_sort, 4, "default object sort order", NULL, TRUE},
+  { "diagram_tree_hidden", PREF_STRING, PREF_OFFSET(dia_tree.hidden),
+    &DIA_TREE_DEFAULT_HIDDEN, 4, "hidden type list", NULL, TRUE},
 };
 
 #define NUM_PREFS_DATA (sizeof(prefs_data)/sizeof(struct DiaPrefsData))
@@ -214,6 +285,12 @@ prefs_set_defaults(void)
     case PREF_COLOUR:
       *(Color *)ptr = *(Color *)prefs_data[i].default_value;
       break;
+#ifdef PREF_CHOICE
+    case PREF_CHOICE:
+#endif
+    case PREF_STRING:
+      *(gchar **)ptr = *(gchar **)prefs_data[i].default_value;
+      break;
     case PREF_NONE:
       break;
     }
@@ -267,6 +344,13 @@ prefs_save(void)
     case PREF_COLOUR:
       fprintf(file, "%f %f %f\n", (double) ((Color *)ptr)->red,
 	      (double) ((Color *)ptr)->green, (double) ((Color *)ptr)->blue);
+      break;
+#ifdef PREF_CHOICE
+    case PREF_CHOICE:
+#endif
+    case PREF_STRING:
+      fprintf(file, "\"%s\"\n", *(gchar **)ptr);
+      break;
     case PREF_NONE:
       break;
     }
@@ -338,6 +422,15 @@ prefs_parse_line(GScanner *scanner)
       return G_TOKEN_FLOAT;
     ((Color *)ptr)->blue = scanner->value.v_float;
 
+    break;
+#ifdef PREF_CHOICE
+  case PREF_CHOICE:
+#endif
+  case PREF_STRING:
+    if (token != G_TOKEN_STRING)
+      return G_TOKEN_STRING;
+
+    *(char **)ptr = g_strdup(scanner->value.v_string);
     break;
   case PREF_NONE:
     break;
@@ -415,8 +508,10 @@ prefs_load(void)
   }
   
   g_scanner_destroy (scanner);
-  
+ 
   close(fd);
+  render_bounding_boxes = prefs.render_bounding_boxes;
+  pretty_formated_xml = prefs.pretty_formated_xml;
 }
 
 static gint
@@ -457,6 +552,14 @@ prefs_set_value_in_widget(GtkWidget * widget, enum DiaPrefType type,
   case PREF_COLOUR:
     dia_color_selector_set_color(DIACOLORSELECTOR(widget), (Color *)ptr);
     break;
+#ifdef PREF_CHOICE
+  case PREF_CHOICE:
+    gtk_
+    break;
+#endif
+  case PREF_STRING:
+    gtk_entry_set_text(GTK_ENTRY(widget), (gchar *)(*((gchar **)ptr)));
+    break;
   case PREF_NONE:
     break;
   }
@@ -483,6 +586,14 @@ prefs_get_value_from_widget(GtkWidget * widget, enum DiaPrefType type,
   case PREF_COLOUR:
     dia_color_selector_get_color(DIACOLORSELECTOR(widget), (Color *)ptr);
     break;
+#ifdef PREF_CHOICE
+  case PREF_CHOICE:
+    break;
+#endif
+  case PREF_STRING:
+    *((gchar **)ptr) = (gchar *)
+      gtk_entry_get_text(GTK_ENTRY(widget));
+    break;
   case PREF_NONE:
     break;
   }
@@ -543,6 +654,8 @@ prefs_get_property_widget(enum DiaPrefType type)
   case PREF_COLOUR:
     widget = dia_color_selector_new();
     break;
+  case PREF_STRING:
+    widget = gtk_entry_new();
   case PREF_NONE:
     widget = NULL;
     break;
@@ -671,7 +784,9 @@ prefs_create_dialog(void)
     GtkTable *table;
     GtkWidget *widget;
     int row;
-    
+
+    if (prefs_data[i].hidden) continue;
+
     label = gtk_label_new (gettext(prefs_data[i].label_text));
     gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.3);
     gtk_widget_show (label);
@@ -703,6 +818,7 @@ prefs_update_prefs_from_dialog(void)
   char *ptr;
   
   for (i=0;i<NUM_PREFS_DATA;i++) {
+    if (prefs_data[i].hidden) continue;
     widget = prefs_data[i].widget;
     ptr = (char *)&prefs + prefs_data[i].offset;
     
@@ -718,6 +834,7 @@ prefs_update_dialog_from_prefs(void)
   char *ptr;
   
   for (i=0;i<NUM_PREFS_DATA;i++) {
+    if (prefs_data[i].hidden) continue;
     widget = prefs_data[i].widget;
     ptr = (char *)&prefs + prefs_data[i].offset;
     
