@@ -15,6 +15,35 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
+
+/* The eps_dump_truetype_body function and much inspiration for font dumping
+ * came from ttfps, which bears the following license notice:
+
+ Copyright (c) 1997 by Juliusz Chroboczek 
+
+ Copying
+ *******
+
+ This software is provided with no guarantee, not even of any kind.
+
+ Feel free to do whatever you wish with it as long as you don't ask me
+ to maintain it.
+
+*/
+
+/* The Document Structure Definitions used for the output is available at
+ * http://www-cdf.fnal.gov/offline/PostScript/psstruct.ps
+ * (Appendix G of the Red and White Book)
+ */
+
+/* Note: There is a use of setmatrix in ellipse, which is supposed to be
+ * avoided.  Could it be?
+ */
+
+/* Note that the EPS renderer now has two phases:  One to collect font
+ * info (and conceivably more, like color defs), and one to actually render.
+ */
+
 #include <config.h>
 
 #include <string.h>
@@ -24,20 +53,21 @@
 #include <unistd.h>
 #endif
 #include <locale.h>
+#include <errno.h>
 
 #include "intl.h"
 #include "render_eps.h"
 #include "message.h"
 #include "diagramdata.h"
 #include "charconv.h" 
+#include "font.h"
 
 #ifdef HAVE_UNICODE
 #include <unicode.h>
 #include "ps-utf8.h"
 #endif
 
-
-static void begin_render(RendererEPS *renderer, DiagramData *data);
+static void begin_render(RendererEPS *renderer);
 static void end_render(RendererEPS *renderer);
 static void set_linewidth(RendererEPS *renderer, real linewidth);
 static void set_linecaps(RendererEPS *renderer, LineCaps mode);
@@ -45,7 +75,7 @@ static void set_linejoin(RendererEPS *renderer, LineJoin mode);
 static void set_linestyle(RendererEPS *renderer, LineStyle mode);
 static void set_dashlength(RendererEPS *renderer, real length);
 static void set_fillstyle(RendererEPS *renderer, FillStyle mode);
-static void set_font(RendererEPS *renderer, Font *font, real height);
+static void set_font(RendererEPS *renderer, DiaFont *font, real height);
 static void draw_line(RendererEPS *renderer, 
 		      Point *start, Point *end, 
 		      Color *line_color);
@@ -99,39 +129,98 @@ static void draw_image(RendererEPS *renderer,
 		       real width, real height,
 		       DiaImage image);
 
-static RenderOps EpsRenderOps = {
-  (BeginRenderFunc) begin_render,
-  (EndRenderFunc) end_render,
+/* These functions are used to create the prolog.  Currently takes care
+ * of defining necessary fonts.
+ */
+static void begin_prolog(RendererEPS *renderer);
+static void end_prolog(RendererEPS *renderer);
+static void prolog_define_font(RendererEPS *renderer,
+			       DiaFont *font, real height);
+static void prolog_check_string(RendererEPS *renderer,
+				const char *text,
+				Point *pos, Alignment alignment,
+				Color *color);
 
-  (SetLineWidthFunc) set_linewidth,
-  (SetLineCapsFunc) set_linecaps,
-  (SetLineJoinFunc) set_linejoin,
-  (SetLineStyleFunc) set_linestyle,
-  (SetDashLengthFunc) set_dashlength,
-  (SetFillStyleFunc) set_fillstyle,
-  (SetFontFunc) set_font,
+static void init_eps_renderer();
+
+static void null_func() {}
+
+static RenderOps *EpsRenderOps;
+static RenderOps *EpsPrologOps;
+
+static void
+init_eps_renderer() {
+  EpsRenderOps = create_renderops_table();
+
+  EpsRenderOps->begin_render = (BeginRenderFunc) begin_render;
+  EpsRenderOps->end_render = (EndRenderFunc) end_render;
+
+  EpsRenderOps->set_linewidth = (SetLineWidthFunc) set_linewidth;
+  EpsRenderOps->set_linecaps = (SetLineCapsFunc) set_linecaps;
+  EpsRenderOps->set_linejoin = (SetLineJoinFunc) set_linejoin;
+  EpsRenderOps->set_linestyle = (SetLineStyleFunc) set_linestyle;
+  EpsRenderOps->set_dashlength = (SetDashLengthFunc) set_dashlength;
+  EpsRenderOps->set_fillstyle = (SetFillStyleFunc) set_fillstyle;
+  EpsRenderOps->set_font = (SetFontFunc) set_font;
   
-  (DrawLineFunc) draw_line,
-  (DrawPolyLineFunc) draw_polyline,
+  EpsRenderOps->draw_line = (DrawLineFunc) draw_line;
+  EpsRenderOps->draw_polyline = (DrawPolyLineFunc) draw_polyline;
   
-  (DrawPolygonFunc) draw_polygon,
-  (FillPolygonFunc) fill_polygon,
+  EpsRenderOps->draw_polygon = (DrawPolygonFunc) draw_polygon;
+  EpsRenderOps->fill_polygon = (FillPolygonFunc) fill_polygon;
 
-  (DrawRectangleFunc) draw_rect,
-  (FillRectangleFunc) fill_rect,
+  EpsRenderOps->draw_rect = (DrawRectangleFunc) draw_rect;
+  EpsRenderOps->fill_rect = (FillRectangleFunc) fill_rect;
 
-  (DrawArcFunc) draw_arc,
-  (FillArcFunc) fill_arc,
+  EpsRenderOps->draw_arc = (DrawArcFunc) draw_arc;
+  EpsRenderOps->fill_arc = (FillArcFunc) fill_arc;
 
-  (DrawEllipseFunc) draw_ellipse,
-  (FillEllipseFunc) fill_ellipse,
+  EpsRenderOps->draw_ellipse = (DrawEllipseFunc) draw_ellipse;
+  EpsRenderOps->fill_ellipse = (FillEllipseFunc) fill_ellipse;
 
-  (DrawBezierFunc) draw_bezier,
-  (FillBezierFunc) fill_bezier,
+  EpsRenderOps->draw_bezier = (DrawBezierFunc) draw_bezier;
+  EpsRenderOps->fill_bezier = (FillBezierFunc) fill_bezier;
 
-  (DrawStringFunc) draw_string,
+  EpsRenderOps->draw_string = (DrawStringFunc) draw_string;
 
-  (DrawImageFunc) draw_image,
+  EpsRenderOps->draw_image = (DrawImageFunc) draw_image;
+
+
+  EpsPrologOps = create_renderops_table();
+
+  EpsPrologOps->begin_render = (BeginRenderFunc) begin_prolog;
+  EpsPrologOps->end_render = (EndRenderFunc) end_prolog;
+
+  EpsPrologOps->set_linewidth = (SetLineWidthFunc) null_func;
+  EpsPrologOps->set_linecaps = (SetLineCapsFunc) null_func;
+  EpsPrologOps->set_linejoin = (SetLineJoinFunc) null_func;
+  EpsPrologOps->set_linestyle = (SetLineStyleFunc) null_func;
+  EpsPrologOps->set_dashlength = (SetDashLengthFunc) null_func;
+  EpsPrologOps->set_fillstyle = (SetFillStyleFunc) null_func;
+  EpsPrologOps->set_font = (SetFontFunc) prolog_define_font;
+  
+  EpsPrologOps->draw_line = (DrawLineFunc) null_func;
+  EpsPrologOps->draw_polyline = (DrawPolyLineFunc) null_func;
+  
+  EpsPrologOps->draw_polygon = (DrawPolygonFunc) null_func;
+  EpsPrologOps->fill_polygon = (FillPolygonFunc) null_func;
+
+  EpsPrologOps->draw_rect = (DrawRectangleFunc) null_func;
+  EpsPrologOps->fill_rect = (FillRectangleFunc) null_func;
+
+  EpsPrologOps->draw_arc = (DrawArcFunc) null_func;
+  EpsPrologOps->fill_arc = (FillArcFunc) null_func;
+
+  EpsPrologOps->draw_ellipse = (DrawEllipseFunc) null_func;
+  EpsPrologOps->fill_ellipse = (FillEllipseFunc) null_func;
+
+  EpsPrologOps->draw_bezier = (DrawBezierFunc) null_func;
+  EpsPrologOps->fill_bezier = (FillBezierFunc) null_func;
+
+  EpsPrologOps->draw_string = (DrawStringFunc) prolog_check_string;
+
+  EpsPrologOps->draw_image = (DrawImageFunc) null_func;
+  
 };
 
 
@@ -160,11 +249,254 @@ static PSUnicoderCallbacks eps_unicoder_callbacks = {
   eps_show_string,
   eps_get_string_width,
 };
+#endif
 
-#else /* !HAVE_UNICODE */
+#ifdef HAVE_FREETYPE
+/* Dumps a PFB style chunk into a file as ASCII.
+ * 'from' should start at the ASCII/BINARY indicator (after 080).
+ * Returns 1 if more is expected.
+ */
+static int
+dump_pfb_chunk(FILE *from, FILE *file) {
+  gchar chunktype = fgetc(from);
+  int length = 0;
+  int i, charcount;
+  int ch;
+  static char *hexChar = "0123456789abcdef";
 
-static void print_reencode_font(FILE *file, char *fontname)
+  if (chunktype == 3) { /* Close chunk */
+    fgetc(from); /* Read up to EOF */
+    return 0;
+  }
+  if (ch == -1) {
+    printf("Read error: %s\n", strerror(errno));
+    return 0;
+  }
+
+  for (i = 0; i < 4; i++) {
+    ch = fgetc(from);
+    if (ch == EOF) {
+      printf("Read error: %s\n", strerror(errno));
+      return 0;
+    }
+    length += ch << (i*8);
+  }
+  switch (chunktype) {
+  case 1: /* ^A -- ASCII */
+    for (i = 0; i < length; i++) {
+      ch = fgetc(from);
+      if (ch == EOF) {
+	printf("Read error: %s\n", strerror(errno));
+	return 0;
+      }
+      if (fputc((ch == '\r'?'\n':ch), file) == EOF) { /* Translate newlines */
+	printf("Write error: %s\n", strerror(errno));
+	return 0;
+      }
+    }
+    fgetc(from); /* Skip 080 char */
+    break;
+  case 2: /* ^B -- Binary */
+    charcount = 0;
+    for (i = 0; i < length; i++) {
+      ch = fgetc(from);
+      if (ch == EOF) {
+	printf("Read error: %s\n", strerror(errno));
+	return 0;
+      }
+      /* Translate binary into hex */
+      if (fputc(hexChar[(ch>>4)&0xf], file) == EOF ||
+	  fputc(hexChar[ch&0xf], file) == EOF) {
+	printf("Write error: %s\n", strerror(errno));
+	return 0;
+      }
+      if ((++charcount)%32 == 0) /* Break lines after 64 chars */
+	if (fputc('\n', file) == EOF) {
+	  printf("Write error: %s\n", strerror(errno));
+	  return 0;
+	}
+    }
+    if (charcount%32 != 1) fputc('\n', file);
+    fgetc(from); /* Skip 080 char */
+    break;
+  default:
+    printf("Unknown chunk type %d\n", chunktype);
+    return 0;
+  }
+  return 1;
+}
+
+#define CHUNKSIZE 4000
+
+/* Taken from ttfps */
+void 
+eps_dump_truetype_body(FILE *out, int fd)
 {
+  unsigned char *buffer;
+  int i,j;
+
+  lseek(fd,0,SEEK_SET);
+
+  buffer=malloc(CHUNKSIZE);
+
+  fprintf(out,"/sfnts [");
+  for(;;) {
+    i=read(fd,buffer,CHUNKSIZE);
+    if(i==0)
+      break;
+    fprintf(out,"\n<");
+    for(j=0;j<i;j++) {
+      if(j!=0 && j%36==0)
+        fprintf(out,"\n");
+      fprintf(out,"%02X",(int)buffer[j]);
+    }
+    fprintf(out,"00>");         /* Adobe bug? */
+    if(i<CHUNKSIZE)
+      break;
+  }
+  fprintf(out,"\n] def\n");
+}
+
+#define NAMEBUF_SIZE 1000
+static void
+eps_dump_truetype(FILE *in, FILE *out, DiaFont *font) {
+  int nglyphs;
+  int n, i;
+  FT_Face face = font_get_freetype_face(font);
+  char *namebuf[NAMEBUF_SIZE];
+
+  TT_Header *head = (TT_Header *)FT_Get_Sfnt_Table(face, 0);
+  if (head == NULL) {
+    g_warning("Couldn't dump font %s\n", font_get_psfontname(font));
+    return;
+  }
+
+  fprintf(out,"%%!PS-TrueTypeFont\n");
+  /*  if(pt->maxMemType42) */
+  /*    fprintf(out,"%%%%VMUsage: %ld %ld\n",pt->minMemType42, pt->maxMemType42); */
+  fprintf(out,"%d dict begin\n",11);
+  /* Hopefully this shall be the ps name */
+  fprintf(out,"/FontName /%s def\n", font_get_psfontname(font));
+  fprintf(out,"/Encoding StandardEncoding def\n");
+  fprintf(out,"/PaintType 0 def\n/FontMatrix [1 0 0 1 0 0] def\n");
+  fprintf(out,"/FontBBox [%ld %ld %ld %ld] def\n",
+	  head->xMin, head->yMin,
+	  head->xMax, head->yMax);
+  fprintf(out,"/FontType 42 def\n");
+  fprintf(out,"/FontInfo 8 dict dup begin\n");
+  /* Is this Table_Version or Font_Revision? */
+  fprintf(out,"/version (%d.%d) def\n",
+	  (head->Font_Revision>>16)&0xFFFF,
+	  head->Font_Revision&0xFFFF);
+  fprintf(out,"end readonly def\n");
+
+  eps_dump_truetype_body(out, fileno(in));
+
+  nglyphs = face->num_glyphs;
+
+  fprintf(out,"/CharStrings %d dict dup begin\n", nglyphs);
+
+  if (!head->Mac_Style) {
+    for(n=i=0;i<nglyphs;i++) {
+      if(n!=0 && n%4==0)
+        fprintf(out,"\n");
+      FT_Get_Glyph_Name(face, i, namebuf, NAMEBUF_SIZE);
+      if (*namebuf) {
+        fprintf(out,"/%s %d def ",namebuf,i);
+        n++;
+      }
+    }
+  } else {
+    g_warning("No glyph names for %s, implement Mac encoding\n", 
+	      font_get_psfontname(font));
+    /*
+    for(i=0; i<258 && i<nglyphs; i++) {
+      fprintf(out,"/%s %d def ",macGlyphEncoding[i],i);
+      if(i!=0 && i%4==0)
+        fprintf(out,"\n");
+    }
+    */
+  }
+  fprintf(out,"end readonly def\n");
+  fprintf(out,"FontName currentdict end definefont pop\n");
+
+}
+
+/* Simplest version:  Just dump the font file directly.
+ */
+static void
+eps_dump_font_file(FILE *file, char *from_file, DiaFont *font) {
+  char buf[81];
+  FILE *from = fopen(from_file, "r");
+  int num_read;
+  char first;
+  int i;
+  FT_Face face = font_get_freetype_face(font);
+
+  if (from == NULL) {
+    printf("Can't open font file %s: %s\n", from_file, strerror(errno));
+    return;
+  }
+
+  fprintf(file, "%%%%BeginResource: font %s\n", 
+	  font_get_psfontname(font));
+
+  if (FT_IS_SFNT(face)) { /* Truetype font */
+    eps_dump_truetype(from, file, font);
+  } else {
+    first = fgetc(from);
+    if (first == '%') {
+      /* ASCII file, just copy it blindly */
+      fputc('%', file);
+      while ((num_read = fread(buf, 1, 80, from)) != 0) {
+	fwrite(buf, 1, num_read, file);
+      }
+    } else {
+      /* Binary file. */
+      while (dump_pfb_chunk(from, file))
+	;
+    }
+  }
+  fprintf(file, "\n%%%%EndResource\n");
+  fclose(from);
+
+  return;
+}
+
+/* Add a font into the file */
+static void
+eps_add_font(FILE *file, DiaFont *font)
+{
+  char glyphname_max = 100;
+  char glyphname[glyphname_max+1];
+  int i, j;
+  
+  /* First find the correct face */
+  GList *face = font->family->freetype_family->faces;
+  for (; face != NULL; face = g_list_next(face)) {
+    FreetypeFace *a_face = (FreetypeFace*)face->data;
+    if (!strcmp(a_face->face->style_name, font->style) &&
+	!strcmp(a_face->face->family_name, font->name)) {
+      eps_dump_font_file(file, a_face->from_file, font);
+
+      return;
+    }
+  }
+}
+#endif
+
+static void print_define_font(gpointer key, gpointer value,
+			      gpointer data)
+{
+  DiaFont *font = (DiaFont *)value;
+  gchar *fontname = (gchar *)key;
+  FILE *file = (FILE *)data;
+ 
+#ifdef HAVE_FREETYPE   
+  eps_add_font(file, font);
+#endif
+
+#ifndef HAVE_UNICODE
   /* Don't reencode the Symbol font, as it doesn't work in latin1 encoding.
    * Instead, just define Symbol-latin1 to be the same as Symbol. */
   if (!strcmp(fontname, "Symbol"))
@@ -181,8 +513,222 @@ static void print_reencode_font(FILE *file, char *fontname)
 	    "	/Encoding isolatin1encoding def\n"
 	    "    currentdict end\n"
 	    "definefont pop\n", fontname, fontname);
+#endif
 }
+
+/* This hashtable keeps track of the fonts used in the diagram.
+ * Indexed by psfontname, elements are DiaFonts.
+ */
+static GHashTable *font_table;
+
+void
+begin_prolog(RendererEPS *renderer)
+{
+  font_table = g_hash_table_new(g_str_hash, g_str_equal);
+  fprintf(renderer->file, "%%%%BeginProlog\n");
+
+#ifndef HAVE_UNICODE
+  fprintf(renderer->file,
+	  "[ /.notdef /.notdef /.notdef /.notdef /.notdef /.notdef /.notdef /.notdef /.notdef /.notdef\n"
+	  "/.notdef /.notdef /.notdef /.notdef /.notdef /.notdef /.notdef /.notdef /.notdef /.notdef\n"
+	  "/.notdef /.notdef /.notdef /.notdef /.notdef /.notdef /.notdef /.notdef /.notdef /.notdef\n"
+	  "/.notdef /.notdef /space /exclam /quotedbl /numbersign /dollar /percent /ampersand /quoteright\n"
+	  "/parenleft /parenright /asterisk /plus /comma /hyphen /period /slash /zero /one\n"
+	  "/two /three /four /five /six /seven /eight /nine /colon /semicolon\n"
+	  "/less /equal /greater /question /at /A /B /C /D /E\n"
+	  "/F /G /H /I /J /K /L /M /N /O\n"
+	  "/P /Q /R /S /T /U /V /W /X /Y\n"
+	  "/Z /bracketleft /backslash /bracketright /asciicircum /underscore /quoteleft /a /b /c\n"
+	  "/d /e /f /g /h /i /j /k /l /m\n"
+	  "/n /o /p /q /r /s /t /u /v /w\n"
+	  "/x /y /z /braceleft /bar /braceright /asciitilde /.notdef /.notdef /.notdef\n"
+	  "/.notdef /.notdef /.notdef /.notdef /.notdef /.notdef /.notdef /.notdef /.notdef /.notdef\n"
+	  "/.notdef /.notdef /.notdef /.notdef /.notdef /.notdef /.notdef /.notdef /.notdef /.notdef\n"
+	  "/.notdef /.notdef /.notdef /.notdef /.notdef /.notdef /.notdef /.notdef /.notdef /.notdef\n"
+	  "/space /exclamdown /cent /sterling /currency /yen /brokenbar /section /dieresis /copyright\n"
+	  "/ordfeminine /guillemotleft /logicalnot /hyphen /registered /macron /degree /plusminus /twosuperior /threesuperior\n"
+	  "/acute /mu /paragraph /periodcentered /cedilla /onesuperior /ordmasculine /guillemotright /onequarter /onehalf\n"
+	  "/threequarters /questiondown /Agrave /Aacute /Acircumflex /Atilde /Adieresis /Aring /AE /Ccedilla\n"
+	  "/Egrave /Eacute /Ecircumflex /Edieresis /Igrave /Iacute /Icircumflex /Idieresis /Eth /Ntilde\n"
+	  "/Ograve /Oacute /Ocircumflex /Otilde /Odieresis /multiply /Oslash /Ugrave /Uacute /Ucircumflex\n"
+	  "/Udieresis /Yacute /Thorn /germandbls /agrave /aacute /acircumflex /atilde /adieresis /aring\n"
+	  "/ae /ccedilla /egrave /eacute /ecircumflex /edieresis /igrave /iacute /icircumflex /idieresis\n"
+	  "/eth /ntilde /ograve /oacute /ocircumflex /otilde /odieresis /divide /oslash /ugrave\n"
+	  "/uacute /ucircumflex /udieresis /yacute /thorn /ydieresis] /isolatin1encoding exch def\n");
+
+#endif /* !HAVE_UNICODE */
+  fprintf(renderer->file,
+	  "/cp {closepath} bind def\n"
+	  "/c {curveto} bind def\n"
+	  "/f {fill} bind def\n"
+	  "/a {arc} bind def\n"
+	  "/ef {eofill} bind def\n"
+	  "/ex {exch} bind def\n"
+	  "/gr {grestore} bind def\n"
+	  "/gs {gsave} bind def\n"
+	  "/sa {save} bind def\n"
+	  "/rs {restore} bind def\n"
+	  "/l {lineto} bind def\n"
+	  "/m {moveto} bind def\n"
+	  "/rm {rmoveto} bind def\n"
+	  "/n {newpath} bind def\n"
+	  "/s {stroke} bind def\n"
+	  "/sh {show} bind def\n"
+	  "/slc {setlinecap} bind def\n"
+	  "/slj {setlinejoin} bind def\n"
+	  "/slw {setlinewidth} bind def\n"
+	  "/srgb {setrgbcolor} bind def\n"
+	  "/rot {rotate} bind def\n"
+	  "/sc {scale} bind def\n"
+	  "/sd {setdash} bind def\n"
+	  "/ff {findfont} bind def\n"
+	  "/sf {setfont} bind def\n"
+	  "/scf {scalefont} bind def\n"
+	  "/sw {stringwidth pop} bind def\n"
+	  "/tr {translate} bind def\n"
+
+	  "\n/ellipsedict 8 dict def\n"
+	  "ellipsedict /mtrx matrix put\n"
+	  "/ellipse\n"
+	  "{ ellipsedict begin\n"
+          "   /endangle exch def\n"
+          "   /startangle exch def\n"
+          "   /yrad exch def\n"
+          "   /xrad exch def\n"
+          "   /y exch def\n"
+          "   /x exch def"
+	  "   /savematrix mtrx currentmatrix def\n"
+          "   x y tr xrad yrad sc\n"
+          "   0 0 1 startangle endangle arc\n"
+          "   savematrix setmatrix\n"
+          "   end\n"
+	  "} def\n\n"
+
+	  /*
+	    "/colortogray {\n"
+	    "/rgbdata exch store\n"
+	    "rgbdata length 3 idiv\n"
+	    "/npixls exch store\n"
+	    "/rgbindx 0 store\n"
+	    "0 1 npixls 1 sub {\n"
+	    "grays exch\n"
+	    "rgbdata rgbindx       get 20 mul\n"
+	    "rgbdata rgbindx 1 add get 32 mul\n"
+	    "rgbdata rgbindx 2 add get 12 mul\n"
+	    "add add 64 idiv\n"
+	    "put\n"
+	    "/rgbindx rgbindx 3 add store\n"
+	    "} for\n"
+	    "grays 0 npixls getinterval\n"
+	    "} bind def\n"
+	  */
+	  "/mergeprocs {\n"
+	  "dup length\n"
+	  "3 -1 roll\n"
+	  "dup\n"
+	  "length\n"
+	  "dup\n"
+	  "5 1 roll\n"
+	  "3 -1 roll\n"
+	  "add\n"
+	  "array cvx\n"
+	  "dup\n"
+	  "3 -1 roll\n"
+	  "0 exch\n"
+	  "putinterval\n"
+	  "dup\n"
+	  "4 2 roll\n"
+	  "putinterval\n"
+	  "} bind def\n"
+	  /*
+	    "/colorimage {\n"
+	    "pop pop\n"
+	    "{colortogray} mergeprocs\n"
+	    "image\n"
+	    "} bind def\n\n"
+	  */);
+}
+
+static void
+end_prolog(RendererEPS *renderer)
+{}
+
+static void
+prolog_define_font(RendererEPS *renderer, DiaFont *font, real height)
+{
+  g_hash_table_insert(font_table, font_get_psfontname(font), font);
+}
+
+static void
+prolog_check_string(RendererEPS *renderer,
+                    const char *text,
+                    Point *pos, Alignment alignment,
+                    Color *color)
+{
+#ifdef HAVE_UNICODE    
+    const char *utf8_buffer;
+    int utf8_len;
+
+    if ((renderer->psu) && (text) && (text != (const char *)(1))) {
+
+#ifndef UNICODE_WORK_IN_PROGRESS
+  /* <FIXME:> dia doesn't talk UTF-8 but the local charset. The PSUnicoder
+     talks UTF-8. We do a quick-and-dirty translation for now. */
+        utf8_buffer = charconv_local8_to_utf8(text);
+            /* </FIXME> */
+#else
+        utf8_buffer=text;
+#endif
+
+        utf8_len = strlen(utf8_buffer); /* deliberate */
+
+        if (utf8_len > 0) {
+            psu_check_string_encodings(renderer->psu,utf8_buffer);
+#ifndef UNICODE_WORK_IN_PROGRESS
+            g_free(utf8_buffer);
+#endif
+        }
+    }
 #endif /* HAVE_UNICODE */
+    
+  /* In here we can grab all the chars needed, to allow incremental
+   * font defs (or even just partial font defs).
+   */
+  /* But we won't do that just yet.
+     
+   */
+}
+
+/* This function switches the RendererEPS into render mode */
+void
+eps_renderer_prolog_done(RendererEPS *renderer) {
+  g_hash_table_foreach(font_table, print_define_font, renderer->file);
+
+  fprintf(renderer->file, 
+	  "%%%%EndProlog\n\n"
+	  "%%%%BeginSetup\n"
+	  "%%%%EndSetup\n");
+  renderer->renderer.ops = EpsRenderOps;
+}
+
+static void
+eps_renderer_set_scale(DiagramData *data, RendererEPS *renderer)
+{
+  double scale;
+  Rectangle *extent;
+  extent = &data->extents;
+  
+  scale = 28.346 * data->paper.scaling;
+
+  fprintf(renderer->file,
+	  "%f %f scale\n"
+	  "%f %f translate\n\n",
+	  scale, -scale,
+	  -extent->left, -extent->bottom );
+}
+
+  /*** PROLOG STUFF ENDS HERE ***/
+
 
 static RendererEPS *
 create_eps_renderer(DiagramData *data, const char *filename,
@@ -202,8 +748,11 @@ create_eps_renderer(DiagramData *data, const char *filename,
     return NULL;
   }
 
+  if (EpsPrologOps == NULL)
+    init_eps_renderer();
+
   renderer = g_new(RendererEPS, 1);
-  renderer->renderer.ops = &EpsRenderOps;
+  renderer->renderer.ops = EpsPrologOps;
   renderer->renderer.is_interactive = 0;
   renderer->renderer.interactive_ops = NULL;
 
@@ -235,178 +784,14 @@ create_eps_renderer(DiagramData *data, const char *filename,
 	  "%%%%Orientation: Portrait\n"
 	  "%%%%BoundingBox: 0 0 %d %d\n" 
 	  "%%%%Pages: 1\n"
-	  "%%%%BeginSetup\n"
-	  "%%%%EndSetup\n"
 	  "%%%%EndComments\n",
-	  diafilename,
+	  g_basename(diafilename),
 	  VERSION,
 	  ctime(&time_now),
 	  name,
 	  (int) ceil((extent->right - extent->left)*scale),
 	  (int) ceil((extent->bottom - extent->top)*scale) );
 
-  fprintf(file, "%%%%BeginProlog\n");
-#ifndef HAVE_UNICODE
-  fprintf(file,
-	  "[ /.notdef /.notdef /.notdef /.notdef /.notdef /.notdef /.notdef /.notdef /.notdef /.notdef\n"
-	  "/.notdef /.notdef /.notdef /.notdef /.notdef /.notdef /.notdef /.notdef /.notdef /.notdef\n"
-	  "/.notdef /.notdef /.notdef /.notdef /.notdef /.notdef /.notdef /.notdef /.notdef /.notdef\n"
-	  "/.notdef /.notdef /space /exclam /quotedbl /numbersign /dollar /percent /ampersand /quoteright\n"
-	  "/parenleft /parenright /asterisk /plus /comma /hyphen /period /slash /zero /one\n"
-	  "/two /three /four /five /six /seven /eight /nine /colon /semicolon\n"
-	  "/less /equal /greater /question /at /A /B /C /D /E\n"
-	  "/F /G /H /I /J /K /L /M /N /O\n"
-	  "/P /Q /R /S /T /U /V /W /X /Y\n"
-	  "/Z /bracketleft /backslash /bracketright /asciicircum /underscore /quoteleft /a /b /c\n"
-	  "/d /e /f /g /h /i /j /k /l /m\n"
-	  "/n /o /p /q /r /s /t /u /v /w\n"
-	  "/x /y /z /braceleft /bar /braceright /asciitilde /.notdef /.notdef /.notdef\n"
-	  "/.notdef /.notdef /.notdef /.notdef /.notdef /.notdef /.notdef /.notdef /.notdef /.notdef\n"
-	  "/.notdef /.notdef /.notdef /.notdef /.notdef /.notdef /.notdef /.notdef /.notdef /.notdef\n"
-	  "/.notdef /.notdef /.notdef /.notdef /.notdef /.notdef /.notdef /.notdef /.notdef /.notdef\n"
-	  "/space /exclamdown /cent /sterling /currency /yen /brokenbar /section /dieresis /copyright\n"
-	  "/ordfeminine /guillemotleft /logicalnot /hyphen /registered /macron /degree /plusminus /twosuperior /threesuperior\n"
-	  "/acute /mu /paragraph /periodcentered /cedilla /onesuperior /ordmasculine /guillemotright /onequarter /onehalf\n"
-	  "/threequarters /questiondown /Agrave /Aacute /Acircumflex /Atilde /Adieresis /Aring /AE /Ccedilla\n"
-	  "/Egrave /Eacute /Ecircumflex /Edieresis /Igrave /Iacute /Icircumflex /Idieresis /Eth /Ntilde\n"
-	  "/Ograve /Oacute /Ocircumflex /Otilde /Odieresis /multiply /Oslash /Ugrave /Uacute /Ucircumflex\n"
-	  "/Udieresis /Yacute /Thorn /germandbls /agrave /aacute /acircumflex /atilde /adieresis /aring\n"
-	  "/ae /ccedilla /egrave /eacute /ecircumflex /edieresis /igrave /iacute /icircumflex /idieresis\n"
-	  "/eth /ntilde /ograve /oacute /ocircumflex /otilde /odieresis /divide /oslash /ugrave\n"
-	  "/uacute /ucircumflex /udieresis /yacute /thorn /ydieresis] /isolatin1encoding exch def\n");
-
-  print_reencode_font(file, "Times-Roman");
-  print_reencode_font(file, "Times-Italic");
-  print_reencode_font(file, "Times-Bold");
-  print_reencode_font(file, "Times-BoldItalic");
-  print_reencode_font(file, "AvantGarde-Book");
-  print_reencode_font(file, "AvantGarde-BookOblique");
-  print_reencode_font(file, "AvantGarde-Demi");
-  print_reencode_font(file, "AvantGarde-DemiOblique");
-  print_reencode_font(file, "Bookman-Light");
-  print_reencode_font(file, "Bookman-LightItalic");
-  print_reencode_font(file, "Bookman-Demi");
-  print_reencode_font(file, "Bookman-DemiItalic");
-  print_reencode_font(file, "Courier");
-  print_reencode_font(file, "Courier-Oblique");
-  print_reencode_font(file, "Courier-Bold");
-  print_reencode_font(file, "Courier-BoldOblique");
-  print_reencode_font(file, "Helvetica");
-  print_reencode_font(file, "Helvetica-Oblique");
-  print_reencode_font(file, "Helvetica-Bold");
-  print_reencode_font(file, "Helvetica-BoldOblique");
-  print_reencode_font(file, "Helvetica-Narrow");
-  print_reencode_font(file, "Helvetica-Narrow-Oblique");
-  print_reencode_font(file, "Helvetica-Narrow-Bold");
-  print_reencode_font(file, "Helvetica-Narrow-BoldOblique");
-  print_reencode_font(file, "NewCenturySchoolbook-Roman");
-  print_reencode_font(file, "NewCenturySchoolbook-Italic");
-  print_reencode_font(file, "NewCenturySchoolbook-Bold");
-  print_reencode_font(file, "NewCenturySchoolbook-BoldItalic");
-  print_reencode_font(file, "Palatino-Roman");
-  print_reencode_font(file, "Palatino-Italic");
-  print_reencode_font(file, "Palatino-Bold");
-  print_reencode_font(file, "Palatino-BoldItalic");
-  print_reencode_font(file, "Symbol");
-  print_reencode_font(file, "ZapfChancery-MediumItalic");
-  print_reencode_font(file, "ZapfDingbats");
-#endif /* !HAVE_UNICODE */
-
-  fprintf(file,
-	  "/cp {closepath} bind def\n"
-	  "/c {curveto} bind def\n"
-	  "/f {fill} bind def\n"
-	  "/a {arc} bind def\n"
-	  "/ef {eofill} bind def\n"
-	  "/ex {exch} bind def\n"
-	  "/gr {grestore} bind def\n"
-	  "/gs {gsave} bind def\n"
-	  "/sa {save} bind def\n"
-	  "/rs {restore} bind def\n"
-	  "/l {lineto} bind def\n"
-	  "/m {moveto} bind def\n"
-	  "/rm {rmoveto} bind def\n"
-	  "/n {newpath} bind def\n"
-	  "/s {stroke} bind def\n"
-	  "/sh {show} bind def\n"
-	  "/slc {setlinecap} bind def\n"
-	  "/slj {setlinejoin} bind def\n"
-	  "/slw {setlinewidth} bind def\n"
-	  "/srgb {setrgbcolor} bind def\n"
-	  "/rot {rotate} bind def\n"
-	  "/sc {scale} bind def\n"
-	  "/sd {setdash} bind def\n"
-	  "/ff {findfont} bind def\n"
-	  "/sf {setfont} bind def\n"
-	  "/scf {scalefont} bind def\n"
-	  "/sw {stringwidth pop} bind def\n"
-	  "/tr {translate} bind def\n"
-
-	  "\n/ellipsedict 8 dict def\n"
-	  "ellipsedict /mtrx matrix put\n"
-	  "/ellipse\n"
-	  "{ ellipsedict begin\n"
-          "   /endangle exch def\n"
-          "   /startangle exch def\n"
-          "   /yrad exch def\n"
-          "   /xrad exch def\n"
-          "   /y exch def\n"
-          "   /x exch def"
-	  "   /savematrix mtrx currentmatrix def\n"
-          "   x y tr xrad yrad sc\n"
-          "   0 0 1 startangle endangle arc\n"
-          "   savematrix setmatrix\n"
-          "   end\n"
-	  "} def\n\n"
-
-	  /*
-	  "/colortogray {\n"
-	  "/rgbdata exch store\n"
-	  "rgbdata length 3 idiv\n"
-	  "/npixls exch store\n"
-	  "/rgbindx 0 store\n"
-	  "0 1 npixls 1 sub {\n"
-	  "grays exch\n"
-	  "rgbdata rgbindx       get 20 mul\n"
-	  "rgbdata rgbindx 1 add get 32 mul\n"
-	  "rgbdata rgbindx 2 add get 12 mul\n"
-	  "add add 64 idiv\n"
-	  "put\n"
-	  "/rgbindx rgbindx 3 add store\n"
-	  "} for\n"
-	  "grays 0 npixls getinterval\n"
-	  "} bind def\n"
-	  */
-	  "/mergeprocs {\n"
-	  "dup length\n"
-	  "3 -1 roll\n"
-	  "dup\n"
-	  "length\n"
-	  "dup\n"
-	  "5 1 roll\n"
-	  "3 -1 roll\n"
-	  "add\n"
-	  "array cvx\n"
-	  "dup\n"
-	  "3 -1 roll\n"
-	  "0 exch\n"
-	  "putinterval\n"
-	  "dup\n"
-	  "4 2 roll\n"
-	  "putinterval\n"
-	  "} bind def\n"
-	  /*	  
-	  "/colorimage {\n"
-	  "pop pop\n"
-	  "{colortogray} mergeprocs\n"
-	  "image\n"
-	  "} bind def\n\n"
-	  */
-	  "%f %f scale\n"
-	  "%f %f translate\n"
-	  "%%%%EndProlog\n\n\n",
-	  scale, -scale,
-	  -extent->left, -extent->bottom );
 #ifdef HAVE_UNICODE
   renderer->psu = ps_unicoder_new(&eps_unicoder_callbacks,(gpointer)renderer);
 #endif 
@@ -435,12 +820,13 @@ new_psprint_renderer(Diagram *dia, FILE *file)
 {
   RendererEPS *renderer;
   time_t time_now;
-  double scale;
-  Rectangle *extent;
   char *name;
 
+  if (EpsPrologOps == NULL)
+    init_eps_renderer();
+
   renderer = g_new(RendererEPS, 1);
-  renderer->renderer.ops = &EpsRenderOps;
+  renderer->renderer.ops = EpsPrologOps;
   renderer->renderer.is_interactive = 0;
   renderer->renderer.interactive_ops = NULL;
 
@@ -454,10 +840,7 @@ new_psprint_renderer(Diagram *dia, FILE *file)
   renderer->saved_line_style = LINESTYLE_SOLID;
   
   time_now  = time(NULL);
-  extent = &dia->data->extents;
-  
-  scale = 28.346;
-  
+
   name = getlogin();
   if (name==NULL)
     name = "a user";
@@ -470,8 +853,6 @@ new_psprint_renderer(Diagram *dia, FILE *file)
 	  "%%%%For: %s\n"
 	  "%%%%DocumentPaperSizes: %s\n"
 	  "%%%%Orientation: %s\n"
-	  "%%%%BeginSetup\n"
-	  "%%%%EndSetup\n"
 	  "%%%%EndComments\n",
 	  dia->filename,
 	  VERSION,
@@ -479,165 +860,6 @@ new_psprint_renderer(Diagram *dia, FILE *file)
 	  name,
 	  dia->data->paper.name,
 	  dia->data->paper.is_portrait ? "Portrait" : "Landscape");
-
-  fprintf(file, "%%%%BeginProlog\n");
-
-#ifndef HAVE_UNICODE
-  fprintf(file,
-	  "[ /.notdef /.notdef /.notdef /.notdef /.notdef /.notdef /.notdef /.notdef /.notdef /.notdef\n"
-	  "/.notdef /.notdef /.notdef /.notdef /.notdef /.notdef /.notdef /.notdef /.notdef /.notdef\n"
-	  "/.notdef /.notdef /.notdef /.notdef /.notdef /.notdef /.notdef /.notdef /.notdef /.notdef\n"
-	  "/.notdef /.notdef /space /exclam /quotedbl /numbersign /dollar /percent /ampersand /quoteright\n"
-	  "/parenleft /parenright /asterisk /plus /comma /hyphen /period /slash /zero /one\n"
-	  "/two /three /four /five /six /seven /eight /nine /colon /semicolon\n"
-	  "/less /equal /greater /question /at /A /B /C /D /E\n"
-	  "/F /G /H /I /J /K /L /M /N /O\n"
-	  "/P /Q /R /S /T /U /V /W /X /Y\n"
-	  "/Z /bracketleft /backslash /bracketright /asciicircum /underscore /quoteleft /a /b /c\n"
-	  "/d /e /f /g /h /i /j /k /l /m\n"
-	  "/n /o /p /q /r /s /t /u /v /w\n"
-	  "/x /y /z /braceleft /bar /braceright /asciitilde /.notdef /.notdef /.notdef\n"
-	  "/.notdef /.notdef /.notdef /.notdef /.notdef /.notdef /.notdef /.notdef /.notdef /.notdef\n"
-	  "/.notdef /.notdef /.notdef /.notdef /.notdef /.notdef /.notdef /.notdef /.notdef /.notdef\n"
-	  "/.notdef /.notdef /.notdef /.notdef /.notdef /.notdef /.notdef /.notdef /.notdef /.notdef\n"
-	  "/space /exclamdown /cent /sterling /currency /yen /brokenbar /section /dieresis /copyright\n"
-	  "/ordfeminine /guillemotleft /logicalnot /hyphen /registered /macron /degree /plusminus /twosuperior /threesuperior\n"
-	  "/acute /mu /paragraph /periodcentered /cedilla /onesuperior /ordmasculine /guillemotright /onequarter /onehalf\n"
-	  "/threequarters /questiondown /Agrave /Aacute /Acircumflex /Atilde /Adieresis /Aring /AE /Ccedilla\n"
-	  "/Egrave /Eacute /Ecircumflex /Edieresis /Igrave /Iacute /Icircumflex /Idieresis /Eth /Ntilde\n"
-	  "/Ograve /Oacute /Ocircumflex /Otilde /Odieresis /multiply /Oslash /Ugrave /Uacute /Ucircumflex\n"
-	  "/Udieresis /Yacute /Thorn /germandbls /agrave /aacute /acircumflex /atilde /adieresis /aring\n"
-	  "/ae /ccedilla /egrave /eacute /ecircumflex /edieresis /igrave /iacute /icircumflex /idieresis\n"
-	  "/eth /ntilde /ograve /oacute /ocircumflex /otilde /odieresis /divide /oslash /ugrave\n"
-	  "/uacute /ucircumflex /udieresis /yacute /thorn /ydieresis] /isolatin1encoding exch def\n");
-
-  print_reencode_font(file, "Times-Roman");
-  print_reencode_font(file, "Times-Italic");
-  print_reencode_font(file, "Times-Bold");
-  print_reencode_font(file, "Times-BoldItalic");
-  print_reencode_font(file, "AvantGarde-Book");
-  print_reencode_font(file, "AvantGarde-BookOblique");
-  print_reencode_font(file, "AvantGarde-Demi");
-  print_reencode_font(file, "AvantGarde-DemiOblique");
-  print_reencode_font(file, "Bookman-Light");
-  print_reencode_font(file, "Bookman-LightItalic");
-  print_reencode_font(file, "Bookman-Demi");
-  print_reencode_font(file, "Bookman-DemiItalic");
-  print_reencode_font(file, "Courier");
-  print_reencode_font(file, "Courier-Oblique");
-  print_reencode_font(file, "Courier-Bold");
-  print_reencode_font(file, "Courier-BoldOblique");
-  print_reencode_font(file, "Helvetica");
-  print_reencode_font(file, "Helvetica-Oblique");
-  print_reencode_font(file, "Helvetica-Bold");
-  print_reencode_font(file, "Helvetica-BoldOblique");
-  print_reencode_font(file, "Helvetica-Narrow");
-  print_reencode_font(file, "Helvetica-Narrow-Oblique");
-  print_reencode_font(file, "Helvetica-Narrow-Bold");
-  print_reencode_font(file, "Helvetica-Narrow-BoldOblique");
-  print_reencode_font(file, "NewCenturySchoolbook-Roman");
-  print_reencode_font(file, "NewCenturySchoolbook-Italic");
-  print_reencode_font(file, "NewCenturySchoolbook-Bold");
-  print_reencode_font(file, "NewCenturySchoolbook-BoldItalic");
-  print_reencode_font(file, "Palatino-Roman");
-  print_reencode_font(file, "Palatino-Italic");
-  print_reencode_font(file, "Palatino-Bold");
-  print_reencode_font(file, "Palatino-BoldItalic");
-  print_reencode_font(file, "Symbol");
-  print_reencode_font(file, "ZapfChancery-MediumItalic");
-  print_reencode_font(file, "ZapfDingbats");
-#endif /* !HAVE_UNICODE */
-  fprintf(file,
-	  "/cp {closepath} bind def\n"
-	  "/c {curveto} bind def\n"
-	  "/f {fill} bind def\n"
-	  "/a {arc} bind def\n"
-	  "/ef {eofill} bind def\n"
-	  "/ex {exch} bind def\n"
-	  "/gr {grestore} bind def\n"
-	  "/gs {gsave} bind def\n"
-	  "/sa {save} bind def\n"
-	  "/rs {restore} bind def\n"
-	  "/l {lineto} bind def\n"
-	  "/m {moveto} bind def\n"
-	  "/rm {rmoveto} bind def\n"
-	  "/n {newpath} bind def\n"
-	  "/s {stroke} bind def\n"
-	  "/sh {show} bind def\n"
-	  "/slc {setlinecap} bind def\n"
-	  "/slj {setlinejoin} bind def\n"
-	  "/slw {setlinewidth} bind def\n"
-	  "/srgb {setrgbcolor} bind def\n"
-	  "/rot {rotate} bind def\n"
-	  "/sc {scale} bind def\n"
-	  "/sd {setdash} bind def\n"
-	  "/ff {findfont} bind def\n"
-	  "/sf {setfont} bind def\n"
-	  "/scf {scalefont} bind def\n"
-	  "/sw {stringwidth pop} bind def\n"
-	  "/tr {translate} bind def\n"
-
-	  "\n/ellipsedict 8 dict def\n"
-	  "ellipsedict /mtrx matrix put\n"
-	  "/ellipse\n"
-	  "{ ellipsedict begin\n"
-          "   /endangle exch def\n"
-          "   /startangle exch def\n"
-          "   /yrad exch def\n"
-          "   /xrad exch def\n"
-          "   /y exch def\n"
-          "   /x exch def"
-	  "   /savematrix mtrx currentmatrix def\n"
-          "   x y tr xrad yrad sc\n"
-          "   0 0 1 startangle endangle arc\n"
-          "   savematrix setmatrix\n"
-          "   end\n"
-	  "} def\n\n"
-
-	  /*
-	  "/colortogray {\n"
-	  "/rgbdata exch store\n"
-	  "rgbdata length 3 idiv\n"
-	  "/npixls exch store\n"
-	  "/rgbindx 0 store\n"
-	  "0 1 npixls 1 sub {\n"
-	  "grays exch\n"
-	  "rgbdata rgbindx       get 20 mul\n"
-	  "rgbdata rgbindx 1 add get 32 mul\n"
-	  "rgbdata rgbindx 2 add get 12 mul\n"
-	  "add add 64 idiv\n"
-	  "put\n"
-	  "/rgbindx rgbindx 3 add store\n"
-	  "} for\n"
-	  "grays 0 npixls getinterval\n"
-	  "} bind def\n"
-	  */
-	  "/mergeprocs {\n"
-	  "dup length\n"
-	  "3 -1 roll\n"
-	  "dup\n"
-	  "length\n"
-	  "dup\n"
-	  "5 1 roll\n"
-	  "3 -1 roll\n"
-	  "add\n"
-	  "array cvx\n"
-	  "dup\n"
-	  "3 -1 roll\n"
-	  "0 exch\n"
-	  "putinterval\n"
-	  "dup\n"
-	  "4 2 roll\n"
-	  "putinterval\n"
-	  "} bind def\n"
-	  /*
-	  "/colorimage {\n"
-	  "pop pop\n"
-	  "{colortogray} mergeprocs\n"
-	  "image\n"
-	  "} bind def\n\n"
-	  */
-	  "%%%%EndProlog\n\n\n");
   
 #ifdef HAVE_UNICODE
   renderer->psu = ps_unicoder_new(&eps_unicoder_callbacks,(gpointer)renderer);
@@ -646,7 +868,7 @@ new_psprint_renderer(Diagram *dia, FILE *file)
 }
 
 static void
-begin_render(RendererEPS *renderer, DiagramData *data)
+begin_render(RendererEPS *renderer)
 {
 }
 
@@ -1077,7 +1299,8 @@ eps_select_ps_font(gpointer usrdata, const gchar *fontname, float size )
 static void eps_show_string(gpointer usrdata, const gchar *string)
 {
   RendererEPS *renderer = (RendererEPS *)usrdata;
-  /* string has nothing we would have to escape. */
+  /* string has nothing we would have to escape (the custom encoding skips
+   the characters we have to escape). */
   fprintf(renderer->file, "(%s)\n", string);  
 }
                   
@@ -1095,7 +1318,7 @@ static void eps_get_string_width(gpointer usrdata, const gchar *string,
                   
 
 static void
-set_font(RendererEPS *renderer, Font *font, real height)
+set_font(RendererEPS *renderer, DiaFont *font, real height)
 {
   psu_set_font_face(renderer->psu,
                     font_get_psfontname(font), 
@@ -1104,7 +1327,7 @@ set_font(RendererEPS *renderer, Font *font, real height)
 
 static void
 draw_string(RendererEPS *renderer,
-	    const char *text,
+	    const utfchar *text,
 	    Point *pos, Alignment alignment,
 	    Color *color)
 {
@@ -1115,17 +1338,26 @@ draw_string(RendererEPS *renderer,
 
   lazy_setcolor(renderer,color);
 
-  /* <FIXME:> dia doesn't talk UTF-8 but the local charset (in fact, it 
-     doesn't know what it talks, and assumes it's latin-1). The PSUnicoder
+#ifndef UNICODE_WORK_IN_PROGRESS
+  /* <FIXME:> dia doesn't talk UTF-8 but the local charset. The PSUnicoder
      talks UTF-8. We do a quick-and-dirty translation for now. */
   utf8_buffer = charconv_local8_to_utf8(text);
   /* </FIXME> */
-  utf8_len = strlen(utf8_buffer); /* deliberate */
+#else
+  utf8_buffer=text;
+#endif
 
+  utf8_len = strlen(utf8_buffer); /* deliberate */
+  
   if (utf8_len <= 0) {
+#ifndef UNICODE_WORK_IN_PROGRESS
+    g_free(utf8_buffer);
+#endif
     return; /* null string -> we won't display anything 
                and we will crash the Postscript stack. */
   }
+  /* normally not needed (if predraw_string() has been called beforehand),
+     but harmless : */
   psu_check_string_encodings(renderer->psu,utf8_buffer);
   
   switch (alignment) {
@@ -1149,17 +1381,17 @@ draw_string(RendererEPS *renderer,
 
   psu_show_string(renderer->psu,utf8_buffer);
   
+#ifndef UNICODE_WORK_IN_PROGRESS
   g_free(utf8_buffer);
+#endif
   
-  if (utf8_len>0) {        /* always true, normally. */
-    fprintf(renderer->file, " gs 1 -1 sc sh gr\n");
-  }
+  fprintf(renderer->file, " gs 1 -1 sc sh gr\n");
 }
 
 #else /* !HAVE_UNICODE*/
 
 static void
-set_font(RendererEPS *renderer, Font *font, real height)
+set_font(RendererEPS *renderer, DiaFont *font, real height)
 {
   fprintf(renderer->file, "/%s-latin1 ff %f scf sf\n",
 	  font_get_psfontname(font), (double)height);
@@ -1588,6 +1820,9 @@ export_eps(DiagramData *data, const gchar *filename,
 
   old_locale = setlocale(LC_NUMERIC, "C");
   if ((renderer = create_eps_renderer(data, filename, diafilename))) {
+    data_render(data, (Renderer *)renderer, NULL, NULL, NULL);
+    eps_renderer_prolog_done(renderer);
+    eps_renderer_set_scale(data, renderer);
     data_render(data, (Renderer *)renderer, NULL, NULL, NULL);
     destroy_eps_renderer(renderer);
   }
