@@ -19,6 +19,8 @@
 #include <config.h>
 
 #include <math.h>
+#include <stdlib.h>
+#include <string.h>
 #include <gdk/gdk.h>
 
 #include "render_gdk.h"
@@ -32,7 +34,7 @@ static void set_linejoin(RendererGdk *renderer, LineJoin mode);
 static void set_linestyle(RendererGdk *renderer, LineStyle mode);
 static void set_dashlength(RendererGdk *renderer, real length);
 static void set_fillstyle(RendererGdk *renderer, FillStyle mode);
-static void set_font(RendererGdk *renderer, Font *font, real height);
+static void set_font(RendererGdk *renderer, DiaFont *font, real height);
 static void draw_line(RendererGdk *renderer, 
 		      Point *start, Point *end, 
 		      Color *line_color);
@@ -78,7 +80,7 @@ static void fill_bezier(RendererGdk *renderer,
 			int numpoints,
 			Color *color);
 static void draw_string(RendererGdk *renderer,
-			const char *text,
+			const utfchar *text,
 			Point *pos, Alignment alignment,
 			Color *color);
 static void draw_image(RendererGdk *renderer,
@@ -106,40 +108,7 @@ static void fill_pixel_rect(RendererGdk *renderer,
 				 int width, int height,
 				 Color *color);
 
-static RenderOps GdkRenderOps = {
-  (BeginRenderFunc) begin_render,
-  (EndRenderFunc) end_render,
-
-  (SetLineWidthFunc) set_linewidth,
-  (SetLineCapsFunc) set_linecaps,
-  (SetLineJoinFunc) set_linejoin,
-  (SetLineStyleFunc) set_linestyle,
-  (SetDashLengthFunc) set_dashlength,
-  (SetFillStyleFunc) set_fillstyle,
-  (SetFontFunc) set_font,
-  
-  (DrawLineFunc) draw_line,
-  (DrawPolyLineFunc) draw_polyline,
-  
-  (DrawPolygonFunc) draw_polygon,
-  (FillPolygonFunc) fill_polygon,
-
-  (DrawRectangleFunc) draw_rect,
-  (FillRectangleFunc) fill_rect,
-
-  (DrawArcFunc) draw_arc,
-  (FillArcFunc) fill_arc,
-
-  (DrawEllipseFunc) draw_ellipse,
-  (FillEllipseFunc) fill_ellipse,
-
-  (DrawBezierFunc) draw_bezier,
-  (FillBezierFunc) fill_bezier,
-
-  (DrawStringFunc) draw_string,
-
-  (DrawImageFunc) draw_image,
-};
+static void init_gdk_renderops();
 
 static InteractiveRenderOps GdkInteractiveRenderOps = {
   (GetTextWidthFunc) get_text_width,
@@ -152,13 +121,56 @@ static InteractiveRenderOps GdkInteractiveRenderOps = {
   (FillPixelRectangleFunc) fill_pixel_rect,
 };
 
+static RenderOps *GdkRenderOps;
+
+static void
+init_gdk_renderops() {
+  GdkRenderOps = create_renderops_table();
+
+  GdkRenderOps->begin_render = (BeginRenderFunc) begin_render;
+  GdkRenderOps->end_render = (EndRenderFunc) end_render;
+
+  GdkRenderOps->set_linewidth = (SetLineWidthFunc) set_linewidth;
+  GdkRenderOps->set_linecaps = (SetLineCapsFunc) set_linecaps;
+  GdkRenderOps->set_linejoin = (SetLineJoinFunc) set_linejoin;
+  GdkRenderOps->set_linestyle = (SetLineStyleFunc) set_linestyle;
+  GdkRenderOps->set_dashlength = (SetDashLengthFunc) set_dashlength;
+  GdkRenderOps->set_fillstyle = (SetFillStyleFunc) set_fillstyle;
+  GdkRenderOps->set_font = (SetFontFunc) set_font;
+  
+  GdkRenderOps->draw_line = (DrawLineFunc) draw_line;
+  GdkRenderOps->draw_polyline = (DrawPolyLineFunc) draw_polyline;
+  
+  GdkRenderOps->draw_polygon = (DrawPolygonFunc) draw_polygon;
+  GdkRenderOps->fill_polygon = (FillPolygonFunc) fill_polygon;
+
+  GdkRenderOps->draw_rect = (DrawRectangleFunc) draw_rect;
+  GdkRenderOps->fill_rect = (FillRectangleFunc) fill_rect;
+
+  GdkRenderOps->draw_arc = (DrawArcFunc) draw_arc;
+  GdkRenderOps->fill_arc = (FillArcFunc) fill_arc;
+
+  GdkRenderOps->draw_ellipse = (DrawEllipseFunc) draw_ellipse;
+  GdkRenderOps->fill_ellipse = (FillEllipseFunc) fill_ellipse;
+
+  GdkRenderOps->draw_bezier = (DrawBezierFunc) draw_bezier;
+  GdkRenderOps->fill_bezier = (FillBezierFunc) fill_bezier;
+
+  GdkRenderOps->draw_string = (DrawStringFunc) draw_string;
+
+  GdkRenderOps->draw_image = (DrawImageFunc) draw_image;
+}
+
 RendererGdk *
 new_gdk_renderer(DDisplay *ddisp)
 {
   RendererGdk *renderer;
 
+  if (GdkRenderOps == NULL)
+    init_gdk_renderops();
+
   renderer = g_new(RendererGdk, 1);
-  renderer->renderer.ops = &GdkRenderOps;
+  renderer->renderer.ops = GdkRenderOps;
   renderer->renderer.is_interactive = 1;
   renderer->renderer.interactive_ops = &GdkInteractiveRenderOps;
   renderer->ddisp = ddisp;
@@ -399,12 +411,16 @@ set_fillstyle(RendererGdk *renderer, FillStyle mode)
 }
 
 static void
-set_font(RendererGdk *renderer, Font *font, real height)
+set_font(RendererGdk *renderer, DiaFont *font, real height)
 {
   renderer->font_height =
     ddisplay_transform_length(renderer->ddisp, height);
 
+#ifdef HAVE_FREETYPE
+  renderer->freetype_font = font_get_freetypefont(font, renderer->font_height);
+#else
   renderer->gdk_font = font_get_gdkfont(font, renderer->font_height);
+#endif
 }
 
 static void
@@ -892,22 +908,59 @@ fill_bezier(RendererGdk *renderer,
 		   bezier.gdk_points, bezier.currpoint);
 }
 
-static void
-draw_string(RendererGdk *renderer,
-	    const char *text,
-	    Point *pos, Alignment alignment,
-	    Color *color)
+struct gdk_freetype_user_data {
+  GdkPixmap *pixmap;
+  GdkGC *gc;
+};
+
+#ifdef HAVE_FREETYPE
+void
+gdk_freetype_copy_glyph(FT_GlyphSlot glyph, int pen_x, int pen_y,
+			gpointer userdata)
 {
+  struct gdk_freetype_user_data *data = 
+    (struct gdk_freetype_user_data *)userdata;
+  FT_Bitmap *bitmap = &glyph->bitmap;
+  guchar *buffer = bitmap->buffer;
+  int rowstride = bitmap->pitch;
+
+  /* Seems FT and GDK disagree on what color '0' is, so we have to */
+  /* swap the foreground and background colors. */
+  if (rowstride < 0) { /* Cartesian bitmap */
+    buffer = buffer+rowstride*(bitmap->rows-1);
+  }
+
+  if (bitmap->pixel_mode == ft_pixel_mode_mono) {
+    
+  } else {
+    gdk_draw_gray_image(data->pixmap, data->gc, pen_x, pen_y,
+			bitmap->width, bitmap->rows,
+			GDK_RGB_DITHER_NONE,
+			buffer, rowstride);
+  }
+}
+#endif
+
+static void
+draw_string (RendererGdk *renderer,
+	     const utfchar *text,
+	     Point *pos, Alignment alignment,
+	     Color *color)
+{
+#ifdef HAVE_FREETYPE
   DDisplay *ddisp = renderer->ddisp;
   GdkGC *gc = renderer->render_gc;
-  GdkColor gdkcolor;
   int x,y;
   int iwidth;
-  
-  ddisplay_transform_coords(ddisp, pos->x, pos->y,
-			    &x, &y);
-  
-  iwidth = gdk_string_width(renderer->gdk_font, text);
+  FreetypeString *fts;
+  struct gdk_freetype_user_data userdata;
+  GdkColor gdkcolor;
+
+  gdk_gc_set_function(renderer->render_gc, GDK_COPY_INVERT);
+
+  ddisplay_transform_coords(ddisp, pos->x, pos->y, &x, &y);
+  fts = freetype_load_string(text, renderer->freetype_font, strlen(text));
+  iwidth = fts->width;
 
   switch (alignment) {
   case ALIGN_LEFT:
@@ -922,10 +975,83 @@ draw_string(RendererGdk *renderer,
   
   color_convert(color, &gdkcolor);
   gdk_gc_set_foreground(gc, &gdkcolor);
-      
+
+  userdata.pixmap = renderer->pixmap;
+  userdata.gc = gc;
+
+  freetype_render_string(fts, x, y, gdk_freetype_copy_glyph, &userdata);
+  gdk_gc_set_function(renderer->render_gc, GDK_COPY);
+#else
+  DDisplay *ddisp = renderer->ddisp;
+  GdkGC *gc = renderer->render_gc;
+  GdkColor gdkcolor;
+  int x,y;
+  int iwidth;
+  GdkWChar *wcstr;
+  gchar *str, *mbstr;
+  int length, wclength;
+  int i;
+  
+  ddisplay_transform_coords(ddisp, pos->x, pos->y,
+			    &x, &y);
+
+# ifdef GTK_DOESNT_TALK_UTF8_WE_DO
+  str = charconv_utf8_to_local8 (text);
+  length = strlen (str);
+  wcstr = g_new0 (GdkWChar, length + 1);
+  /* Despite the manpage, mbstowcs is *not* const wrt the src. */
+  mbstr = g_strdup(str);
+  wclength = mbstowcs (wcstr, mbstr, length);
+  g_free(mbstr);
+
+  if (wclength > 0) {
+	  length = wclength;
+  } else {
+	  for (i = 0; i < length; i++) {
+		  wcstr[i] = (unsigned char) str[i];
+	  }
+  }
+  iwidth = gdk_text_width_wc (renderer->gdk_font, wcstr, length);
+# else /* both talk the same */
+  iwidth = gdk_string_width(renderer->gdk_font, text);
+# endif /* GTK_DOESNT_TALK_UTF8_WE_DO */
+
+  switch (alignment) {
+  case ALIGN_LEFT:
+    break;
+  case ALIGN_CENTER:
+    x -= iwidth/2;
+    break;
+  case ALIGN_RIGHT:
+    x -= iwidth;
+    break;
+  }
+  
+  color_convert(color, &gdkcolor);
+  gdk_gc_set_foreground(gc, &gdkcolor);
+
+# ifdef GTK_DOESNT_TALK_UTF8_WE_DO
+  gdk_draw_text_wc (renderer->pixmap,
+		    renderer->gdk_font, gc,
+		    x, y, wcstr, length);
+  g_free (wcstr);
+  g_free (str);
+# else
+#  if defined (GTK_TALKS_UTF8_WE_DONT)
+  {
+    utfchar *utfbuf = charconv_local8_to_utf8(text);
+    gdk_draw_string(renderer->pixmap,
+		    renderer->gdk_font, gc,
+		    x,y, utfbuf);
+    g_free(utfbuf);
+  }
+#  else
   gdk_draw_string(renderer->pixmap,
 		  renderer->gdk_font, gc,
 		  x,y, text);
+#  endif
+#endif /* GTK_DOESNT_TALK_UTF8_WE_DO */
+#endif
 }
 
 static void
@@ -945,14 +1071,61 @@ draw_image(RendererGdk *renderer,
 		 real_width, real_height);
 }
 
+/* Get the width of the given text in cm */
 static real
 get_text_width(RendererGdk *renderer,
-	       const char *text, int length)
+	       const utfchar *text, int length)
 {
   int iwidth;
-  
-  iwidth = gdk_text_width(renderer->gdk_font, text, length);
+  GdkWChar *wcstr;
+  gchar *mbstr, *str;
+  int len, wclength, i;
+  utfchar *utfbuf, *utf, *p;
 
+#ifdef HAVE_FREETYPE
+  iwidth = freetype_load_string(text, renderer->freetype_font, length)->width;
+#elif defined (GTK_TALKS_UTF8) && defined (UNICODE_WORK_IN_PROGRESS)
+  /* length is in num glyphs, we need bytes here */
+  p = text;
+  for (i = 0; i < length; i++)
+    p = g_utf8_next_char (p);
+  iwidth = gdk_text_width(renderer->gdk_font, text, p - text);
+#else
+# if defined UNICODE_WORK_IN_PROGRESS
+  p = utfbuf = g_strdup (text);
+  for (i = 0; i < length; i++)
+	  p = uni_next (p);
+
+  utf = g_new (utfchar, p - utfbuf + 1);
+  strncpy (utf, text, p - utfbuf);
+  utf[p - utfbuf] = 0;
+  str = charconv_utf8_to_local8 (utf);
+  len = strlen (str);
+
+  g_free (utfbuf);
+  g_free (utf);
+#  else
+  str = g_strdup (text);
+  len = length;
+#  endif
+  wcstr = g_new0 (GdkWChar, len + 1);
+  mbstr = g_strdup(str);
+  wclength = mbstowcs (wcstr, mbstr, len);
+  g_free(mbstr);
+
+  if (wclength > 0) {
+    len = wclength;
+  } else {
+    for (i = 0; i < len; i++) {
+      wcstr[i] = (unsigned char) str[i];
+    }
+  }
+  g_free (str);
+
+  iwidth = gdk_text_width_wc (renderer->gdk_font, wcstr, len);
+
+  g_free (wcstr);
+#endif
   return ddisplay_untransform_length(renderer->ddisp, (real) iwidth);
 }
 
