@@ -24,6 +24,7 @@
 
 #include "render_libart.h"
 #include "message.h"
+#include "font.h"
 
 #ifdef HAVE_LIBART
 
@@ -47,7 +48,7 @@ static void set_linejoin(RendererLibart *renderer, LineJoin mode);
 static void set_linestyle(RendererLibart *renderer, LineStyle mode);
 static void set_dashlength(RendererLibart *renderer, real length);
 static void set_fillstyle(RendererLibart *renderer, FillStyle mode);
-static void set_font(RendererLibart *renderer, Font *font, real height);
+static void set_font(RendererLibart *renderer, DiaFont *font, real height);
 static void draw_line(RendererLibart *renderer, 
 		      Point *start, Point *end, 
 		      Color *line_color);
@@ -93,7 +94,7 @@ static void fill_bezier(RendererLibart *renderer,
 			int numpoints,
 			Color *color);
 static void draw_string(RendererLibart *renderer,
-			const char *text,
+			const utfchar *text,
 			Point *pos, Alignment alignment,
 			Color *color);
 static void draw_image(RendererLibart *renderer,
@@ -102,7 +103,7 @@ static void draw_image(RendererLibart *renderer,
 		       DiaImage image);
 
 static real get_text_width(RendererLibart *renderer,
-			   const char *text, int length);
+			   const utfchar *text, int length);
 
 static void clip_region_clear(RendererLibart *renderer);
 static void clip_region_add_rect(RendererLibart *renderer,
@@ -121,40 +122,7 @@ static void fill_pixel_rect(RendererLibart *renderer,
 				 int width, int height,
 				 Color *color);
 
-static RenderOps LibartRenderOps = {
-  (BeginRenderFunc) begin_render,
-  (EndRenderFunc) end_render,
-
-  (SetLineWidthFunc) set_linewidth,
-  (SetLineCapsFunc) set_linecaps,
-  (SetLineJoinFunc) set_linejoin,
-  (SetLineStyleFunc) set_linestyle,
-  (SetDashLengthFunc) set_dashlength,
-  (SetFillStyleFunc) set_fillstyle,
-  (SetFontFunc) set_font,
-  
-  (DrawLineFunc) draw_line,
-  (DrawPolyLineFunc) draw_polyline,
-  
-  (DrawPolygonFunc) draw_polygon,
-  (FillPolygonFunc) fill_polygon,
-
-  (DrawRectangleFunc) draw_rect,
-  (FillRectangleFunc) fill_rect,
-
-  (DrawArcFunc) draw_arc,
-  (FillArcFunc) fill_arc,
-
-  (DrawEllipseFunc) draw_ellipse,
-  (FillEllipseFunc) fill_ellipse,
-
-  (DrawBezierFunc) draw_bezier,
-  (FillBezierFunc) fill_bezier,
-
-  (DrawStringFunc) draw_string,
-
-  (DrawImageFunc) draw_image,
-};
+static void init_libart_renderer();
 
 static InteractiveRenderOps LibartInteractiveRenderOps = {
   (GetTextWidthFunc) get_text_width,
@@ -167,13 +135,57 @@ static InteractiveRenderOps LibartInteractiveRenderOps = {
   (FillPixelRectangleFunc) fill_pixel_rect,
 };
 
+static RenderOps *LibartRenderOps;
+
+/* Here we set the functions that we define for this renderer. */
+static void
+init_libart_renderer() {
+  LibartRenderOps = create_renderops_table();
+  
+  LibartRenderOps->begin_render = (BeginRenderFunc) begin_render;
+  LibartRenderOps->end_render = (EndRenderFunc) end_render;
+
+  LibartRenderOps->set_linewidth = (SetLineWidthFunc) set_linewidth;
+  LibartRenderOps->set_linecaps = (SetLineCapsFunc) set_linecaps;
+  LibartRenderOps->set_linejoin = (SetLineJoinFunc) set_linejoin;
+  LibartRenderOps->set_linestyle = (SetLineStyleFunc) set_linestyle;
+  LibartRenderOps->set_dashlength = (SetDashLengthFunc) set_dashlength;
+  LibartRenderOps->set_fillstyle = (SetFillStyleFunc) set_fillstyle;
+  LibartRenderOps->set_font = (SetFontFunc) set_font;
+  
+  LibartRenderOps->draw_line = (DrawLineFunc) draw_line;
+  LibartRenderOps->draw_polyline = (DrawPolyLineFunc) draw_polyline;
+  
+  LibartRenderOps->draw_polygon = (DrawPolygonFunc) draw_polygon;
+  LibartRenderOps->fill_polygon = (FillPolygonFunc) fill_polygon;
+
+  LibartRenderOps->draw_rect = (DrawRectangleFunc) draw_rect;
+  LibartRenderOps->fill_rect = (FillRectangleFunc) fill_rect;
+
+  LibartRenderOps->draw_arc = (DrawArcFunc) draw_arc;
+  LibartRenderOps->fill_arc = (FillArcFunc) fill_arc;
+
+  LibartRenderOps->draw_ellipse = (DrawEllipseFunc) draw_ellipse;
+  LibartRenderOps->fill_ellipse = (FillEllipseFunc) fill_ellipse;
+
+  LibartRenderOps->draw_bezier = (DrawBezierFunc) draw_bezier;
+  LibartRenderOps->fill_bezier = (FillBezierFunc) fill_bezier;
+
+  LibartRenderOps->draw_string = (DrawStringFunc) draw_string;
+
+  LibartRenderOps->draw_image = (DrawImageFunc) draw_image;
+}
+
 RendererLibart *
 new_libart_renderer(DDisplay *ddisp, int interactive)
 {
   RendererLibart *renderer;
 
+  if (LibartRenderOps == NULL)
+    init_libart_renderer();
+
   renderer = g_new(RendererLibart, 1);
-  renderer->renderer.ops = &LibartRenderOps;
+  renderer->renderer.ops = LibartRenderOps;
   renderer->renderer.is_interactive = interactive;
   renderer->renderer.interactive_ops = &LibartInteractiveRenderOps;
   renderer->ddisp = ddisp;
@@ -191,8 +203,12 @@ new_libart_renderer(DDisplay *ddisp, int interactive)
   renderer->dash_length = 10;
   renderer->dot_length = 1;
 
+#ifdef HAVE_FREETYPE
+  renderer->freetype_font = NULL;
+#else
   renderer->gdk_font = NULL;
   renderer->suck_font = NULL;
+#endif
 
   return renderer;
 }
@@ -249,6 +265,18 @@ renderer_libart_copy_to_window(RendererLibart *renderer, GdkWindow *window,
 		     w*3);
 }
 
+static guint32 color_to_abgr(Color *col)
+{
+  int rgba;
+
+  rgba = 0x0;
+  rgba |= (guint)(0xFF*col->blue) << 16;
+  rgba |= (guint)(0xFF*col->green) << 8;
+  rgba |= (guint)(0xFF*col->red);
+  
+  return rgba;
+}
+
 static guint32 color_to_rgba(Color *col)
 {
   int rgba;
@@ -402,18 +430,24 @@ set_fillstyle(RendererLibart *renderer, FillStyle mode)
   case FILLSTYLE_SOLID:
     break;
   default:
-    message_error("gdk_renderer: Unsupported fill mode specified!\n");
+    message_error(_("gdk_renderer: Unsupported fill mode specified!\n"));
   }
 }
 
 static void
-set_font(RendererLibart *renderer, Font *font, real height)
+set_font(RendererLibart *renderer, DiaFont *font, real height)
 {
   renderer->font_height =
     ddisplay_transform_length(renderer->ddisp, height);
 
+#ifdef HAVE_FREETYPE
+  renderer->freetype_font = font_get_freetypefont(font, renderer->font_height);
+#else
   renderer->gdk_font = font_get_gdkfont(font, renderer->font_height);
-  renderer->suck_font = font_get_suckfont(font, renderer->font_height);
+  if (renderer->gdk_font == NULL)
+    message_error(_("Can't find font '%s' in height %d\n"),
+		  font->name, renderer->font_height);
+#endif
 }
 
 static void
@@ -1110,11 +1144,60 @@ fill_bezier(RendererLibart *renderer,
   art_svp_free( svp );
 }
 
+#ifdef HAVE_FREETYPE
+struct libart_freetype_user_data {
+  RendererLibart *renderer;
+  guint32 rgba;
+};
+
 static void
-draw_string(RendererLibart *renderer,
-	    const char *text,
-	    Point *pos, Alignment alignment,
-	    Color *color)
+libart_freetype_copy_glyph(FT_GlyphSlot glyph, int xpos, int ypos, 
+			   gpointer userdata) {
+  struct libart_freetype_user_data *data =
+    (struct libart_freetype_user_data *)userdata;
+  double affine[6];
+  FT_Bitmap *bitmap = &glyph->bitmap;
+  guchar *buffer = bitmap->buffer;
+  int rowstride = bitmap->pitch;
+  int width = bitmap->width;
+  int height = bitmap->rows;
+  /* For now, we're wasteful of memory and allocate a new RGBA buffer */
+  /* every time. */
+  guint32 *rgba_buffer = (guint32 *)g_malloc(sizeof(guint32)*width*height);
+  int x, y;
+
+  if (rowstride < 0) { /* Cartesian bitmap */
+    buffer = buffer+rowstride*(bitmap->rows-1);
+  }
+
+  for (x = 0; x < width; x++) {
+    for (y = 0; y < height; y++) {
+      rgba_buffer[x+y*width] = 
+	data->rgba|
+	(buffer[x+y*rowstride]<<24);
+    }
+  }  
+
+  art_affine_translate(affine, xpos, ypos);
+  art_rgb_rgba_affine(data->renderer->rgb_buffer,
+		      0, 0,
+		      data->renderer->renderer.pixel_width,
+		      data->renderer->renderer.pixel_height,
+		      data->renderer->renderer.pixel_width*3,
+		      rgba_buffer,
+		      width, height,
+		      width*4,
+		      affine,
+		      ART_FILTER_NEAREST, NULL);
+  g_free(rgba_buffer);
+}
+#endif
+
+static void
+draw_string (RendererLibart *renderer,
+	     const utfchar *text,
+	     Point *pos, Alignment alignment,
+	     Color *color)
 {
   DDisplay *ddisp = renderer->ddisp;
   int x, y, i, dx;
@@ -1125,10 +1208,16 @@ draw_string(RendererLibart *renderer,
   double xpos, ypos;
   guint32 rgba;
 
+#ifdef HAVE_FREETYPE
+  FreetypeString *fts;
+  struct libart_freetype_user_data data;
+
   ddisplay_transform_coords(ddisp, pos->x, pos->y,
 			    &x, &y);
-  
-  iwidth = gdk_string_width(renderer->gdk_font, text);
+
+  fts = freetype_load_string(text, renderer->freetype_font, strlen(text));
+  iwidth = fts->width;
+
   switch (alignment) {
   case ALIGN_LEFT:
     break;
@@ -1140,38 +1229,86 @@ draw_string(RendererLibart *renderer,
     break;
   }
 
-  suckfont = renderer->suck_font;
+  xpos = (double) x + 1; 
+  ypos = (double) y;
+  
+  data.rgba = color_to_abgr(color);
+  data.renderer = renderer;
+  
+  freetype_render_string(fts, xpos, ypos, libart_freetype_copy_glyph, &data);
+
+#else
+  GdkWChar *wcstr;
+  gchar *str, *mbstr;
+  int length, wclength;
+
+  ddisplay_transform_coords(ddisp, pos->x, pos->y,
+			    &x, &y);
+
+# ifdef GTK_DOESNT_TALK_UTF8_WE_DO
+  str = charconv_utf8_to_local8 (text);
+  length = strlen (str);
+  wcstr = g_new0 (GdkWChar, length + 1);
+  mbstr = g_strdup (str);
+  wclength = mbstowcs (wcstr, mbstr, length);
+  g_free (mbstr);
+
+  if (wclength > 0) {
+	  length = wclength;
+  } else {
+	  for (i = 0; i < length; i++) {
+		  wcstr[i] = (unsigned char) str[i];
+	  }
+  }
+  iwidth = gdk_text_width_wc (renderer->gdk_font, wcstr, length);
+
+  g_free (wcstr);
+#else
+  iwidth = gdk_string_width(renderer->gdk_font, text);
+#endif
+
+  switch (alignment) {
+  case ALIGN_LEFT:
+    break;
+  case ALIGN_CENTER:
+    x -= iwidth/2;
+    break;
+  case ALIGN_RIGHT:
+    x -= iwidth;
+    break;
+  }
+
+  suckfont = font_get_suckfont (renderer->gdk_font, text);
+
+  /* Couldn't find a font, bail out */
+  if (suckfont == NULL) return;
 
   xpos = (double) x + 1; 
   ypos = (double) (y - suckfont->ascent);
   
   rgba = color_to_rgba(color);
-  
-  len = strlen(text);
-  for (i = 0; i < len; i++) {
-    SuckChar *ch;
-    
-    ch = &suckfont->chars[(unsigned char)text[i]];
-    art_affine_translate(affine, xpos, ypos);
-    art_rgb_bitmap_affine(renderer->rgb_buffer,
-			  0, 0,
-			  renderer->renderer.pixel_width,
-			  renderer->renderer.pixel_height,
-			  renderer->renderer.pixel_width*3,
-			   suckfont->bitmap + (ch->bitmap_offset >> 3),
-			   ch->width,
-			   suckfont->bitmap_height,
-			   suckfont->bitmap_width >> 3,
-			   rgba,
-			   affine,
-			   ART_FILTER_NEAREST, NULL);
-    
-    dx = ch->left_sb + ch->width + ch->right_sb;
-    xpos += dx;
+
+  {
+	  SuckChar *ch;
+
+	  ch = &suckfont->chars[0];
+	  art_affine_translate (affine, xpos, ypos);
+	  art_rgb_bitmap_affine (renderer->rgb_buffer,
+				 0, 0,
+				 renderer->renderer.pixel_width,
+				 renderer->renderer.pixel_height,
+				 renderer->renderer.pixel_width * 3,
+				 suckfont->bitmap + (ch->bitmap_offset >> 3),
+				 ch->width,
+				 suckfont->bitmap_height,
+				 suckfont->bitmap_width >> 3,
+				 rgba,
+				 affine,
+				 ART_FILTER_NEAREST, NULL);
   }
 
-  
-  
+  suck_font_free (suckfont);
+#endif
 }
 
 static void
@@ -1241,11 +1378,52 @@ draw_image(RendererLibart *renderer,
 
 static real
 get_text_width(RendererLibart *renderer,
-	       const char *text, int length)
+	       const utfchar *text, int length)
 {
   int iwidth;
+  GdkWChar *wcstr;
+  gchar *mbstr, *str;
+  int len, wclength, i;
+  utfchar *utfbuf, *utf, *p;
   
-  iwidth = gdk_text_width(renderer->gdk_font, text, length);
+#ifdef HAVE_FREETYPE
+  iwidth = freetype_load_string(text, renderer->freetype_font, length)->width;
+#else
+# ifdef GTK_DOESNT_TALK_UTF8_WE_DO
+  p = utfbuf = g_strdup (text);
+  for (i = 0; i < length; i++)
+	  p = uni_next (p);
+
+  utf = g_new (utfchar, p - utfbuf + 1);
+  strncpy (utf, text, p - utfbuf);
+  utf[p - utfbuf] = 0;
+  str = charconv_utf8_to_local8 (utf);
+  len = strlen (str);
+
+  g_free (utfbuf);
+  g_free (utf);
+# else
+  str = g_strdup (text);
+  len = length;
+# endif
+  wcstr = g_new0 (GdkWChar, len + 1);
+  mbstr = g_strdup (str);
+  wclength = mbstowcs (wcstr, mbstr, len);
+  g_free (mbstr);
+
+  if (wclength > 0) {
+	  len = wclength;
+  } else {
+	  for (i = 0; i < len; i++) {
+		  wcstr[i] = (unsigned char) str[i];
+	  }
+  }
+  g_free (str);
+
+  iwidth = gdk_text_width_wc (renderer->gdk_font, wcstr, len);
+
+  g_free (wcstr);
+#endif
 
   return ddisplay_untransform_length(renderer->ddisp, (real) iwidth);
 }
