@@ -24,11 +24,13 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
+#define UNICODE_WORK_IN_PROGRESS /* here, it's mandatory or we break. */
 #include <config.h>
 #ifdef HAVE_UNICODE
 #include <stdlib.h>
 #include <glib.h>
 #include <unicode.h>
+#include "charconv.h"
 #include "ps-utf8.h"
 
 /* forward prototypes */
@@ -44,11 +46,11 @@ static void use_font(PSUnicoder *psu, PSFontDescriptor *fd);
 static PSEncodingPage *encoding_page_new(int num);
 static void encoding_page_destroy(PSEncodingPage *ep);
 static int encoding_page_add_unichar(PSEncodingPage *ep, 
-                                     unicode_char_t unichar);
+                                     unichar uchar);
 
 static void use_encoding(PSUnicoder *psu, PSEncodingPage *ep);
  
-static void psu_add_encoding(PSUnicoder *psu, unicode_char_t unichar);
+static void psu_add_encoding(PSUnicoder *psu, unichar uchar);
 static void psu_make_new_encoding_page(PSUnicoder *psu);
 /* Unicoder functions */
 
@@ -119,19 +121,19 @@ static void psu_make_new_encoding_page(PSUnicoder *psu)
   }
 }
 
-static void psu_add_encoding(PSUnicoder *psu, unicode_char_t unichar)
+static void psu_add_encoding(PSUnicoder *psu, unichar uchar)
 {
   if (g_hash_table_lookup(psu->unicode_to_page,
-                          GINT_TO_POINTER(unichar))) {
+                          GINT_TO_POINTER(uchar))) {
     return;
   }
-  if (!encoding_page_add_unichar(psu->last_page,unichar)) {
+  if (!encoding_page_add_unichar(psu->last_page,uchar)) {
     psu_make_new_encoding_page(psu);
-    if (!encoding_page_add_unichar(psu->last_page,unichar))
+    if (!encoding_page_add_unichar(psu->last_page,uchar))
       g_assert_not_reached();
   }
   g_hash_table_insert(psu->unicode_to_page,
-                      GINT_TO_POINTER(unichar),
+                      GINT_TO_POINTER(uchar),
                       psu->last_page);
   if (psu->last_page == psu->current_encoding) {
     psu->current_encoding = NULL; /* so that it's re-emitted. */
@@ -141,30 +143,21 @@ static void psu_add_encoding(PSUnicoder *psu, unicode_char_t unichar)
 
 extern void 
 psu_check_string_encodings(PSUnicoder *psu, 
-                           const gchar *utf8_string)
+                           const utfchar *utf8_string)
 {
-  unicode_char_t unichar;
-  const gchar *p = utf8_string;
+  unichar uchar;
+  const utfchar *p = utf8_string;
 
   while (p && (*p)) {
-    p = unicode_get_utf8(p,&unichar);
+    p = uni_get_utf8(p,&uchar);
     if (!p) {
       g_warning("malformed string.");
       return; /* malformed string. */
     }
 
-    psu_add_encoding(psu,unichar);
-    if ((unichar > 32) && (unichar < 2048)) {
-      int i;
-      /* hack: we'll also put in the neighbours characters (for "alphabetic
-         alphabets" only, betting on the high probability of their use
-         anyway (this should cut a little down the encoding redefinitions) */
-      for (i=1; i<3; i++) {
-        //if ((unichar+i)<0xFFFE)  /* gcc is still too dumb ... */
-        psu_add_encoding(psu,unichar+i);
-        //if ((unichar-i)>0)       /* ... to see that by itself */
-        psu_add_encoding(psu,unichar-i);
-      }
+    psu_add_encoding(psu,uchar);
+    if ((uchar > 32) && (uchar < 2048)) {
+      psu_add_encoding(psu,uchar);
     }
   }  
 }
@@ -186,22 +179,23 @@ static void psu_show_flush_buffer(const PSUnicoder *psu,
 }
 
 
-static void prepare_psu_show_string(PSUnicoder *psu,
-                                    const gchar *utf8_string,
-                                    FlushFunc flushfunc)
+static void 
+encoded_psu_show_string(PSUnicoder *psu,
+                        const utfchar *utf8_string,
+                        FlushFunc flushfunc)
 {
-  unicode_char_t unichar;
+  unichar uchar;
   gchar c;
   gchar buf[BUFSIZE];
   int bufu = 0;
   gboolean first = TRUE;
   size_t len = 0;
-  const gchar *p = utf8_string;
+  const utfchar *p = utf8_string;
 
   while (p && (*p)) {
-    p = unicode_get_utf8(p,&unichar);
+    p = uni_get_utf8(p,&uchar);
     if (!p) {
-      g_warning("malformed utf8 string in prepare_psu_show_string.");      
+      g_warning("malformed utf8 string in encoded_psu_show_string.");      
       return; /* malformed string. */
     }
     len++;
@@ -209,7 +203,7 @@ static void prepare_psu_show_string(PSUnicoder *psu,
     if (psu->current_encoding) {
       c = 
         (gchar)GPOINTER_TO_INT(g_hash_table_lookup(psu->current_encoding->backpage,
-                                                   GINT_TO_POINTER(unichar)));
+                                                   GINT_TO_POINTER(uchar)));
     } else {
       c = 0;
     }
@@ -217,22 +211,22 @@ static void prepare_psu_show_string(PSUnicoder *psu,
     if (!c) {
       /* No page or not the right page... switch ! */
       PSEncodingPage *ep = g_hash_table_lookup(psu->unicode_to_page,
-                                               GINT_TO_POINTER(unichar));
+                                               GINT_TO_POINTER(uchar));
       if (ep) {
         use_encoding(psu,ep);
         /* now psu->current_encoding == ep */
         c = (gchar)GPOINTER_TO_INT(g_hash_table_lookup(ep->backpage,
-                                                       GINT_TO_POINTER(unichar)));
+                                                       GINT_TO_POINTER(uchar)));
       } else {
         c = 31; 
       }
 
       if ((!c)||(c==31)) {
-        g_message("unichar %.4X has not been found in the encoding pages !",unichar);
+        g_message("uchar %.4X has not been found in the encoding pages !",uchar);
         g_assert_not_reached();
       }
     }
-    /* We now are in the right page, and c is the image of unichar with that
+    /* We now are in the right page, and c is the image of uchar with that
        encoding. */
     
     if ( ((!psu->current_font) || 
@@ -270,6 +264,69 @@ static void prepare_psu_show_string(PSUnicoder *psu,
     psu_show_flush_buffer(psu,buf,&bufu,flushfunc,first);
 }
 
+static void 
+symbol_psu_show_string(PSUnicoder *psu,
+                        const utfchar *utf8_string,
+                        FlushFunc flushfunc)
+{
+  /* Special case with Symbol fonts: */
+  unichar uchar;
+  gchar c;
+  gchar buf[BUFSIZE];
+  int bufu = 0;
+  gboolean first = TRUE;
+  size_t len = 0;
+  const utfchar *p = utf8_string;
+  PSFontDescriptor *fd;
+  const gchar *font_name = "Symbol";
+
+  /* Locate and use the Symbol font (bare): */
+
+  fd = g_hash_table_lookup(psu->defined_fonts,font_name);
+  if (fd) {
+    font_name = fd->name;
+  } else {
+    fd = font_descriptor_new(psu->face,
+                             NULL,
+                             font_name);
+    font_name = fd->name;
+    g_hash_table_insert(psu->defined_fonts,(gpointer)font_name,fd);
+  }
+  use_font(psu,fd);
+
+  /* We have the Symbol font loaded. */
+
+  while (p && (*p)) {
+    p = uni_get_utf8(p,&uchar);
+    if (!p) {
+      g_warning("malformed utf8 string in symbol_show_string.");      
+      return; /* malformed string. */
+    }
+    len++;
+
+    if (uchar < 256) c = uchar;
+    else c = '?';
+
+    switch (c) {
+    case ')':
+    case '(':
+    case '\\':
+      buf[bufu++] = '\\';
+      buf[bufu++] = c;
+      break;
+    default:
+      buf[bufu++] = c;
+    }
+
+    if (bufu >= BUFSIZE - 3) {
+      psu_show_flush_buffer(psu,buf,&bufu,flushfunc,first);
+      first = FALSE;
+    }
+  }
+
+  if ((bufu)||(!len)) /* even if !len, to avoid crashing the PS stack. */
+    psu_show_flush_buffer(psu,buf,&bufu,flushfunc,first);
+}
 
 static void 
 flush_show_string(const PSUnicoder *psu,
@@ -281,14 +338,19 @@ flush_show_string(const PSUnicoder *psu,
 
 extern void 
 psu_show_string(PSUnicoder *psu,
-                const gchar *utf8_string)
+                const utfchar *utf8_string)
 {
-  prepare_psu_show_string(psu,utf8_string,&flush_show_string);
+  if (0==strcmp(psu->face,"Symbol")) {
+    symbol_psu_show_string(psu,utf8_string,&flush_show_string);
+  } else { 
+    encoded_psu_show_string(psu,utf8_string,&flush_show_string);
+  }
 }
 
+
 static void 
 flush_get_string_width(const PSUnicoder *psu,
-                            const gchar *buf,
+                            const utfchar *buf,
                             gboolean first)
 {
   psu->callbacks->get_string_width(psu->usrdata,buf,first);
@@ -296,9 +358,13 @@ flush_get_string_width(const PSUnicoder *psu,
 
 extern void 
 psu_get_string_width(PSUnicoder *psu,
-                     const gchar *utf8_string)
+                     const utfchar *utf8_string)
 {
-  prepare_psu_show_string(psu,utf8_string,&flush_get_string_width);
+  if (0==strcmp(psu->face,"Symbol")) {
+    symbol_psu_show_string(psu,utf8_string,&flush_get_string_width);
+  } else { 
+    encoded_psu_show_string(psu,utf8_string,&flush_get_string_width);
+  }
 }
 
 
@@ -326,7 +392,7 @@ static void encoding_page_destroy(PSEncodingPage *ep)
 }
 
 static int 
-encoding_page_add_unichar(PSEncodingPage *ep, unicode_char_t unichar)
+encoding_page_add_unichar(PSEncodingPage *ep, unichar uchar)
 {
   int res;
   if (ep->entries >= PSEPAGE_SIZE) return 0; /* page is full. */
@@ -337,11 +403,11 @@ encoding_page_add_unichar(PSEncodingPage *ep, unicode_char_t unichar)
 
   res = ep->entries++;
   
-  ep->page[res] = unichar;
+  ep->page[res] = uchar;
   
   res += PSEPAGE_BEGIN;
   g_hash_table_insert(ep->backpage,
-                      GINT_TO_POINTER(unichar),GINT_TO_POINTER(res));
+                      GINT_TO_POINTER(uchar),GINT_TO_POINTER(res));
   ep->serial_num++;
   return res;
   
@@ -359,7 +425,7 @@ use_encoding(PSUnicoder *psu, PSEncodingPage *ep)
   psu->current_encoding = ep;
 }
 
-/* Font descriptor functions */
+/* DiaFont descriptor functions */
 
 static gchar *
 make_font_descriptor_name(const gchar *face,
@@ -396,19 +462,27 @@ font_descriptor_destroy(PSFontDescriptor *fd)
 static void 
 use_font(PSUnicoder *psu, PSFontDescriptor *fd)
 {
-  gboolean undef,define,select;
+  if (psu->current_font == fd) return; /* Already done. */
+
+  if (!fd->encoding) { /* bare font. No encoding. */
+    psu->callbacks->select_ps_font(psu->usrdata,fd->name,psu->size);
+  } else {
+    gboolean undef,define,select;
   
-  define = (fd->encoding->serial_num != fd->encoding_serial_num);
-  undef = (define && (fd->encoding_serial_num <= 0));
-  select = ((psu->current_font != fd) || (psu->current_size != psu->size));
+    define = (fd->encoding->serial_num != fd->encoding_serial_num);
+    undef = (define && (fd->encoding_serial_num <= 0));
+    select = ((psu->current_font != fd) || (psu->current_size != psu->size));
+
+    if (undef) psu->callbacks->destroy_ps_font(psu->usrdata,fd->name);
+    if (define) psu->callbacks->build_ps_font(psu->usrdata,fd->name,
+                                              fd->face, fd->encoding->name);
+    fd->encoding_serial_num = fd->encoding->serial_num;
+    if (select) psu->callbacks->select_ps_font(psu->usrdata,
+                                               fd->name,
+                                               psu->size);
+  }
 
-  if (undef) psu->callbacks->destroy_ps_font(psu->usrdata,fd->name);
-  if (define) psu->callbacks->build_ps_font(psu->usrdata,fd->name,
-                                            fd->face, fd->encoding->name);
-  fd->encoding_serial_num = fd->encoding->serial_num;
-  if (select) psu->callbacks->select_ps_font(psu->usrdata,fd->name,psu->size);
   psu->current_size = psu->size;
-
   psu->current_font = fd;
   psu->current_encoding = fd->encoding;
 }
@@ -1713,19 +1787,24 @@ new_uni_to_adobe_hash(void)
 }
 
 extern const char *
-unicode_to_ps_name(unicode_char_t val)
+unicode_to_ps_name (unichar val)
 {
-  char *ps;
+	static GHashTable *std2ps = NULL;
+	char *ps;
 
-  if (!val) return "xi"; /* of course, you shouldn't see this everywhere */
+	if (!val) return "xi"; /* of course, you shouldn't see this everywhere */
 
-  if (!uni2ps) new_uni_to_adobe_hash();
-  ps = g_hash_table_lookup(uni2ps, GINT_TO_POINTER(val));
-  if (!ps) {
-    ps = g_strdup_printf("uni%.4X",val);
-    g_hash_table_insert(uni2ps,GINT_TO_POINTER(val),ps);
-  }
-  return ps;
+	if (!uni2ps) new_uni_to_adobe_hash ();
+	ps = g_hash_table_lookup (uni2ps, GINT_TO_POINTER (val));
+	if (!ps) {
+		if (!std2ps) std2ps = g_hash_table_new (NULL, NULL);
+		ps = g_hash_table_lookup (std2ps, GINT_TO_POINTER (val));
+		if (!ps) {
+			ps = g_strdup_printf ("uni%.4X", val);
+			g_hash_table_insert (uni2ps, GINT_TO_POINTER (val), ps);
+		}
+	}
+	return ps;
 }
 
 #endif /* HAVE_UNICODE */
