@@ -21,12 +21,16 @@
 #include "widgets.h"
 #include "message.h"
 
+#include <stdlib.h>
 #include <glib.h>
 #include <gdk/gdk.h>
 #include <gtk/gtkradiomenuitem.h>
 #include <gtk/gtklabel.h>
 #include <gtk/gtksignal.h>
 
+void dia_font_selector_set_styles (DiaFontSelector *fs, DiaFont *font);
+guchar **dia_font_selector_get_sorted_names();
+
 struct menudesc {
   char *name;
   int enum_value;
@@ -62,9 +66,122 @@ dia_font_selector_class_init (DiaFontSelectorClass *class)
   object_class = (GtkObjectClass*) class;
 }
 
+#ifdef HAVE_FREETYPE
+/* This function is stolen from dillo.
+ */
+static void
+dia_font_selector_scroll_popup(GtkWidget *widget)
+{
+  /*
+   * todo:
+   *   1) Scrolling menues should rather be the task of Gtk+. This is
+   *      a hack, and I don't know if it does not break anything.
+   *   2) It could be improved, e.g. a timeout could be added for
+   *      better mouse navigation.
+    */
+  int y, h, mx, my, sh;
+
+  y = widget->allocation.y;
+  h = widget->allocation.height;
+  gdk_window_get_geometry (widget->parent->parent->window,
+			   &mx, &my, NULL, NULL, NULL);
+  sh = gdk_screen_height ();
+
+  if (y + my < 0)
+    gdk_window_move (widget->parent->parent->window, mx, - y + 1);
+  else if (y + my > sh - h)
+    gdk_window_move (widget->parent->parent->window, mx, sh - h - y - 1);
+}
+
+static void
+dia_font_selector_font_selected(GtkWidget *button, gpointer data)
+{
+  DiaFontSelector *fs = DIAFONTSELECTOR(data);
+
+  GtkWidget *active = gtk_menu_get_active(fs->font_menu);
+  char *fontname = (char *)gtk_object_get_user_data(GTK_OBJECT(active));
+  dia_font_selector_set_styles(fs, font_getfont(fontname));
+}
+
+struct select_pair { guchar **array; int counter; };
+
+static void
+dia_font_selector_collect_names(gpointer key, gpointer value,
+				gpointer userdata) {
+  struct select_pair *pair = (struct select_pair *)userdata;
+  pair->array[pair->counter++] = (guchar *)key;
+}
+
+static int
+dia_font_selector_sort_string(const void *s1, const void *s2) {
+  return strcasecmp(*(char **)s1, *(char **)s2);
+}
+
+static guchar **
+dia_font_selector_get_fonts_sorted() {
+  struct select_pair pair;
+  int num_fonts;
+
+  num_fonts = g_hash_table_size(freetype_fonts);
+  pair.array = (guchar **)g_malloc(sizeof(guchar*)*num_fonts);
+  pair.counter = 0;
+  g_hash_table_foreach(freetype_fonts, dia_font_selector_collect_names,
+		       (gpointer)&pair);
+  
+  qsort(pair.array, num_fonts, sizeof(guchar *), dia_font_selector_sort_string);
+  return pair.array;
+}
+
+#endif
+
 static void
 dia_font_selector_init (DiaFontSelector *fs)
 {
+#ifdef HAVE_FREETYPE
+  /* Go through hash table and build menu */
+  GtkWidget *menu;
+  GtkWidget *omenu;
+  int i;
+  guchar **fontnames;
+  int num_fonts;
+
+  omenu = gtk_option_menu_new();
+  fs->font_omenu = GTK_OPTION_MENU(omenu);
+  menu = gtk_menu_new ();
+  fs->font_menu = GTK_MENU(menu);
+
+  fontnames = dia_font_selector_get_fonts_sorted();
+  num_fonts = g_hash_table_size(freetype_fonts);
+
+  for (i = 0; i < num_fonts; i++) {
+    GtkWidget *menuitem;
+    menuitem = gtk_menu_item_new_with_label (fontnames[i]);
+    gtk_object_set_user_data(GTK_OBJECT(menuitem), fontnames[i]);
+    gtk_signal_connect(GTK_OBJECT(menuitem), "select", dia_font_selector_scroll_popup, NULL);
+    gtk_menu_append (GTK_MENU (menu), menuitem);
+    gtk_widget_show (menuitem);
+  }
+
+  g_free(fontnames);
+
+  gtk_option_menu_set_menu (GTK_OPTION_MENU (fs->font_omenu), menu);
+  gtk_widget_show(menu);
+  gtk_widget_show(omenu);
+
+  gtk_signal_connect(GTK_OBJECT(menu), "unmap", dia_font_selector_font_selected, fs);
+
+  omenu = gtk_option_menu_new();
+  fs->style_omenu = GTK_OPTION_MENU(omenu);
+  menu = gtk_menu_new ();
+  fs->style_menu = GTK_MENU(menu);
+  gtk_option_menu_set_menu (GTK_OPTION_MENU (fs->style_omenu), menu);
+
+  gtk_widget_show(menu);
+  gtk_widget_show(omenu);
+
+  gtk_box_pack_start_defaults(GTK_BOX(fs), GTK_WIDGET(fs->font_omenu));
+  gtk_box_pack_start_defaults(GTK_BOX(fs), GTK_WIDGET(fs->style_omenu));
+#else
   GtkWidget *menu;
   GtkWidget *submenu;
   GtkWidget *menuitem;
@@ -90,9 +207,9 @@ dia_font_selector_init (DiaFontSelector *fs)
   }
   
   gtk_option_menu_set_menu (GTK_OPTION_MENU (fs), menu);
+#endif
 }
 
-
 guint
 dia_font_selector_get_type        (void)
 {
@@ -100,6 +217,10 @@ dia_font_selector_get_type        (void)
   GList *list;
   char *fontname;
   int i;
+#ifdef HAVE_FREETYPE
+  guchar **fontnames;
+  int num_fonts;
+#endif
 
   if (!dfs_type) {
     GtkTypeInfo dfs_info = {
@@ -112,13 +233,27 @@ dia_font_selector_get_type        (void)
       (GtkArgGetFunc) NULL
     };
     
+#ifdef HAVE_FREETYPE
+    dfs_type = gtk_type_unique (gtk_hbox_get_type (), &dfs_info);
+#else
     dfs_type = gtk_type_unique (gtk_option_menu_get_type (), &dfs_info);
-
+#endif
 
     /* Init the font hash_table: */
     font_nr_hashtable = g_hash_table_new(g_str_hash, g_str_equal);
 
-    i=0;
+#ifdef HAVE_FREETYPE
+    num_fonts = g_hash_table_size(freetype_fonts);
+    fontnames = dia_font_selector_get_fonts_sorted();
+    for (i = 0; i < num_fonts; i++) {
+      /* Notice adding 1, to be able to discern from NULL return value */
+      g_hash_table_insert(font_nr_hashtable,
+			  fontnames[i], GINT_TO_POINTER(i+1));
+    }
+    g_free(fontnames);
+#else
+    /* Notice starting at 1, to be able to discern from NULL return value */
+    i=1;
     list = font_names;
     while (list != NULL) {
       fontname = (char *) list->data;
@@ -130,7 +265,7 @@ dia_font_selector_get_type        (void)
       list = g_list_next(list);
       i++;
     }
-    
+#endif
   }
   
   return dfs_type;
@@ -142,8 +277,38 @@ dia_font_selector_new ()
   return GTK_WIDGET ( gtk_type_new (dia_font_selector_get_type ()));
 }
 
+#ifdef HAVE_FREETYPE
 void
-dia_font_selector_set_font(DiaFontSelector *fs, Font *font)
+dia_font_selector_set_styles(DiaFontSelector *fs, DiaFont *font)
+{
+  int i=0, select = 0;
+  GList *style_list = font->family->diafonts;
+
+  GtkWidget *menu = gtk_menu_new();
+
+  while (style_list != NULL) {
+    DiaFont *style_font = (DiaFont *)style_list->data;
+    GtkWidget *menuitem = gtk_menu_item_new_with_label (style_font->style);
+    gtk_object_set_user_data(GTK_OBJECT(menuitem), style_font->style);
+    if (!strcmp(style_font->style, font->style))
+      select = i;
+    i++;
+    gtk_menu_append (GTK_MENU (menu), menuitem);
+    gtk_widget_show (menuitem);
+    style_list = g_list_next(style_list);
+  }
+  gtk_widget_show(menu);
+  /* Need to dealloc the menu, methinks */
+  gtk_option_menu_remove_menu(fs->style_omenu);
+  gtk_option_menu_set_menu(fs->style_omenu, menu);
+  fs->style_menu = GTK_MENU(menu);
+  gtk_option_menu_set_history(GTK_OPTION_MENU(fs->style_omenu), select);
+  gtk_menu_set_active(fs->style_menu, select);
+}
+#endif
+
+void
+dia_font_selector_set_font(DiaFontSelector *fs, DiaFont *font)
 {
   void *font_nr_ptr;
   int font_nr;
@@ -152,26 +317,43 @@ dia_font_selector_set_font(DiaFontSelector *fs, Font *font)
 				    font->name);
 
   if (font_nr_ptr==NULL) {
-    message_error("Trying to set invalid font!\n");
+    message_error("Trying to set invalid font %s!\n", font->name);
     font_nr = 0;
   } else {
-    font_nr = GPOINTER_TO_INT(font_nr_ptr);
+    /* Notice subtracting 1, to be able to discern from NULL return value */
+    font_nr = GPOINTER_TO_INT(font_nr_ptr)-1;
   }
   
+#ifdef HAVE_FREETYPE
+  gtk_option_menu_set_history(GTK_OPTION_MENU(fs->font_omenu), font_nr);
+  gtk_menu_set_active(fs->font_menu, font_nr);
+
+  dia_font_selector_set_styles(fs, font);
+#else
   gtk_option_menu_set_history(GTK_OPTION_MENU(fs), font_nr);
   gtk_menu_set_active(fs->font_menu, font_nr);
+#endif
 }
 
-Font *
+DiaFont *
 dia_font_selector_get_font(DiaFontSelector *fs)
 {
   GtkWidget *menuitem;
   char *fontname;
-  
+#ifdef HAVE_FREETYPE
+  char *stylename;
+#endif
+
   menuitem = gtk_menu_get_active(fs->font_menu);
   fontname = gtk_object_get_user_data(GTK_OBJECT(menuitem));
 
+#ifdef HAVE_FREETYPE
+  menuitem = gtk_menu_get_active(fs->style_menu);
+  stylename = gtk_object_get_user_data(GTK_OBJECT(menuitem));
+  return font_getfont_with_style(fontname, stylename);
+#else
   return font_getfont(fontname);
+#endif
 }
 
 /************* DiaAlignmentSelector: ***************/
@@ -598,7 +780,7 @@ dia_color_selector_get_color(DiaColorSelector *cs, Color *color)
 
 void
 dia_color_selector_set_color (DiaColorSelector *cs,
-			      Color *color)
+			      const Color *color)
 {
   GdkColor col;
   
@@ -625,6 +807,7 @@ dia_color_selector_set_color (DiaColorSelector *cs,
 
 
 /************* DiaArrowSelector: ***************/
+
 static void
 dia_arrow_selector_class_init (DiaArrowSelectorClass *class)
 {
@@ -672,8 +855,11 @@ static struct menudesc arrow_types[] =
  {N_("Blanked Box"),ARROW_BLANKED_BOX},
  {N_("Slashed"),ARROW_SLASH_ARROW},
  {N_("Integral Symbol"),ARROW_INTEGRAL_SYMBOL},
+ {N_("Crow Foot"),ARROW_CROW_FOOT},
+ {N_("Cross"),ARROW_CROSS},
  {NULL,0}};
 
+
 static void
 dia_arrow_selector_init (DiaArrowSelector *as)
 {
@@ -917,12 +1103,14 @@ dia_file_selector_new ()
 void
 dia_file_selector_set_file(DiaFileSelector *fs, gchar *file)
 {
+  /* UTF8 conversions here ? */
   gtk_entry_set_text(GTK_ENTRY(fs->entry), file);
 }
 
 gchar *
 dia_file_selector_get_file(DiaFileSelector *fs)
 {
+  /* UTF8 conversions here ? */
   return gtk_entry_get_text(GTK_ENTRY(fs->entry));
 }
 
