@@ -1,3 +1,4 @@
+/* -*- Mode: C; indent-tabs-mode: nil; c-basic-offset: 8 c-style: "K&R" -*- */
 /* Dia -- an diagram creation/manipulation program
  * Copyright (C) 1998 Alexander Larsson
  *
@@ -29,6 +30,7 @@
 #include "render.h"
 #include "attributes.h"
 #include "properties.h"
+#include "charconv.h"
 
 #include "class.h"
 
@@ -57,8 +59,8 @@ static Object *umlclass_load(ObjectNode obj_node, int version,
 			     const char *filename);
 
 static PropDescription *umlclass_describe_props(UMLClass *umlclass);
-static void umlclass_get_props(UMLClass *umlclass, Property *props, guint nprops);
-static void umlclass_set_props(UMLClass *umlclass, Property *props, guint nprops);
+static void umlclass_get_props(UMLClass *umlclass, GPtrArray *props);
+static void umlclass_set_props(UMLClass *umlclass, GPtrArray *props);
 
 static ObjectTypeOps umlclass_type_ops =
 {
@@ -146,17 +148,19 @@ static PropOffset umlclass_offsets[] = {
 };
 
 static void
-umlclass_get_props(UMLClass * umlclass, Property *props, guint nprops)
+umlclass_get_props(UMLClass * umlclass, GPtrArray *props)
 {
   object_get_props_from_offsets(&umlclass->element.object, 
-                                umlclass_offsets, props, nprops);
+                                umlclass_offsets, props);
 }
 
 static void
-umlclass_set_props(UMLClass *umlclass, Property *props, guint nprops)
+umlclass_set_props(UMLClass *umlclass, GPtrArray *props)
 {
   object_set_props_from_offsets(&umlclass->element.object, umlclass_offsets,
-                                props, nprops);
+                                props);
+  /* Update data: */
+  umlclass_calculate_data(umlclass);
   umlclass_update_data(umlclass);
 }
 
@@ -198,7 +202,8 @@ umlclass_draw(UMLClass *umlclass, Renderer *renderer)
   Element *elem;
   real x,y;
   Point p, p1, p2, p3;
-  Font *font;
+  DiaFont *font;
+  real font_height;
   int i;
   GList *list;
   
@@ -242,14 +247,17 @@ umlclass_draw(UMLClass *umlclass, Renderer *renderer)
   }
 
   /* name: */
-  if (umlclass->abstract) 
+  if (umlclass->abstract) {
     font = umlclass->abstract_classname_font;
-  else 
+    font_height = umlclass->abstract_classname_font_height;
+  } else {
     font = umlclass->classname_font;
+    font_height = umlclass->classname_font_height;
+  }
 
-  p.y += umlclass->classname_font_height;
+  p.y += font_height;
 
-  renderer->ops->set_font(renderer, font, umlclass->classname_font_height);
+  renderer->ops->set_font(renderer, font, font_height);
   renderer->ops->draw_string(renderer,
 			     umlclass->name,
 			     &p, ALIGN_CENTER, 
@@ -270,35 +278,42 @@ umlclass_draw(UMLClass *umlclass, Renderer *renderer)
 
     if (!umlclass->suppress_attributes) {
       p.x = x + UMLCLASS_BORDER/2.0 + 0.1;
-      p.y = p1.y + 0.1 + umlclass->font_ascent;
+      p.y = p1.y + 0.1;
 
       i = 0;
       list = umlclass->attributes;
       while (list != NULL) {
 	UMLAttribute *attr = (UMLAttribute *)list->data;
-	if (attr->abstract) 
+	if (attr->abstract) {
 	  font = umlclass->abstract_font;
-	else
+          font_height = umlclass->abstract_font_height;
+          p.y += umlclass->abstract_font_ascent;
+        } else {
 	  font = umlclass->normal_font;
+          font_height = umlclass->font_height;
+          p.y += umlclass->font_ascent;
+        }
 
-	renderer->ops->set_font(renderer, font, umlclass->font_height);
+        renderer->ops->set_font (renderer, font, font_height);
 	renderer->ops->draw_string(renderer,
 				   umlclass->attributes_strings[i],
 				   &p, ALIGN_LEFT, 
 				   &umlclass->color_foreground);
 	if (attr->class_scope) {
-	  p1 = p; 
-	  p1.y += umlclass->font_height*0.1;
+	  p1 = p;
+          p1.y += font_height * 0.1;
 	  p3 = p1;
 	  p3.x += font_string_width(umlclass->attributes_strings[i],
-				    font, umlclass->font_height);
+				    font, font_height);
 	  renderer->ops->set_linewidth(renderer, UMLCLASS_UNDERLINEWIDTH);
 	  renderer->ops->draw_line(renderer, &p1, &p3,  &umlclass->color_foreground);
 	  renderer->ops->set_linewidth(renderer, UMLCLASS_BORDER);
 	}
 	
-	
-	p.y += umlclass->font_height;
+	if (attr->abstract) 
+          p.y += font_height-umlclass->abstract_font_ascent;
+        else
+          p.y += font_height-umlclass->font_ascent;
 	
 	list = g_list_next(list);
 	i++;
@@ -320,18 +335,24 @@ umlclass_draw(UMLClass *umlclass, Renderer *renderer)
 			     &umlclass->color_foreground);
     if (!umlclass->suppress_operations) {
       p.x = x + UMLCLASS_BORDER/2.0 + 0.1;
-      p.y = p1.y + 0.1 + umlclass->font_ascent;
+      p.y = p1.y + 0.1;
 
       i = 0;
       list = umlclass->operations;
       while (list != NULL) {
 	UMLOperation *op = (UMLOperation *)list->data;
-	if (op->abstract) 
+	/* Must add a new font for virtual yet not abstract methods. Bold Italic for abstract? */
+	if (op->inheritance_type != UML_LEAF) {
 	  font = umlclass->abstract_font;
-	else
+          font_height = umlclass->abstract_font_height;
+          p.y += umlclass->abstract_font_ascent;
+	} else {
 	  font = umlclass->normal_font;
+          font_height = umlclass->font_height;
+          p.y += umlclass->font_ascent;
+        }
 
-	renderer->ops->set_font(renderer, font, umlclass->font_height);
+	renderer->ops->set_font(renderer, font, font_height);
 	renderer->ops->draw_string(renderer,
 				   umlclass->operations_strings[i],
 				   &p, ALIGN_LEFT, 
@@ -339,16 +360,19 @@ umlclass_draw(UMLClass *umlclass, Renderer *renderer)
 
 	if (op->class_scope) {
 	  p1 = p; 
-	  p1.y += umlclass->font_height*0.1;
+	  p1.y += font_height * 0.1;
 	  p3 = p1;
 	  p3.x += font_string_width(umlclass->operations_strings[i],
-				    font, umlclass->font_height);
+				    font, font_height);
 	  renderer->ops->set_linewidth(renderer, UMLCLASS_UNDERLINEWIDTH);
 	  renderer->ops->draw_line(renderer, &p1, &p3,  &umlclass->color_foreground);
 	  renderer->ops->set_linewidth(renderer, UMLCLASS_BORDER);
 	}
 
-	p.y += umlclass->font_height;
+	if (op->inheritance_type != UML_LEAF) 
+          p.y += font_height-umlclass->abstract_font_ascent;
+        else
+          p.y += font_height-umlclass->font_ascent;
 
 	list = g_list_next(list);
 	i++;
@@ -476,21 +500,20 @@ umlclass_update_data(UMLClass *umlclass)
 void
 umlclass_calculate_data(UMLClass *umlclass)
 {
-  real font_height;
   int i;
   GList *list;
   real maxwidth;
   real width;
   
-  font_height = umlclass->font_height;
-  umlclass->font_ascent = font_ascent(umlclass->normal_font, font_height);
+  umlclass->font_ascent = font_ascent(umlclass->normal_font, umlclass->font_height);
+  umlclass->abstract_font_ascent = font_ascent(umlclass->abstract_font, umlclass->abstract_font_height);
 
   /* name box: */
 
   if (umlclass->abstract) 
     maxwidth = font_string_width(umlclass->name,
 				 umlclass->abstract_classname_font,
-				 umlclass->classname_font_height);
+				 umlclass->abstract_classname_font_height);
   else 
     maxwidth = font_string_width(umlclass->name,
 				 umlclass->classname_font,
@@ -501,21 +524,30 @@ umlclass_calculate_data(UMLClass *umlclass)
     g_free(umlclass->stereotype_string);
   }
   if (umlclass->stereotype != NULL) {
-    umlclass->namebox_height += font_height;
-    umlclass->stereotype_string =
-      g_malloc(sizeof(char)*(strlen(umlclass->stereotype)+2+1));
-    
-    umlclass->stereotype_string[0] = UML_STEREOTYPE_START;
-    umlclass->stereotype_string[1] = 0;
-    
-    strcat(umlclass->stereotype_string, umlclass->stereotype);
+	  utfchar *utfstart, *utfend;
 
-    i = strlen(umlclass->stereotype_string);
-    umlclass->stereotype_string[i] = UML_STEREOTYPE_END;
-    umlclass->stereotype_string[i+1] = 0;
-    
-    width = font_string_width(umlclass->stereotype_string, umlclass->normal_font, font_height);
-    maxwidth = MAX(width, maxwidth);
+#ifdef GTK_DOESNT_TALK_UTF8_WE_DO
+	  utfstart = charconv_local8_to_utf8 (UML_STEREOTYPE_START);
+	  utfend = charconv_local8_to_utf8 (UML_STEREOTYPE_END);
+#else
+          utfstart = g_strdup (UML_STEREOTYPE_START);
+          utfend = g_strdup (UML_STEREOTYPE_END);
+#endif
+	  umlclass->namebox_height += umlclass->font_height;
+	  umlclass->stereotype_string =
+		  g_malloc (sizeof (utfchar) * (strlen (utfstart) +
+                                                strlen (umlclass->stereotype) +
+                                                strlen (utfend) + 1));
+
+	  strcpy (umlclass->stereotype_string, utfstart);
+	  strcat (umlclass->stereotype_string, umlclass->stereotype);
+	  strcat (umlclass->stereotype_string, utfend);
+
+          g_free (utfstart);
+          g_free (utfend);
+
+	  width = font_string_width (umlclass->stereotype_string, umlclass->normal_font, umlclass->font_height);
+	  maxwidth = MAX(width, maxwidth);
   } else {
     umlclass->stereotype_string = NULL;
   }
@@ -528,19 +560,12 @@ umlclass_calculate_data(UMLClass *umlclass)
     g_free(umlclass->attributes_strings);
   }
   umlclass->num_attributes = g_list_length(umlclass->attributes);
-  umlclass->attributesbox_height = font_height * umlclass->num_attributes + 2*0.1;
-
-  if ((umlclass->attributesbox_height<0.4) ||
-      umlclass->suppress_attributes )
-      umlclass->attributesbox_height = 0.4;
-
-  if (!umlclass->visible_attributes )
-    umlclass->attributesbox_height = 0.0;
+  umlclass->attributesbox_height = 2*0.1;
 
   umlclass->attributes_strings = NULL;
   if (umlclass->num_attributes != 0) {
     umlclass->attributes_strings =
-      g_malloc(sizeof(char *)*umlclass->num_attributes);
+      g_malloc (sizeof (utfchar *) * umlclass->num_attributes);
     i = 0;
     list = umlclass->attributes;
     while (list != NULL) {
@@ -549,10 +574,14 @@ umlclass_calculate_data(UMLClass *umlclass)
       attr = (UMLAttribute *) list->data;
       umlclass->attributes_strings[i] = uml_get_attribute_string(attr);
 
-      if (attr->abstract)
-	width = font_string_width(umlclass->attributes_strings[i], umlclass->abstract_font, font_height);
-      else
-	width = font_string_width(umlclass->attributes_strings[i], umlclass->normal_font, font_height);
+      if (attr->abstract) {
+	width = font_string_width(umlclass->attributes_strings[i], umlclass->abstract_font, umlclass->abstract_font_height);
+        umlclass->attributesbox_height += umlclass->abstract_font_height;
+      }
+      else {
+	width = font_string_width(umlclass->attributes_strings[i], umlclass->normal_font, umlclass->font_height);
+        umlclass->attributesbox_height += umlclass->font_height;
+      }
       maxwidth = MAX(width, maxwidth);
 
       i++;
@@ -560,6 +589,10 @@ umlclass_calculate_data(UMLClass *umlclass)
     }
   }
 
+  if ((umlclass->attributesbox_height<0.4) ||
+      umlclass->suppress_attributes )
+          umlclass->attributesbox_height = 0.4;
+
   /* operations box: */
   if (umlclass->operations_strings != NULL) {
     for (i=0;i<umlclass->num_operations;i++) {
@@ -569,15 +602,12 @@ umlclass_calculate_data(UMLClass *umlclass)
   }
   umlclass->num_operations = g_list_length(umlclass->operations);
 
-  umlclass->operationsbox_height = font_height * umlclass->num_operations + 2*0.1;
-  if ((umlclass->operationsbox_height<0.4) ||
-      umlclass->suppress_operations )
-      umlclass->operationsbox_height = 0.4;
-  
+  umlclass->operationsbox_height = 2*0.1;
+
   umlclass->operations_strings = NULL;
   if (umlclass->num_operations != 0) {
     umlclass->operations_strings =
-      g_malloc(sizeof(char *)*umlclass->num_operations);
+      g_malloc (sizeof (utfchar *) * umlclass->num_operations);
     i = 0;
     list = umlclass->operations;
     while (list != NULL) {
@@ -586,10 +616,13 @@ umlclass_calculate_data(UMLClass *umlclass)
       op = (UMLOperation *) list->data;
       umlclass->operations_strings[i] = uml_get_operation_string(op);
       
-      if (op->abstract)
-	width = font_string_width(umlclass->operations_strings[i], umlclass->abstract_font, font_height);
-      else
-	width = font_string_width(umlclass->operations_strings[i], umlclass->normal_font, font_height);
+      if (op->inheritance_type != UML_LEAF) {
+	width = font_string_width(umlclass->operations_strings[i], umlclass->abstract_font, umlclass->abstract_font_height);
+        umlclass->operationsbox_height += umlclass->abstract_font_height;
+      } else {
+	width = font_string_width(umlclass->operations_strings[i], umlclass->normal_font, umlclass->font_height);
+        umlclass->operationsbox_height += umlclass->font_height;
+      }
       maxwidth = MAX(width, maxwidth);
 
       i++;
@@ -599,6 +632,10 @@ umlclass_calculate_data(UMLClass *umlclass)
 
   umlclass->element.width = maxwidth + 2*0.3;
 
+  if ((umlclass->operationsbox_height<0.4) ||
+      umlclass->suppress_operations )
+          umlclass->operationsbox_height = 0.4;
+  
   umlclass->element.height = umlclass->namebox_height;
   if (umlclass->visible_attributes)
     umlclass->element.height += umlclass->attributesbox_height;
@@ -614,7 +651,7 @@ umlclass_calculate_data(UMLClass *umlclass)
   }
   umlclass->num_templates = g_list_length(umlclass->formal_params);
 
-  umlclass->templates_height = font_height * umlclass->num_templates + 2*0.1;
+  umlclass->templates_height = umlclass->font_height * umlclass->num_templates + 2*0.1;
   umlclass->templates_height = MAX(umlclass->templates_height, 1.0);
 
 
@@ -622,7 +659,7 @@ umlclass_calculate_data(UMLClass *umlclass)
   maxwidth = 2.3;
   if (umlclass->num_templates != 0) {
     umlclass->templates_strings =
-      g_malloc(sizeof(char *)*umlclass->num_templates);
+      g_malloc (sizeof (utfchar *) * umlclass->num_templates);
     i = 0;
     list = umlclass->formal_params;
     while (list != NULL) {
@@ -632,7 +669,7 @@ umlclass_calculate_data(UMLClass *umlclass)
       umlclass->templates_strings[i] = uml_get_formalparameter_string(param);
       
       width = font_string_width(umlclass->templates_strings[i],
-				umlclass->normal_font, font_height);
+				umlclass->normal_font, umlclass->font_height);
       maxwidth = MAX(width, maxwidth);
 
       i++;
@@ -645,12 +682,30 @@ umlclass_calculate_data(UMLClass *umlclass)
 static void
 fill_in_fontdata(UMLClass *umlclass)
 {
-  umlclass->font_height = 0.8;
-  umlclass->classname_font_height = 1.0;
-  umlclass->normal_font = font_getfont("Courier");
-  umlclass->abstract_font = font_getfont("Courier-Oblique");
-  umlclass->classname_font = font_getfont("Helvetica-Bold");
-  umlclass->abstract_classname_font = font_getfont("Helvetica-BoldOblique");
+        if (umlclass->normal_font == NULL) {
+                umlclass->font_height = 0.8;
+                /* choose default font name for your locale. see also font_data structure
+                   in lib/font.c. if "Courier" works for you, it would be better.  */
+                umlclass->normal_font = font_getfont(_("Courier"));
+        }
+        if (umlclass->abstract_font == NULL) {
+                umlclass->abstract_font_height = 0.8;
+                /* choose default font name for abstract for your locale. see also font_data
+                   structure in lib/font.c. */
+                umlclass->abstract_font = font_getfont(_("Courier-Oblique"));
+        }
+        if (umlclass->classname_font == NULL) {
+                umlclass->classname_font_height = 1.0;
+                /* choose default font name for class name font for your locale. see also font_data
+                   structure in lib/font.c. */
+                umlclass->classname_font = font_getfont(_("Helvetica-Bold"));
+        }
+        if (umlclass->abstract_classname_font == NULL) {
+                umlclass->abstract_classname_font_height = 1.0;
+                /* choose default font name for abstract class name for your locale. see also font_data
+                   structure in lib/font.c. */
+                umlclass->abstract_classname_font = font_getfont(_("Helvetica-BoldOblique"));
+        }
 }
 
 static Object *
@@ -679,7 +734,11 @@ umlclass_create(Point *startpoint,
   umlclass->properties_dialog = NULL;
   fill_in_fontdata(umlclass);
 
-  umlclass->name = strdup(_("Class"));
+#ifdef GTK_DOESNT_TALK_UTF8_WE_DO
+  umlclass->name = charconv_local8_to_utf8 (_("Class"));
+#else
+  umlclass->name = g_strdup (_("Class"));
+#endif
   umlclass->stereotype = NULL;
   
   umlclass->abstract = FALSE;
@@ -822,7 +881,9 @@ umlclass_copy(UMLClass *umlclass)
   element_copy(elem, newelem);
 
   newumlclass->font_height = umlclass->font_height;
+  newumlclass->abstract_font_height = umlclass->abstract_font_height;
   newumlclass->classname_font_height = umlclass->classname_font_height;
+  newumlclass->abstract_classname_font_height = umlclass->abstract_classname_font_height;
   newumlclass->normal_font = umlclass->normal_font;
   newumlclass->abstract_font = umlclass->abstract_font;
   newumlclass->classname_font = umlclass->classname_font;
@@ -973,6 +1034,22 @@ umlclass_save(UMLClass *umlclass, ObjectNode obj_node,
 		   &umlclass->color_foreground);		   
   data_add_color(new_attribute(obj_node, "background_color"), 
 		   &umlclass->color_background);		   
+  data_add_font (new_attribute (obj_node, "normal_font"),
+                 umlclass->normal_font);
+  data_add_font (new_attribute (obj_node, "abstract_font"),
+                 umlclass->abstract_font);
+  data_add_font (new_attribute (obj_node, "classname_font"),
+                 umlclass->classname_font);
+  data_add_font (new_attribute (obj_node, "abstract_classname_font"),
+                 umlclass->abstract_classname_font);
+  data_add_real (new_attribute (obj_node, "font_height"),
+                 umlclass->font_height);
+  data_add_real (new_attribute (obj_node, "abstract_font_height"),
+                 umlclass->abstract_font_height);
+  data_add_real (new_attribute (obj_node, "classname_font_height"),
+                 umlclass->classname_font_height);
+  data_add_real (new_attribute (obj_node, "abstract_classname_font_height"),
+                 umlclass->abstract_classname_font_height);
 
   /* Attribute info: */
   attr_node = new_attribute(obj_node, "attributes");
@@ -1074,7 +1151,47 @@ static Object *umlclass_load(ObjectNode obj_node, int version,
   attr_node = object_find_attribute(obj_node, "background_color");
   if(attr_node != NULL)
     data_color(attribute_first_data(attr_node), &umlclass->color_background); 
-    
+
+  umlclass->normal_font = NULL;
+  attr_node = object_find_attribute (obj_node, "normal_font");
+  if (attr_node != NULL)
+          umlclass->normal_font = data_font (attribute_first_data (attr_node));
+
+  umlclass->abstract_font = NULL;
+  attr_node = object_find_attribute (obj_node, "abstract_font");
+  if (attr_node != NULL)
+          umlclass->abstract_font = data_font (attribute_first_data (attr_node));
+
+  umlclass->classname_font = NULL;
+  attr_node = object_find_attribute (obj_node, "classname_font");
+  if (attr_node != NULL)
+          umlclass->classname_font = data_font (attribute_first_data (attr_node));
+
+  umlclass->abstract_classname_font = NULL;
+  attr_node = object_find_attribute (obj_node, "abstract_classname_font");
+  if (attr_node != NULL)
+          umlclass->abstract_classname_font = data_font (attribute_first_data (attr_node));
+
+  umlclass->font_height = 0.0;
+  attr_node = object_find_attribute (obj_node, "font_height");
+  if (attr_node != NULL)
+          umlclass->font_height = data_real (attribute_first_data (attr_node));
+
+  umlclass->abstract_font_height = 0.0;
+  attr_node = object_find_attribute (obj_node, "abstract_font_height");
+  if (attr_node != NULL)
+          umlclass->abstract_font_height = data_real (attribute_first_data (attr_node));
+
+  umlclass->classname_font_height = 0.0;
+  attr_node = object_find_attribute (obj_node, "classname_font_height");
+  if (attr_node != NULL)
+          umlclass->classname_font_height = data_real (attribute_first_data (attr_node));
+
+  umlclass->abstract_classname_font_height = 0.0;
+  attr_node = object_find_attribute (obj_node, "abstract_classname_font_height");
+  if (attr_node != NULL)
+          umlclass->abstract_classname_font_height = data_real (attribute_first_data (attr_node));
+
   /* Attribute info: */
   attr_node = object_find_attribute(obj_node, "attributes");
   num_attr = num = attribute_num_data(attr_node);
