@@ -71,10 +71,8 @@ static void bezierline_destroy(Bezierline *bezierline);
 static Object *bezierline_copy(Bezierline *bezierline);
 
 static PropDescription *bezierline_describe_props(Bezierline *bezierline);
-static void bezierline_get_props(Bezierline *bezierline,
-				 Property *props, guint nprops);
-static void bezierline_set_props(Bezierline *bezierline,
-				 Property *props, guint nprops);
+static void bezierline_get_props(Bezierline *bezierline, GPtrArray *props);
+static void bezierline_set_props(Bezierline *bezierline, GPtrArray *props);
 
 static void bezierline_save(Bezierline *bezierline, ObjectNode obj_node,
 			  const char *filename);
@@ -151,17 +149,17 @@ static PropOffset bezierline_offsets[] = {
 };
 
 static void
-bezierline_get_props(Bezierline *bezierline, Property *props, guint nprops)
+bezierline_get_props(Bezierline *bezierline, GPtrArray *props)
 {
   object_get_props_from_offsets(&bezierline->bez.object, bezierline_offsets,
-				props, nprops);
+				props);
 }
 
 static void
-bezierline_set_props(Bezierline *bezierline, Property *props, guint nprops)
+bezierline_set_props(Bezierline *bezierline, GPtrArray *props)
 {
   object_set_props_from_offsets(&bezierline->bez.object, bezierline_offsets,
-				props, nprops);
+				props);
   bezierline_update_data(bezierline);
 }
 
@@ -276,22 +274,25 @@ bezierline_create(Point *startpoint,
   Point defaultlen = { .3, .3 };
 
   /*bezierline_init_defaults();*/
-  bezierline = g_new(Bezierline, 1);
+  bezierline = g_new0(Bezierline, 1);
   bez = &bezierline->bez;
   obj = &bez->object;
   
   obj->type = &bezierline_type;
   obj->ops = &bezierline_ops;
 
-  bezierconn_init(bez);
-  
-  bez->points[0].p1 = *startpoint;
-  bez->points[1].p1 = *startpoint;
-  point_add(&bez->points[1].p1, &defaultlen);
-  bez->points[1].p2 = bez->points[1].p1;
-  point_add(&bez->points[1].p2, &defaultlen);
-  bez->points[1].p3 = bez->points[1].p2;
-  point_add(&bez->points[1].p3, &defaultlen);
+  if (user_data == NULL) {
+    bezierconn_init(bez, 2);
+    
+    bez->points[0].p1 = *startpoint;
+    bez->points[1].p1 = *startpoint;
+    point_add(&bez->points[1].p1, &defaultlen);
+    bez->points[1].p2 = bez->points[1].p1;
+    point_add(&bez->points[1].p2, &defaultlen);
+    bez->points[1].p3 = bez->points[1].p2;
+    point_add(&bez->points[1].p3, &defaultlen);
+  } else {
+  }
 
   bezierline_update_data(bezierline);
 
@@ -322,7 +323,7 @@ bezierline_copy(Bezierline *bezierline)
   
   bez = &bezierline->bez;
  
-  newbezierline = g_new(Bezierline, 1);
+  newbezierline = g_new0(Bezierline, 1);
   newbez = &newbezierline->bez;
   newobj = &bez->object;
 
@@ -344,7 +345,7 @@ bezierline_update_data(Bezierline *bezierline)
 {
   BezierConn *bez = &bezierline->bez;
   Object *obj = &bez->object;
-  BezierConnBBExtras *extra = &bez->extra_spacing;
+  PolyBBExtras *extra = &bez->extra_spacing;
 
   bezierconn_update_data(bez);
     
@@ -415,7 +416,7 @@ bezierline_load(ObjectNode obj_node, int version, const char *filename)
   Object *obj;
   AttributeNode attr;
 
-  bezierline = g_new(Bezierline, 1);
+  bezierline = g_new0(Bezierline, 1);
 
   bez = &bezierline->bez;
   obj = &bez->object;
