@@ -254,7 +254,7 @@ set_fillstyle(MyRenderer *renderer, FillStyle mode)
 }
 
 static void
-set_font(MyRenderer *renderer, Font *font, real height)
+set_font(MyRenderer *renderer, DiaFont *font, real height)
 {
   PyObject *func, *res, *arg, *self = PYDIA_RENDERER (renderer);
 
@@ -676,7 +676,7 @@ PyDia_export_data(DiagramData *data, const gchar *filename,
   }
 
   renderer = g_new(MyRenderer, 1);
-  renderer->renderer.ops = &MyRenderOps;
+  renderer->renderer.ops = MyRenderOps;
   renderer->renderer.is_interactive = 0;
   renderer->renderer.interactive_ops = NULL;
 
