@@ -29,18 +29,16 @@
 #include <unistd.h>
 #endif
 #include <locale.h>
-
 #include <entities.h>
-#if defined(LIBXML_VERSION) && LIBXML_VERSION >= 20000
-#define root children
-#define childs children
-#endif
 
 /* the dots per centimetre to render this diagram at */
 /* this matches the setting `100%' setting in dia. */
 #define DPCM 20
 
 #include <tree.h>
+#include <xmlmemory.h>
+#include <parser.h> /* xmlStrdup */
+#include "dia_xml_libxml.h"
 #include "geometry.h"
 #include "render.h"
 #include "filter.h"
@@ -82,7 +80,7 @@ static void set_linejoin(RendererShape *renderer, LineJoin mode);
 static void set_linestyle(RendererShape *renderer, LineStyle mode);
 static void set_dashlength(RendererShape *renderer, real length);
 static void set_fillstyle(RendererShape *renderer, FillStyle mode);
-static void set_font(RendererShape *renderer, Font *font, real height);
+static void set_font(RendererShape *renderer, DiaFont *font, real height);
 static void draw_line(RendererShape *renderer, 
 		      Point *start, Point *end, 
 		      Color *line_colour);
@@ -142,40 +140,45 @@ static void add_rectangle_connection_points(RendererShape *renderer,
 static void add_ellipse_connection_points(RendererShape *renderer,
       Point *center, real width, real height);      
 
-static RenderOps ShapeRenderOps = {
-  (BeginRenderFunc) begin_render,
-  (EndRenderFunc) end_render,
+static RenderOps *ShapeRenderOps;
 
-  (SetLineWidthFunc) set_linewidth,
-  (SetLineCapsFunc) set_linecaps,
-  (SetLineJoinFunc) set_linejoin,
-  (SetLineStyleFunc) set_linestyle,
-  (SetDashLengthFunc) set_dashlength,
-  (SetFillStyleFunc) set_fillstyle,
-  (SetFontFunc) set_font,
+static void
+init_shape_renderops() {
+    ShapeRenderOps = create_renderops_table();
+
+    ShapeRenderOps->begin_render = (BeginRenderFunc) begin_render;
+    ShapeRenderOps->end_render = (EndRenderFunc) end_render;
+
+    ShapeRenderOps->set_linewidth = (SetLineWidthFunc) set_linewidth;
+    ShapeRenderOps->set_linecaps = (SetLineCapsFunc) set_linecaps;
+    ShapeRenderOps->set_linejoin = (SetLineJoinFunc) set_linejoin;
+    ShapeRenderOps->set_linestyle = (SetLineStyleFunc) set_linestyle;
+    ShapeRenderOps->set_dashlength = (SetDashLengthFunc) set_dashlength;
+    ShapeRenderOps->set_fillstyle = (SetFillStyleFunc) set_fillstyle;
+    ShapeRenderOps->set_font = (SetFontFunc) set_font;
   
-  (DrawLineFunc) draw_line,
-  (DrawPolyLineFunc) draw_polyline,
+    ShapeRenderOps->draw_line = (DrawLineFunc) draw_line;
+    ShapeRenderOps->draw_polyline = (DrawPolyLineFunc) draw_polyline;
   
-  (DrawPolygonFunc) draw_polygon,
-  (FillPolygonFunc) fill_polygon,
+    ShapeRenderOps->draw_polygon = (DrawPolygonFunc) draw_polygon;
+    ShapeRenderOps->fill_polygon = (FillPolygonFunc) fill_polygon;
 
-  (DrawRectangleFunc) draw_rect,
-  (FillRectangleFunc) fill_rect,
+    ShapeRenderOps->draw_rect = (DrawRectangleFunc) draw_rect;
+    ShapeRenderOps->fill_rect = (FillRectangleFunc) fill_rect;
 
-  (DrawArcFunc) draw_arc,
-  (FillArcFunc) fill_arc,
+    ShapeRenderOps->draw_arc = (DrawArcFunc) draw_arc;
+    ShapeRenderOps->fill_arc = (FillArcFunc) fill_arc;
 
-  (DrawEllipseFunc) draw_ellipse,
-  (FillEllipseFunc) fill_ellipse,
+    ShapeRenderOps->draw_ellipse = (DrawEllipseFunc) draw_ellipse;
+    ShapeRenderOps->fill_ellipse = (FillEllipseFunc) fill_ellipse;
 
-  (DrawBezierFunc) draw_bezier,
-  (FillBezierFunc) fill_bezier,
+    ShapeRenderOps->draw_bezier = (DrawBezierFunc) draw_bezier;
+    ShapeRenderOps->fill_bezier = (FillBezierFunc) fill_bezier;
 
-  (DrawStringFunc) draw_string,
+    ShapeRenderOps->draw_string = (DrawStringFunc) draw_string;
 
-  (DrawImageFunc) draw_image,
-};
+    ShapeRenderOps->draw_image = (DrawImageFunc) draw_image;
+}
 
 static RendererShape *
 new_shape_renderer(DiagramData *data, const char *filename)
@@ -197,8 +200,11 @@ new_shape_renderer(DiagramData *data, const char *filename)
   }
   fclose(file);
 
+  if (ShapeRenderOps == NULL)
+      init_shape_renderops();
+
   renderer = g_new(RendererShape, 1);
-  renderer->renderer.ops = &ShapeRenderOps;
+  renderer->renderer.ops = ShapeRenderOps;
   renderer->renderer.is_interactive = 0;
   renderer->renderer.interactive_ops = NULL;
 
@@ -210,10 +216,20 @@ new_shape_renderer(DiagramData *data, const char *filename)
 
   /* set up the root node */
   renderer->doc = xmlNewDoc("1.0");
+  renderer->doc->encoding = xmlStrdup("UTF-8");
+#if defined(LIBXML_VERSION) && LIBXML_VERSION >= 20000
+  /*
+   * xmlNewGlobalNs is deprececated and broken (does nothing)
+   * with libxml2.4.[2..20] at least
+   */
+  renderer->root = xmlNewDocNode(renderer->doc, NULL, "shape", NULL);
+  name_space = xmlNewNs(renderer->root, "http://www.daa.com.au/~james/dia-shape-ns", NULL);
+#else
   name_space = xmlNewGlobalNs(renderer->doc, "http://www.daa.com.au/~james/dia-shape-ns", NULL);
   renderer->root = xmlNewDocNode(renderer->doc, name_space, "shape", NULL);
+#endif
   renderer->svg_name_space = xmlNewNs(renderer->root, "http://www.w3.org/2000/svg", "svg");
-  renderer->doc->root = renderer->root;
+  renderer->doc->xmlRootNode = renderer->root;
 
   dirname = g_dirname(filename);
   sheetname = g_basename(dirname);
@@ -253,12 +269,22 @@ begin_render(RendererShape *renderer, DiagramData *data)
 static void
 end_render(RendererShape *renderer)
 {
-  g_free(renderer->linestyle);
+  int old_blanks_default = pretty_formated_xml;
 
+  /* FIXME HACK: we always want nice readable shape files,
+   *  but toggling it by a global var is ugly   --hb 
+   */
+  pretty_formated_xml = TRUE;
+
+  g_free(renderer->linestyle);
+  renderer->linestyle = NULL;
+  
   xmlSetDocCompressMode(renderer->doc, 0);
-  xmlSaveFile(renderer->filename, renderer->doc);
+  xmlDiaSaveFile(renderer->filename, renderer->doc);
   g_free(renderer->filename);
+  renderer->filename = NULL;
   xmlFreeDoc(renderer->doc);
+  pretty_formated_xml = old_blanks_default;
 }
 
 static void 
@@ -390,7 +416,7 @@ set_fillstyle(RendererShape *renderer, FillStyle mode)
 }
 
 static void
-set_font(RendererShape *renderer, Font *font, real height)
+set_font(RendererShape *renderer, DiaFont *font, real height)
 {
   renderer->fontsize = height;
   /* XXXX todo */
@@ -835,16 +861,27 @@ draw_string(RendererShape *renderer,
 	    const char *text,
 	    Point *pos, Alignment alignment,
 	    Color *colour)
-{
-  CHAR *enc;
+{    
   xmlNodePtr node;
   char buf[512], *style, *tmp;
   real saved_width;
 
-  enc = xmlEncodeEntitiesReentrant(renderer->doc, text);
-  node = xmlNewChild(renderer->root, renderer->svg_name_space, "text", enc);
-  free(enc);
-
+#ifdef UNICODE_WORK_IN_PROGRESS
+  {
+      xmlChar *enc = xmlEncodeEntitiesReentrant(renderer->root->doc,text);
+      node = xmlNewChild(renderer->root, renderer->svg_name_space,
+                         "text", enc);
+  }
+#else
+ {
+     utfchar *utf = charconv_local8_to_utf8(text);
+     xmlChar *enc = xmlEncodeEntitiesReentrant(renderer->root->doc,utf);     
+     g_free(utf);
+     node = xmlNewChild(renderer->root, renderer->svg_name_space, "text", enc);
+     xmlFree(enc);
+ }
+#endif
+ 
   saved_width = renderer->linewidth;
   renderer->linewidth = 0.001;
   style = get_fill_style(renderer, colour);
@@ -906,6 +943,7 @@ export_shape(DiagramData *data, const gchar *filename,
     int i;
     gchar *point;
     gchar *png_filename = NULL;
+    gchar *sheet_filename = NULL;
     DiaExportFilter *exportfilter;
     char *old_locale;
     gfloat old_scaling;
@@ -938,6 +976,10 @@ export_shape(DiagramData *data, const gchar *filename,
       data_render(data, (Renderer *)renderer, NULL, NULL, NULL);
       destroy_shape_renderer(renderer);
     }
+
+    /* Create a sheet entry if applicable (../../sheets exists) */
+    
+
     setlocale(LC_NUMERIC, old_locale);
     g_free(png_filename);
 }
