@@ -24,24 +24,24 @@
 #include <string.h>
 #include <time.h>
 #include <math.h>
+#include <glib.h>
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
 #include <locale.h>
 
 #include <entities.h>
-#if defined(LIBXML_VERSION) && LIBXML_VERSION >= 20000
-#define root children
-#define childs children
-#endif
 
 #include <tree.h>
+#include <xmlmemory.h>
+
 #include "geometry.h"
 #include "render.h"
 #include "filter.h"
 #include "intl.h"
 #include "message.h"
 #include "diagramdata.h"
+#include "dia_xml_libxml.h"
 
 typedef struct _RendererSVG RendererSVG;
 struct _RendererSVG {
@@ -75,7 +75,7 @@ static void set_linejoin(RendererSVG *renderer, LineJoin mode);
 static void set_linestyle(RendererSVG *renderer, LineStyle mode);
 static void set_dashlength(RendererSVG *renderer, real length);
 static void set_fillstyle(RendererSVG *renderer, FillStyle mode);
-static void set_font(RendererSVG *renderer, Font *font, real height);
+static void set_font(RendererSVG *renderer, DiaFont *font, real height);
 static void draw_line(RendererSVG *renderer, 
 		      Point *start, Point *end, 
 		      Color *line_colour);
@@ -129,40 +129,47 @@ static void draw_image(RendererSVG *renderer,
 		       real width, real height,
 		       DiaImage image);
 
-static RenderOps SvgRenderOps = {
-  (BeginRenderFunc) begin_render,
-  (EndRenderFunc) end_render,
+static RenderOps *SvgRenderOps;
 
-  (SetLineWidthFunc) set_linewidth,
-  (SetLineCapsFunc) set_linecaps,
-  (SetLineJoinFunc) set_linejoin,
-  (SetLineStyleFunc) set_linestyle,
-  (SetDashLengthFunc) set_dashlength,
-  (SetFillStyleFunc) set_fillstyle,
-  (SetFontFunc) set_font,
+static void
+init_svg_renderops() 
+{
+    SvgRenderOps = create_renderops_table();
+
+    SvgRenderOps->begin_render = (BeginRenderFunc) begin_render;
+    SvgRenderOps->end_render = (EndRenderFunc) end_render;
+
+    SvgRenderOps->set_linewidth = (SetLineWidthFunc) set_linewidth;
+    SvgRenderOps->set_linecaps = (SetLineCapsFunc) set_linecaps;
+    SvgRenderOps->set_linejoin = (SetLineJoinFunc) set_linejoin;
+    SvgRenderOps->set_linestyle = (SetLineStyleFunc) set_linestyle;
+    SvgRenderOps->set_dashlength = (SetDashLengthFunc) set_dashlength;
+    SvgRenderOps->set_fillstyle = (SetFillStyleFunc) set_fillstyle;
+    SvgRenderOps->set_font = (SetFontFunc) set_font;
   
-  (DrawLineFunc) draw_line,
-  (DrawPolyLineFunc) draw_polyline,
+    SvgRenderOps->draw_line = (DrawLineFunc) draw_line;
+    SvgRenderOps->draw_polyline = (DrawPolyLineFunc) draw_polyline;
   
-  (DrawPolygonFunc) draw_polygon,
-  (FillPolygonFunc) fill_polygon,
+    SvgRenderOps->draw_polygon = (DrawPolygonFunc) draw_polygon;
+    SvgRenderOps->fill_polygon = (FillPolygonFunc) fill_polygon;
 
-  (DrawRectangleFunc) draw_rect,
-  (FillRectangleFunc) fill_rect,
+    SvgRenderOps->draw_rect = (DrawRectangleFunc) draw_rect;
+    SvgRenderOps->fill_rect = (FillRectangleFunc) fill_rect;
 
-  (DrawArcFunc) draw_arc,
-  (FillArcFunc) fill_arc,
+    SvgRenderOps->draw_arc = (DrawArcFunc) draw_arc;
+    SvgRenderOps->fill_arc = (FillArcFunc) fill_arc;
 
-  (DrawEllipseFunc) draw_ellipse,
-  (FillEllipseFunc) fill_ellipse,
+    SvgRenderOps->draw_ellipse = (DrawEllipseFunc) draw_ellipse;
+    SvgRenderOps->fill_ellipse = (FillEllipseFunc) fill_ellipse;
 
-  (DrawBezierFunc) draw_bezier,
-  (FillBezierFunc) fill_bezier,
+    SvgRenderOps->draw_bezier = (DrawBezierFunc) draw_bezier;
+    SvgRenderOps->fill_bezier = (FillBezierFunc) fill_bezier;
 
-  (DrawStringFunc) draw_string,
+    SvgRenderOps->draw_string = (DrawStringFunc) draw_string;
+
+    SvgRenderOps->draw_image = (DrawImageFunc) draw_image;
+}
 
-  (DrawImageFunc) draw_image,
-};
 
 static RendererSVG *
 new_svg_renderer(DiagramData *data, const char *filename)
@@ -182,8 +189,11 @@ new_svg_renderer(DiagramData *data, const char *filename)
   }
   fclose(file);
 
+  if (SvgRenderOps == NULL)
+    init_svg_renderops();
+
   renderer = g_new(RendererSVG, 1);
-  renderer->renderer.ops = &SvgRenderOps;
+  renderer->renderer.ops = SvgRenderOps;
   renderer->renderer.is_interactive = 0;
   renderer->renderer.interactive_ops = NULL;
 
@@ -195,12 +205,13 @@ new_svg_renderer(DiagramData *data, const char *filename)
 
   /* set up the root node */
   renderer->doc = xmlNewDoc("1.0");
+  renderer->doc->encoding = xmlStrdup("UTF-8");
   renderer->doc->standalone = FALSE;
   xmlCreateIntSubset(renderer->doc, "svg",
-		     "-//W3C//DTD SVG 20000802//EN",
-		     "http://www.w3.org/TR/2000/CR-SVG-20000802/DTD/svg-20000802.dtd");
+		     "-//W3C//DTD SVG 1.0//EN",
+		     "http://www.w3.org/TR/2001/PR-SVG-20010719/DTD/svg10.dtd");
   renderer->root = xmlNewDocNode(renderer->doc, NULL, "svg", NULL);
-  renderer->doc->root = renderer->root;
+  renderer->doc->xmlRootNode = renderer->root;
 
   /* set the extents of the SVG document */
   extent = &data->extents;
@@ -259,7 +270,7 @@ end_render(RendererSVG *renderer)
   g_free(renderer->linestyle);
 
   xmlSetDocCompressMode(renderer->doc, 0);
-  xmlSaveFile(renderer->filename, renderer->doc);
+  xmlDiaSaveFile(renderer->filename, renderer->doc);
   g_free(renderer->filename);
   xmlFreeDoc(renderer->doc);
 }
@@ -380,7 +391,7 @@ set_fillstyle(RendererSVG *renderer, FillStyle mode)
 }
 
 static void
-set_font(RendererSVG *renderer, Font *font, real height)
+set_font(RendererSVG *renderer, DiaFont *font, real height)
 {
   renderer->fontsize = height;
   /* XXXX todo */
@@ -400,7 +411,7 @@ get_draw_style(RendererSVG *renderer,
   if (!str) str = g_string_new(NULL);
   g_string_truncate(str, 0);
 
-  g_string_sprintf(str, "stroke-width: %g", renderer->linewidth);
+  g_string_sprintf(str, "fill: none; stroke-width: %g", renderer->linewidth);
   if (strcmp(renderer->linecap, "butt"))
     g_string_sprintfa(str, "; stroke-linecap: %s", renderer->linecap);
   if (strcmp(renderer->linejoin, "miter"))
@@ -557,6 +568,19 @@ fill_rect(RendererSVG *renderer,
   xmlSetProp(node, "height", buf);
 }
 
+static int sweep(real x1,real y1,real x2,real y2,real x3,real y3)
+{
+  real X2=x2-x1;
+  real Y2=y2-y1;
+  real X3=x3-x1;
+  real Y3=y3-y1;
+  real L=sqrt(X2*X2+Y2*Y2);
+  real cost=X2/L;
+  real sint=Y2/L;
+  real res=-X3*sint+Y3*cost;
+  return res>0;
+}
+
 static void
 draw_arc(RendererSVG *renderer, 
 	 Point *center,
@@ -567,16 +591,22 @@ draw_arc(RendererSVG *renderer,
   xmlNodePtr node;
   char buf[512];
   real rx = width / 2, ry = height / 2;
+  real x1=center->x + rx*cos(angle1*M_PI/180);
+  real y1=center->y - ry*sin(angle1*M_PI/180);
+  real x2=center->x + rx*cos(angle2*M_PI/180);
+  real y2=center->y - ry*sin(angle2*M_PI/180);
+  int swp = sweep(x1,y1,x2,y2,center->x,center->y);
+  int l_arc = (angle2 - angle1) > 180;
 
   node = xmlNewChild(renderer->root, NULL, "path", NULL);
   
   xmlSetProp(node, "style", get_draw_style(renderer, colour));
 
   /* this path might be incorrect ... */
-  g_snprintf(buf, sizeof(buf), "M %g,%g A %g,%g 0 %d 1 %g,%g",
-	     center->x + rx*cos(angle1), center->y + ry * sin(angle1),
-	     rx, ry, (angle2 - angle1 > 0),
-	     center->x + rx*cos(angle2), center->y + ry * sin(angle2));
+  g_snprintf(buf, sizeof(buf), "M %g,%g A %g,%g 0 %d %d %g,%g",
+	     x1, y1,
+	     rx, ry,l_arc ,swp ,
+	     x2, y2);
 
   xmlSetProp(node, "d", buf);
 }
@@ -591,16 +621,22 @@ fill_arc(RendererSVG *renderer,
   xmlNodePtr node;
   char buf[512];
   real rx = width / 2, ry = height / 2;
+  real x1=center->x + rx*cos(angle1*M_PI/180);
+  real y1=center->y - ry*sin(angle1*M_PI/180);
+  real x2=center->x + rx*cos(angle2*M_PI/180);
+  real y2=center->y - ry*sin(angle2*M_PI/180);
+  int swp = sweep(x1,y1,x2,y2,center->x,center->y);
+  int l_arc = (angle2 - angle1) > 180;
 
   node = xmlNewChild(renderer->root, NULL, "path", NULL);
   
   xmlSetProp(node, "style", get_fill_style(renderer, colour));
 
   /* this path might be incorrect ... */
-  g_snprintf(buf, sizeof(buf), "M %g,%g A %g,%g 0 %d 1 %g,%g L %g,%g z",
-	     center->x + rx*cos(angle1), center->y + ry * sin(angle1),
-	     rx, ry, (angle2 - angle1 > 0),
-	     center->x + rx*cos(angle2), center->y + ry * sin(angle2),
+  g_snprintf(buf, sizeof(buf), "M %g,%g A %g,%g 0 %d %d %g,%g L %g,%g z",
+	     x1, y1,
+	     rx, ry,l_arc ,swp ,
+	     x2, y2,
 	     center->x, center->y);
 
   xmlSetProp(node, "d", buf);
@@ -626,7 +662,7 @@ draw_ellipse(RendererSVG *renderer,
   g_snprintf(buf, sizeof(buf), "%g", width / 2);
   xmlSetProp(node, "rx", buf);
   g_snprintf(buf, sizeof(buf), "%g", height / 2);
-  xmlSetProp(node, "rx", buf);
+  xmlSetProp(node, "ry", buf);
 }
 
 static void
@@ -649,7 +685,7 @@ fill_ellipse(RendererSVG *renderer,
   g_snprintf(buf, sizeof(buf), "%g", width / 2);
   xmlSetProp(node, "rx", buf);
   g_snprintf(buf, sizeof(buf), "%g", height / 2);
-  xmlSetProp(node, "rx", buf);
+  xmlSetProp(node, "ry", buf);
 }
 
 static void
@@ -743,14 +779,23 @@ draw_string(RendererSVG *renderer,
 	    Point *pos, Alignment alignment,
 	    Color *colour)
 {
-  CHAR *enc;
   xmlNodePtr node;
   char buf[512], *style, *tmp;
   real saved_width;
 
-  enc = xmlEncodeEntitiesReentrant(renderer->doc, text);
+#ifdef UNICODE_WORK_IN_PROGRESS
+  xmlChar *enc = xmlEncodeEntitiesReentrant(renderer->root->doc, text);
   node = xmlNewChild(renderer->root, NULL, "text", enc);
-  free(enc);
+  xmlFree(enc);
+#else
+ {
+     utfchar *utf = charconv_local8_to_utf8(text);
+     xmlChar *enc = xmlEncodeEntitiesReentrant(renderer->root->doc, utf);
+     g_free(utf);
+     node = xmlNewChild(renderer->root, NULL, "text", enc);
+     xmlFree(enc);
+ }
+#endif
 
   saved_width = renderer->linewidth;
   renderer->linewidth = 0.001;
@@ -758,13 +803,13 @@ draw_string(RendererSVG *renderer,
   renderer->linewidth = saved_width;
   switch (alignment) {
   case ALIGN_LEFT:
-    style = g_strconcat(style, "; text-align: left", NULL);
+    style = g_strconcat(style, "; text-anchor: start", NULL);
     break;
   case ALIGN_CENTER:
-    style = g_strconcat(style, "; text-align: center", NULL);
+    style = g_strconcat(style, "; text-anchor: middle", NULL);
     break;
   case ALIGN_RIGHT:
-    style = g_strconcat(style, "; text-align: right", NULL);
+    style = g_strconcat(style, "; text-anchor: end", NULL);
     break;
   }
   tmp = g_strdup_printf("%s; font-size: %g", style, renderer->fontsize);
