@@ -47,8 +47,10 @@
 #include "filter.h"
 #include "object.h"
 #include "properties.h"
+#include "propinternals.h"
 #include "../app/group.h"
 
+#include "../objects/standard/create.h"
 #include "xfig.h"
 
 #define BUFLEN 512
@@ -57,6 +59,34 @@ static Color fig_colors[FIG_MAX_USER_COLORS];
 
 gboolean import_fig(const gchar *filename, DiagramData *dia, void* user_data);
 
+static char **warnings;
+
+#define WARNING_NO_POLYGONS 0
+#define WARNING_NO_PATTERNS 1
+#define WARNING_NO_TRIPLE_DOTS 2
+#define WARNING_NEGATIVE_CORNER_RADIUS 3
+#define WARNING_NO_SPLINES 4
+#define WARNING_MISSING_STD_OBJECT 5
+#define MAX_WARNING 6
+
+static void
+fig_warn(int warning) {
+    if (warnings == NULL) {
+	warnings = g_malloc(sizeof(char*)*MAX_WARNING);
+	warnings[0] = _("Polygon import is not implemented yet");
+	warnings[1] = _("Patterns are not supported by Dia");
+	warnings[2] = _("Triple-dotted lines are not supported by Dia, "
+                        "using double-dotted");
+	warnings[3] = _("Negative corner radius, negating");
+	warnings[4] = _("Spline import is not implemented yet");
+	warnings[5] = _("Can't find standard object");
+    }
+    if (warning >= MAX_WARNING) return;
+    if (warnings[warning] != NULL) {
+	message_warning(warnings[warning]);
+	warnings[warning] = NULL;
+    }
+}
 
 static int
 skip_comments(FILE *file) {
@@ -86,57 +116,79 @@ skip_comments(FILE *file) {
 }
 
 static Object *
-create_standard_text(real xpos, real ypos, char *text,
+create_standard_text(real xpos, real ypos,
 		     DiagramData *dia) {
     ObjectType *otype = object_get_type("Standard - Text");
     Object *new_obj;
     Handle *h1, *h2;
     Point point;
-    Property props[4];
+
+    if (otype == NULL){
+	fig_warn(WARNING_MISSING_STD_OBJECT);
+	return NULL;
+    }
 
     point.x = xpos;
     point.y = ypos;
 
     new_obj = otype->ops->create(&point, otype->default_user_data,
 				 &h1, &h2);
-    layer_add_object(dia->active_layer, new_obj);
-
-    props[0].name = "text";
-    props[0].type = PROP_TYPE_STRING;
-    PROP_VALUE_STRING(props[0]) = text;
-    new_obj->ops->set_props(new_obj, props, 1);
-
+    
     return new_obj;
 }
 
+static PropDescription xfig_element_prop_descs[] = {
+    { "elem_corner", PROP_TYPE_POINT },
+    { "elem_width", PROP_TYPE_REAL },
+    { "elem_height", PROP_TYPE_REAL },
+    PROP_DESC_END};
+
+static GPtrArray *make_element_props(real xpos, real ypos,
+                                     real width, real height) 
+{
+    GPtrArray *props;
+    PointProperty *pprop;
+    RealProperty *rprop;
+
+    props = prop_list_from_descs(xfig_element_prop_descs,pdtpp_true);
+    g_assert(props->len == 3);
+    
+    pprop = g_ptr_array_index(props,0);
+    pprop->point_data.x = xpos;
+    pprop->point_data.y = ypos;
+    rprop = g_ptr_array_index(props,1);
+    rprop->real_data = width;
+    rprop = g_ptr_array_index(props,2);
+    rprop->real_data = height;
+    
+    return props;
+}
+
 static Object *
 create_standard_ellipse(real xpos, real ypos, real width, real height,
 			DiagramData *dia) {
     ObjectType *otype = object_get_type("Standard - Ellipse");
     Object *new_obj;
     Handle *h1, *h2;
-    Property props[4];
+    
+    GPtrArray *props;
     Point point;
 
+    if (otype == NULL){
+	fig_warn(WARNING_MISSING_STD_OBJECT);
+	return NULL;
+    }
+
     point.x = xpos;
     point.y = ypos;
 
     new_obj = otype->ops->create(&point, otype->default_user_data,
 				 &h1, &h2);
-    layer_add_object(dia->active_layer, new_obj);
+    /*   layer_add_object(dia->active_layer, new_obj); */
   
-    props[0].name = "elem_corner";
-    props[0].type = PROP_TYPE_POINT;
-    PROP_VALUE_POINT(props[0]).x = xpos;
-    PROP_VALUE_POINT(props[0]).y = ypos;
-    props[1].name = "elem_width";
-    props[1].type = PROP_TYPE_REAL;
-    PROP_VALUE_REAL(props[1]) = width;
-    props[2].name = "elem_height";
-    props[2].type = PROP_TYPE_REAL;
-    PROP_VALUE_REAL(props[2]) = height;
-
-    new_obj->ops->set_props(new_obj, props, 3);
+    props = make_element_props(xpos,ypos,width,height);
+    new_obj->ops->set_props(new_obj, props);
+    prop_list_free(props);
 
     return new_obj;
 }
@@ -149,31 +201,83 @@ create_standard_box(real xpos, real ypos, real width, real height,
   Object *new_obj;
   Handle *h1, *h2;
   Point point;
-  Property props[4];
+  GPtrArray *props;
+
+  if (otype == NULL){
+      fig_warn(WARNING_MISSING_STD_OBJECT);
+      return NULL;
+  }
 
   point.x = xpos;
   point.y = ypos;
 
   new_obj = otype->ops->create(&point, otype->default_user_data,
 			       &h1, &h2);
-  layer_add_object(dia->active_layer, new_obj);
+  /*  layer_add_object(dia->active_layer, new_obj); */
   
-  props[0].name = "elem_corner";
-  props[0].type = PROP_TYPE_POINT;
-  PROP_VALUE_POINT(props[0]).x = xpos;
-  PROP_VALUE_POINT(props[0]).y = ypos;
-  props[1].name = "elem_width";
-  props[1].type = PROP_TYPE_REAL;
-  PROP_VALUE_REAL(props[1]) = width;
-  props[2].name = "elem_height";
-  props[2].type = PROP_TYPE_REAL;
-  PROP_VALUE_REAL(props[2]) = height;
-
-  new_obj->ops->set_props(new_obj, props, 3);
+  props = make_element_props(xpos,ypos,width,height);
+  new_obj->ops->set_props(new_obj, props);
+  prop_list_free(props);
 
   return new_obj;
 }
 
+static Object *
+create_standard_polyline(int num_points, 
+			 Point *points,
+			 DiagramData *dia) {
+    ObjectType *otype = object_get_type("Standard - PolyLine");
+    Object *new_obj;
+    Handle *h1, *h2;
+    PolylineCreateData *pcd;
+
+    if (otype == NULL){
+	fig_warn(WARNING_MISSING_STD_OBJECT);
+	return NULL;
+    }
+
+    pcd = g_new(PolylineCreateData, 1);
+    pcd->num_points = num_points;
+    pcd->points = points;
+
+    new_obj = otype->ops->create(NULL, pcd,
+				 &h1, &h2);
+
+    g_free(pcd);
+    
+    return new_obj;
+}
+
+static Object *
+create_standard_polygon(int num_points, 
+			 Point *points,
+			 DiagramData *dia) {
+    ObjectType *otype = object_get_type("Standard - Polygon");
+    Object *new_obj;
+    Handle *h1, *h2;
+    PolylineCreateData *pcd;
+
+    if (otype == NULL){
+	fig_warn(WARNING_MISSING_STD_OBJECT);
+	return NULL;
+    }
+
+    pcd->num_points = num_points;
+    pcd->points = points;
+
+    new_obj = otype->ops->create(NULL, pcd,
+				 &h1, &h2);
+
+    g_free(pcd);
+    
+    return new_obj;
+}
+
+
+static PropDescription xfig_arc_prop_descs[] = {
+    { "curve_distance", PROP_TYPE_REAL },
+    PROP_DESC_END};
+
 static Object *
 create_standard_arc(real x1, real y1, real x2, real y2,
 		    real radius, DiagramData *dia) {
@@ -181,40 +285,34 @@ create_standard_arc(real x1, real y1, real x2, real y2,
     Object *new_obj;
     Handle *h1, *h2;
     Point point;
-    Property props[4];
+    GPtrArray *props;
+
+    if (otype == NULL){
+	fig_warn(WARNING_MISSING_STD_OBJECT);
+	return NULL;
+    }
 
     point.x = x1;
     point.y = y1;
 
     new_obj = otype->ops->create(&point, otype->default_user_data,
 				 &h1, &h2);
-    layer_add_object(dia->active_layer, new_obj);
-
+    /*    layer_add_object(dia->active_layer, new_obj); */
     
-    /*
-    props[0].name = "elem_corner";
-    props[0].type = PROP_TYPE_POINT;
-    PROP_VALUE_POINT(props[0]).x = xpos;
-    PROP_VALUE_POINT(props[0]).y = ypos;
-    props[1].name = "elem_width";
-    props[1].type = PROP_TYPE_REAL;
-    PROP_VALUE_REAL(props[1]) = width;
-    props[2].name = "elem_height";
-    props[2].type = PROP_TYPE_REAL;
-    PROP_VALUE_REAL(props[2]) = height;
-    props[3].name = "image_file";
-    props[3].type = PROP_TYPE_FILE;
-    PROP_VALUE_FILE(props[3]) = file;
-    */
-    props[0].name = "curve_distance";
-    props[0].type = PROP_TYPE_REAL;
-    PROP_VALUE_REAL(props[0]) = radius;
-
-    new_obj->ops->set_props(new_obj, props, 1);
+    props = prop_list_from_descs(xfig_arc_prop_descs,pdtpp_true);
+    g_assert(props->len == 1);
+    
+    ((RealProperty *)g_ptr_array_index(props,0))->real_data = radius;
+    new_obj->ops->set_props(new_obj, props);
+    prop_list_free(props);
 
     return new_obj;
 }
 
+static PropDescription xfig_file_prop_descs[] = {
+    { "image_file", PROP_TYPE_FILE },
+    PROP_DESC_END};
+
 static Object *
 create_standard_image(real xpos, real ypos, real width, real height,
 		      char *file, DiagramData *dia) {
@@ -222,29 +320,44 @@ create_standard_image(real xpos, real ypos, real width, real height,
     Object *new_obj;
     Handle *h1, *h2;
     Point point;
-    Property props[4];
+    GPtrArray *props;
+    StringProperty *sprop;
+
+    if (otype == NULL){
+	fig_warn(WARNING_MISSING_STD_OBJECT);
+	return NULL;
+    }
 
     point.x = xpos;
     point.y = ypos;
 
     new_obj = otype->ops->create(&point, otype->default_user_data,
 				 &h1, &h2);
-    layer_add_object(dia->active_layer, new_obj);
+    /*    layer_add_object(dia->active_layer, new_obj); */
     
-    props[0].name = "elem_corner";
-    props[0].type = PROP_TYPE_POINT;
-    PROP_VALUE_POINT(props[0]).x = xpos;
-    PROP_VALUE_POINT(props[0]).y = ypos;
-    props[1].name = "elem_width";
-    props[1].type = PROP_TYPE_REAL;
-    PROP_VALUE_REAL(props[1]) = width;
-    props[2].name = "elem_height";
-    props[2].type = PROP_TYPE_REAL;
-    PROP_VALUE_REAL(props[2]) = height;
-    props[3].name = "image_file";
-    props[3].type = PROP_TYPE_FILE;
-    PROP_VALUE_FILE(props[3]) = file;
-    new_obj->ops->set_props(new_obj, props, 4);
+    props = make_element_props(xpos,ypos,width,height);
+    new_obj->ops->set_props(new_obj, props);
+    prop_list_free(props);
+
+
+    props = prop_list_from_descs(xfig_file_prop_descs,pdtpp_true);
+    g_assert(props->len == 1);    
+    sprop = g_ptr_array_index(props,0);
+    g_free(sprop->string_data);
+    sprop->string_data = g_strdup(file);
+    new_obj->ops->set_props(new_obj, props);
+    prop_list_free(props);
+
+    return new_obj;
+}
+
+static Object *
+create_standard_group(GList *items, DiagramData *dia) {
+    Object *new_obj;
+
+    new_obj = group_create((GList*)items);
+
+    /*    layer_add_object(dia->active_layer, new_obj); */
 
     return new_obj;
 }
@@ -277,12 +390,42 @@ fig_area_fill_color(int area_fill, int color_index) {
 	col.green += (0xff-col.green)*(area_fill-20)/20;
 	col.blue += (0xff-col.blue)*(area_fill-20)/20;
     } else {
-	message_warning(_("Patterns are not implemented\n"));
+	fig_warn(WARNING_NO_PATTERNS);
     }
     
     return col;
 }
 
+
+
+static PropDescription xfig_simple_prop_descs_line[] = {
+    { "line_width", PROP_TYPE_REAL },
+    { "line_colour", PROP_TYPE_COLOUR },
+    PROP_DESC_END};
+
+static LineStyle 
+fig_line_style_to_dia(int line_style) 
+{
+    switch (line_style) {
+    case 0:
+        return LINESTYLE_SOLID;
+    case 1:
+        return LINESTYLE_DASHED;
+    case 2:
+	return LINESTYLE_DOTTED;
+    case 3:
+        return LINESTYLE_DASH_DOT;
+    case 4:
+        return LINESTYLE_DASH_DOT_DOT;
+    case 5:
+        fig_warn(WARNING_NO_TRIPLE_DOTS);
+        return LINESTYLE_DASH_DOT_DOT;
+    default:
+        message_error(_("Line style %d should not appear\n"), line_style);
+        return LINESTYLE_SOLID;
+    }
+}
+
 static void
 fig_simple_properties(Object *obj,
 		      int line_style,
@@ -290,57 +433,50 @@ fig_simple_properties(Object *obj,
 		      int pen_color,
 		      int fill_color,
 		      int area_fill) {
-    Property props[5];
-    int num_props = 0;
+    GPtrArray *props = prop_list_from_descs(xfig_simple_prop_descs_line,
+                                            pdtpp_true);
+    RealProperty *rprop;
+    ColorProperty *cprop;
+
+    g_assert(props->len == 2);
+    
+    rprop = g_ptr_array_index(props,0);
+    rprop->real_data = thickness/FIG_ALT_UNIT;
+    
+    cprop = g_ptr_array_index(props,1);
+    cprop->color_data = fig_color(pen_color);
+
 
-    num_props = 0;
     if (line_style != -1) {
-	props[num_props].name = "line_style";
-	props[num_props].type = PROP_TYPE_ENUM;
-	switch (line_style) {
-	case 0:
-	    PROP_VALUE_ENUM(props[num_props]) = LINESTYLE_SOLID;
-	    break;
-	case 1:
-	    PROP_VALUE_ENUM(props[num_props]) = LINESTYLE_DASHED;
-	    break;
-	case 2:
-	    PROP_VALUE_ENUM(props[num_props]) = LINESTYLE_DOTTED;
-	    break;
-	case 3:
-	    PROP_VALUE_ENUM(props[num_props]) = LINESTYLE_DASH_DOT;
-	    break;
-	case 4:
-	    PROP_VALUE_ENUM(props[num_props]) = LINESTYLE_DASH_DOT_DOT;
-	    break;
-	case 5:
-	default:
-	    message_error(_("Line style %d not supported\n"), line_style);
-	    PROP_VALUE_ENUM(props[num_props]) = LINESTYLE_SOLID;
-	}
-	num_props++;
-    }
-    props[num_props].name = "line_width";
-    props[num_props].type = PROP_TYPE_REAL;
-    PROP_VALUE_REAL(props[num_props]) = thickness/FIG_ALT_UNIT;
-    num_props++;
-    props[num_props].name = "line_colour";
-    props[num_props].type = PROP_TYPE_COLOUR;
-    PROP_VALUE_COLOUR(props[num_props]) = fig_color(pen_color);
-    num_props++;
-    if (area_fill == -1) {
-	props[num_props].name = "show_background";
-	props[num_props].type = PROP_TYPE_BOOL;
-	PROP_VALUE_BOOL(props[num_props]) = FALSE;
-	num_props++;
-    } else {
-	props[num_props].name = "fill_colour";
-	props[num_props].type = PROP_TYPE_COLOUR;
-	PROP_VALUE_COLOUR(props[num_props]) = fig_area_fill_color(area_fill, fill_color);
-	num_props++;
+        LinestyleProperty *lsprop = 
+            (LinestyleProperty *)make_new_prop("line_style", 
+                                               PROP_TYPE_LINESTYLE,
+                                               PROP_FLAG_DONT_SAVE);
+        lsprop->dash = 1.0;
+        lsprop->style = fig_line_style_to_dia(line_style);
+
+        g_ptr_array_add(props,lsprop);
     }
 
-    obj->ops->set_props(obj, props, num_props);
+    if (area_fill == -1) {
+        BoolProperty *bprop = 
+            (BoolProperty *)make_new_prop("show_background",
+                                          PROP_TYPE_BOOL,PROP_FLAG_DONT_SAVE);
+        bprop->bool_data = FALSE;
+
+        g_ptr_array_add(props,bprop);
+    } else {
+        ColorProperty *cprop = 
+            (ColorProperty *)make_new_prop("fill_colour",
+                                           PROP_TYPE_COLOUR,
+                                           PROP_FLAG_DONT_SAVE);
+        cprop->color_data = fig_area_fill_color(area_fill, fill_color);
+
+        g_ptr_array_add(props,cprop);
+    }
+
+    obj->ops->set_props(obj, props);
+    prop_list_free(props);
 }
 
 static int
@@ -420,6 +556,14 @@ fig_read_text_line(FILE *file) {
 
 static GList *depths[1000];
 
+/* If there's something in the compound stack, we ignore the depth field,
+   as it will be determined by the group anyway */
+static GSList *compound_stack = NULL;
+/* When collection compounds, this is the *highest* depth an element is
+   found at.  Since Dia groups are of one level, we put them all at that
+   level.  Best we can do now. */
+static int compound_depth;
+
 static Object *
 fig_read_ellipse(FILE *file, DiagramData *dia) {
     int sub_type;
@@ -454,7 +598,7 @@ fig_read_ellipse(FILE *file, DiagramData *dia) {
 	       &center_x, &center_y,
 	       &radius_x, &radius_y,
 	       &start_x, &start_y,
-	       &end_x, &end_y) != 19) {
+	       &end_x, &end_y) < 19) {
 	message_error(_("Couldn't read ellipse info: %s\n"), strerror(errno));
 	return NULL;
     }
@@ -466,7 +610,7 @@ fig_read_ellipse(FILE *file, DiagramData *dia) {
 				     (2*radius_x)/FIG_UNIT,
 				     (2*radius_y)/FIG_UNIT,
 				     dia);
-
+    if (newobj == NULL) return NULL;
     fig_simple_properties(newobj, line_style, thickness,
 			  pen_color, fill_color, area_fill);
 
@@ -476,7 +620,10 @@ fig_read_ellipse(FILE *file, DiagramData *dia) {
     /* Angle -- can't rotate yet */
 
     /* Depth field */
-    depths[depth] = g_list_prepend(depths[depth], newobj);
+    if (compound_stack == NULL)
+	depths[depth] = g_list_prepend(depths[depth], newobj);
+    else
+	if (compound_depth > depth) compound_depth = depth;
 
     return newobj;
 }
@@ -499,8 +646,7 @@ fig_read_polyline(FILE *file, DiagramData *dia) {
      int backward_arrow;
      int npoints;
      Point *points;
-     Property props[5];
-     int num_props;
+     GPtrArray *props = g_ptr_array_new();
      Object *newobj = NULL;
      int flipped = 0;
      char *image_file = NULL;
@@ -548,17 +694,19 @@ fig_read_polyline(FILE *file, DiagramData *dia) {
        return NULL;
      }
      
-     num_props = 0;
      switch (sub_type) {
-     case 4:
+     case 4: {
+         RealProperty *rprop = 
+             (RealProperty *)make_new_prop("corner_radius",
+                                           PROP_TYPE_REAL,PROP_FLAG_DONT_SAVE);
 	 if (radius < 0) {
-	     message_warning("Negative corner radius %lf\n", radius);
+	     fig_warn(WARNING_NEGATIVE_CORNER_RADIUS);
+             rprop->real_data = -radius/FIG_ALT_UNIT;
 	 } else {
-	     props[0].name = "corner_radius";
-	     props[0].type = PROP_TYPE_REAL;
-	     PROP_VALUE_REAL(props[0]) = radius/FIG_ALT_UNIT;
-	     num_props++;
+             rprop->real_data = radius/FIG_ALT_UNIT;
 	 }
+         g_ptr_array_add(props,rprop);
+     }
 	 /* Notice fallthrough */
      case 2: /* box */
 	 if (points[0].x > points[2].x) {
@@ -574,24 +722,28 @@ fig_read_polyline(FILE *file, DiagramData *dia) {
 	 newobj = create_standard_box(points[0].x, points[0].y,
 				      points[2].x-points[0].x,
 				      points[2].y-points[0].y, dia);
-	 newobj->ops->set_props(newobj, props, num_props);
+	 if (newobj == NULL) goto exit;
+	 newobj->ops->set_props(newobj, props);
 	 break;
      case 5: /* imported-picture bounding-box) */
 	 newobj = create_standard_image(points[0].x, points[0].y,
 					points[2].x-points[0].x,
 					points[2].y-points[0].y,
 					image_file, dia);
+	 if (newobj == NULL) goto exit;
 	 break;
      case 1: /* polyline */
+	 newobj = create_standard_polyline(npoints, points, dia);
+	 if (newobj == NULL) goto exit;
+	 break;
      case 3: /* polygon */
 	 /*
 	   newobj = create_standard_polygon(points, dia);
 	 */
      default: 
        message_error(_("Unknown polyline subtype: %d\n"), sub_type);
-       return NULL;
+       goto exit;
      }
-     g_free(image_file);
 
      fig_simple_properties(newobj, line_style, thickness,
 			   pen_color, fill_color, area_fill);
@@ -601,8 +753,14 @@ fig_read_polyline(FILE *file, DiagramData *dia) {
      /* Cap style */
      
      /* Depth field */
-     depths[depth] = g_list_prepend(depths[depth], newobj);
-
+     if (compound_stack == NULL)
+	 depths[depth] = g_list_prepend(depths[depth], newobj);
+     else
+	 if (compound_depth > depth) compound_depth = depth;
+ exit:
+     prop_list_free(props);
+     if (image_file != NULL)
+	 g_free(image_file);
      return newobj;
 }
 
@@ -617,7 +775,6 @@ fig_read_arc(FILE *file, DiagramData *dia) {
     int pen_style;
     int area_fill;
     real style_val;
-    int join_style;
     int cap_style;
     int direction;
     int forward_arrow;
@@ -627,8 +784,6 @@ fig_read_arc(FILE *file, DiagramData *dia) {
     int x1, y1;
     int x2, y2;
     int x3, y3;
-    real ul_x, ul_y;
-    real dr_x, dr_y;
     real radius;
 
     if (fscanf(file, "%d %d %d %d %d %d %d %d %lf %d %d %d %d %lf %lf %d %d %d %d %d %d\n",
@@ -669,6 +824,7 @@ fig_read_arc(FILE *file, DiagramData *dia) {
 	newobj = create_standard_arc(x1/FIG_UNIT, y1/FIG_UNIT,
 				     x3/FIG_UNIT, y3/FIG_UNIT,
 				     radius, dia);
+	if (newobj == NULL) return NULL;
 	break;
     default: 
 	message_error(_("Unknown polyline subtype: %d\n"), sub_type);
@@ -684,15 +840,27 @@ fig_read_arc(FILE *file, DiagramData *dia) {
     /* Cap style */
      
     /* Depth field */
-    depths[depth] = g_list_prepend(depths[depth], newobj);
+    if (compound_stack == NULL)
+	depths[depth] = g_list_prepend(depths[depth], newobj);
+    else
+	if (compound_depth > depth) compound_depth = depth;
 
     return newobj;
 }
 
+static PropDescription xfig_text_descs[] = {
+   { "text", PROP_TYPE_TEXT },
+    PROP_DESC_END
+    /* Can't do the angle */
+    /* Height and length are ignored */
+    /* Flags */
+};
+
 static Object *
 fig_read_text(FILE *file, DiagramData *dia) {
-    Property props[5];
-    int num_props = 0;
+    GPtrArray *props = NULL;
+    TextProperty *tprop;
+
     Object *newobj = NULL;
     int sub_type;
     int color;
@@ -705,7 +873,7 @@ fig_read_text(FILE *file, DiagramData *dia) {
     real height;
     real length;
     int x, y;
-    char *text_buf;
+    char *text_buf = NULL;
 
     if (fscanf(file, " %d %d %d %d %d %lf %lf %d %lf %lf %d %d",
 	       &sub_type,
@@ -726,74 +894,60 @@ fig_read_text(FILE *file, DiagramData *dia) {
     /* Skip one space exactly */
     text_buf = fig_read_text_line(file);
 
-    newobj = create_standard_text(x/FIG_UNIT, y/FIG_UNIT, text_buf, dia);
-    g_free(text_buf);
+    newobj = create_standard_text(x, y, dia);
+    if (newobj == NULL) goto exit;
 
-    props[num_props].name = "text_colour";
-    props[num_props].type = PROP_TYPE_COLOUR;
-    PROP_VALUE_COLOUR(props[num_props]) = fig_color(color);
-    num_props++;
-    props[num_props].name = "text_alignment";
-    props[num_props].type = PROP_TYPE_ENUM;
-    PROP_VALUE_ENUM(props[num_props]) = sub_type;
-    num_props++;
-    props[num_props].name = "text_height";
-    props[num_props].type = PROP_TYPE_REAL;
-    PROP_VALUE_REAL(props[num_props]) = font_size*2.54/72.0;
-    num_props++;
-    /* Can't do the angle */
-    /* Height and length are ignored */
-    /* Flags */
-    props[num_props].name = "text_font";
-    props[num_props].type = PROP_TYPE_FONT;
-    PROP_VALUE_FONT(props[num_props]) = font_getfont(fig_fonts[font]);
-    num_props++;
-
-    newobj->ops->set_props(newobj, props, num_props);
+    props = prop_list_from_descs(xfig_text_descs,pdtpp_true);
+    g_assert(props->len == 4);
 
+    tprop = g_ptr_array_index(props,0);
+    tprop->text_data = g_strdup(text_buf);
+    /*g_free(text_buf); */
+    tprop->attr.alignment = sub_type;
+    tprop->attr.position.x = x/FIG_UNIT;
+    tprop->attr.position.y = y/FIG_UNIT;
+    tprop->attr.font = font_getfont(fig_fonts[font]);
+    tprop->attr.height = font_size*3.54/72.0;
+    tprop->attr.color = fig_color(color);
+    newobj->ops->set_props(newobj, props);
+    
     /* Depth field */
-    depths[depth] = g_list_prepend(depths[depth], newobj);
+    if (compound_stack == NULL)
+	depths[depth] = g_list_prepend(depths[depth], newobj);
+    else
+	if (compound_depth > depth) compound_depth = depth;
 
+ exit:
+    if (text_buf != NULL) free(text_buf);
+    if (props != NULL) prop_list_free(props);
     return newobj;
 }
 
-static int
+static gboolean
 fig_read_object(FILE *file, DiagramData *dia) {
   int objecttype;
-  static GSList *compound_stack = NULL;
   Object *item = NULL;
-  Layer *layer = dia->active_layer;
 
   if (fscanf(file, "%d ", &objecttype) != 1) {
-    if (!feof(file)) {
-      message_error(_("Couldn't identify FIG object: %s\n"), strerror(errno));
-    }
-    return FALSE;
+      if (!feof(file))
+	  message_error(_("Couldn't identify FIG object: %s\n"), strerror(errno));
+      return FALSE;
   }
 
-  
-
   switch (objecttype) {
   case -6: { /* End of compound */
-      /* Pop off compound_stack till we reach a null */
-      Object *subitem;
-      GList *items = NULL;
-
       if (compound_stack == NULL) {
 	  message_error(_("Compound end outside compound\n"));
 	  return FALSE;
       }
 
       /* Make group item with these items */
-      do {
-	  subitem = (Object *)compound_stack->data;
-	  compound_stack = g_slist_next(compound_stack);
-	  if (subitem != NULL) { /* Add object to group */
-	      items = g_list_prepend(items, subitem);
-	  }
-      } while (subitem != NULL);
-      item = group_create(items);
-      layer_add_object(layer, item);
+      item = create_standard_group((GList*)compound_stack->data, dia);
+      compound_stack = g_slist_remove(compound_stack, compound_stack->data);
+      if (compound_stack == NULL) {
+	  depths[compound_depth] = g_list_prepend(depths[compound_depth],
+						  item);
+      }
       break;
   }
   case 0: { /* Color pseudo-object. */
@@ -802,10 +956,8 @@ fig_read_object(FILE *file, DiagramData *dia) {
     Color color;
 
     if (fscanf(file, " %d #%xd", &colornumber, &colorvalues) != 2) {
-      if (!feof(file)) {
 	message_error(_("Couldn't read color: %s\n"), strerror(errno));
-      }
-      return FALSE;
+	return FALSE;
     }
 
     color.red = (colorvalues & 0x00ff0000)>>16;
@@ -817,41 +969,55 @@ fig_read_object(FILE *file, DiagramData *dia) {
   }
   case 1: { /* Ellipse which is a generalization of circle. */
       item = fig_read_ellipse(file, dia);
-      if (item == NULL) return FALSE;
+      if (item == NULL) {
+	  return FALSE;
+      }
       break;
   }
   case 2: /* Polyline which includes polygon and box. */
       item = fig_read_polyline(file, dia);
-      /*      if (item == NULL) return FALSE;*/
+      if (item == NULL) {
+	  return FALSE;
+      }
       break;
+  case 3: /* Spline which includes closed/open control/interpolated spline. */
+      fig_warn(WARNING_NO_SPLINES);
+      return FALSE;
   case 4: /* Text. */
       item = fig_read_text(file, dia);
-      /*      if (item == NULL) return FALSE;*/
+      if (item == NULL) {
+	  return FALSE;
+      }
+      break;
+  case 5: /* Arc. */
+      item = fig_read_arc(file, dia);
+      if (item == NULL) {
+	  return FALSE;
+      }
       break;
   case 6: {/* Compound object which is composed of one or more objects. */
       int dummy;
       if (fscanf(file, " %d %d %d %d\n", &dummy, &dummy, &dummy, &dummy) != 4) {
-	  if (!feof(file)) {
-	      message_error(_("Couldn't read group extend: %s\n"), strerror(errno));
-	  }
+	  message_error(_("Couldn't read group extend: %s\n"), strerror(errno));
 	  return FALSE;
       }
       /* Group extends don't really matter */
+      if (compound_stack == NULL)
+	  compound_depth = 999;
       compound_stack = g_slist_prepend(compound_stack, NULL);
+      return TRUE;
       break;
   }
-  case 5: /* Arc. */
-      item = fig_read_arc(file, dia);
-      break;
-  case 3: /* Spline which includes closed/open control/interpolated spline. */
-    message_warning(_("Object type %d not implemented yet\n"), objecttype);
-    return FALSE;
   default:
-    message_error(_("Unknown object type %d\n"), objecttype);
-    return FALSE;
+      message_error(_("Unknown object type %d\n"), objecttype);
+      return FALSE;
+      break;
+  }
+  if (compound_stack != NULL && item != NULL) { /* We're building a compound */
+      GList *compound = (GList *)compound_stack->data;
+      compound = g_list_prepend(compound, item);
+      compound_stack->data = compound;
   }
-  if (compound_stack != NULL && item != NULL) /* We're building a compound */
-    compound_stack = g_slist_prepend(compound_stack, item);
   return TRUE;
 }
 
@@ -886,7 +1052,7 @@ fig_read_paper_size(FILE *file, DiagramData *dia) {
     return TRUE;
   }
 
-  message_warning("Unknown paper size `%s', using default\n", buf);
+  message_warning(_("Unknown paper size `%s', using default\n"), buf);
   return TRUE;
 }
 
@@ -1031,32 +1197,20 @@ import_fig(const gchar *filename, DiagramData *dia, void* user_data) {
     return FALSE;
   }
   
+  compound_stack = NULL;
+
   do {
-    if (!fig_read_object(figfile, dia)) {
-      if (!feof(figfile)) {
-	  fclose(figfile);
-	  return FALSE;
-      } else {
+      if (! fig_read_object(figfile, dia)) {
 	  fclose(figfile);
 	  break;
       }
-    }
   } while (TRUE);
 
   /* Now we can reorder for the depth fields */
   for (i = 999; i >= 0; i--) {
-      GList *sorted_list;
-      GList *o;
-      
-      for (o = depths[i]; o != NULL; o = g_list_next(o)) {
-	  data_select(dia, (Object *)o->data);
-      }
-      g_list_free(depths[i]);
-      depths[i] = NULL;
-
-      sorted_list = data_get_sorted_selected_remove(dia);
-      layer_add_objects_first(dia->active_layer, sorted_list);
-      }
+      if (depths[i] != NULL)
+	  layer_add_objects_first(dia->active_layer, depths[i]);
+  }
   return TRUE;
 }
 
